package strickling.forms;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;
import java.util.Locale;
import strickling.utils.ARView;
import strickling.utils.AnglePair;
import strickling.utils.AstroUtils;
import strickling.utils.DateTimeUtils;
import strickling.utils.FormatterUtils;
import strickling.utils.OrbElem;
import strickling.utils.PlanetoidData;
import strickling.utils.SystemUtils;

import com.mhuss.AstroLib.AstroOps;
import com.mhuss.AstroLib.Latitude;
import com.mhuss.AstroLib.Longitude;
import com.mhuss.AstroLib.ObsInfo;
import com.mhuss.AstroLib.Planets;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Location;
import android.os.Build;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewConfiguration;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.view.WindowManager.LayoutParams;
import android.widget.RelativeLayout;


@SuppressLint("NewApi")
public class ARActivity_Class extends Activity implements SensorEventListener, OnClickListener  {
	
	/* Augmented reality view on separate screen.
	 * 
	 * Lock screen orientation in manifest: 
	          <activity android:name=".ARActivity"
                  android:screenOrientation="landscape"
                  android:theme="@android:style/Theme.NoTitleBar.Fullscreen"

	 * 
	 * Screen orientation will be computed on basis of accelerator sensor
	 * => canvas.rotate method keeps the coordinate system constant
	 * => coordinate system: x = azimut orientaton, y = elevation orientation 
	 * 
	 */
	
	// these constants should be overwritten or initialized with localized 
	public static boolean   blackBackground = false;           // for creating simulated screenhots
	
	public static String    TAG 			= "ARActivity";    // eg PlanetDroid.TAG;
	public static String    prefsName 		= "ARPreferences"; //eg prefsName = PlanetDroid.PREFS_NAME;
	public static boolean   azimFromNorth 	= true;
	public static double  	geomagDec 		= 0; //geomagnetic declination in radians. Will be added to sensor value. Updated in Subclass
	public Location 		obsLocation		= null;
    public static String    AZI_ELE_STRING 	= "Az. %3.0f\u00B0  El. %3.0f\u00B0";	
	public double 			tdt				= 0;
    public static double    timeZone	    = 0; //hrs
    public static OrbElem	orbElem;
    public static Locale	locale			= Locale.getDefault();
    public static int		planet			= 0;
    public static String	dateTimePattern = DateTimeUtils.DEFAULT_DATETIME_FORM;
    public static double    azimutRise		= -9;
    public static double    azimutSet		= -9;
    public static CharSequence[] planetsList;
    public static String 	ShortNorth, ShortEast, ShortSouth, ShortWest, ShortNE, ShortSE, ShortSW, ShortNW; 
    public static String 	riseStrg, setStrg, calibToast, /*guideToMsg,*/ doNotMove; 
    public static String    okStrg, cancelStrg, enterHorZStrg, calibDialogStrg; 
    
    // these values my be overwritten
    public static boolean   drawEcliptic	= true; 
    public static boolean   drawCross		= false; 
//    public static boolean   usePlanetSymbols= false; 
    
	
	
	public  static 			String ACC_CALIB_FILE_NAME;
	public  static 	final String ACC_CALIB_FILE_NAME0 = "accels.txt";
	
	public SensorManager 	sensorManager;
	public RelativeLayout 	rl;
	public DrawView 		dv;
	
	public static double 	aziRad 				= 0;
	public static double 	eleRad 				= 0;
	public static double 	shiftFactor			= 5;
	public double 		 	oriRad 				= 0;
    public static long 	 	calibReminderStart 	= 0; 
    
	public ARView 			arView;
	// in arrays: 0..MAX_PLANET-1 Sun and Planets, MAX_PLANET = Moon, MAX_PLANET+1 = Selected Object, if not Sun, Moon or bright Planet
	protected static final int MAX_PLANET  	= Planets.URANUS;
	protected static final int OBJ_MOON		= MAX_PLANET-1;
	public static double [] objectAziRad 	= new double [MAX_PLANET+1];  //radians
	public static double [] objectElevRad	= new double [MAX_PLANET+1];  //radians
	public static String [] objectName      = new String [MAX_PLANET+1];
	
	public static final int ECLIPTIC_SIZE  = 36*4;
	static double [] 		eclAzi			= new double [ECLIPTIC_SIZE]; // radians
	static double [] 		eclEle			= new double [ECLIPTIC_SIZE]; // radians			             
    public static final int MAX_SUM_SIZE = 200; // for desire > 40, for Galaxy < 10
    public static int 		SUM_SIZE 		= 10; // for desire > 40, for Galaxy < 10
    
    static final int 		CALIB_CONFIRM = 0;
    static final int 		ENTER_HORZ    = 1;
    
    static int 				TEXT_SIZE;

	
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		ACC_CALIB_FILE_NAME = SystemUtils.writeableDir +ACC_CALIB_FILE_NAME0;  // 762 locs, from Geonames.org

		rl = new RelativeLayout (this);// findViewById(R.id.RL);
		setContentView (rl);
		sensorManager = (SensorManager) this.getSystemService(Context.SENSOR_SERVICE);
		dv = new DrawView (this);
        TEXT_SIZE = (int) (22 * getBaseContext().getResources().getDisplayMetrics().density);//30;

	    loadAccelCalib ();
        SharedPreferences settings = getSharedPreferences(prefsName, 0);
	    SUM_SIZE 					= settings.getInt  		("arSUM_SIZE", 	SUM_SIZE);
	    shiftFactor 				= settings.getFloat		("shiftFactor", (float) shiftFactor);
//	    ARView.autoViewAngle		= settings.getBoolean	("getViewAngle", Build.VERSION.SDK_INT >=8);//remove in Ver automatic only for SDK >= 8 (Andr. 2.2)
	    ARView.autoViewAngle		= settings.getBoolean	("autoViewAngle", settings.getBoolean	("getViewAngle", Build.VERSION.SDK_INT >=8));// automatic only for SDK >= 8 (Andr. 2.2)
	    ARView.horizontalViewAngle	= settings.getFloat		("horizontalViewAngle", (float) ARView.horizontalViewAngle);
	    Log.d(TAG, String.format("AR_Activity_Class.onCreate, read FoV %1.0f°, Auto: "  +ARView.autoViewAngle, ARView.horizontalViewAngle));
	}

	/** Register for the updates when Activity is in foreground */
	@SuppressLint("NewApi")
	@Override
	protected void onResume() {
		super.onResume();

		Log.d(TAG, "ARActivity_Class.onResume");
//	    Log.d(TAG, "arSUM_SIZE: " +SUM_SIZE);

	    calcPlanets();
		arView = new ARView (this); // <3>
		if (! blackBackground) rl.addView(arView); // <4>
		rl.addView(dv);
		if ((Build.VERSION.SDK_INT >= 14 && ! ViewConfiguration.get(this).hasPermanentMenuKey())) {
			dv.setOnClickListener(this);
		}

 		sensorManager.registerListener(this, sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER),  SensorManager.SENSOR_DELAY_NORMAL);
 		sensorManager.registerListener(this, sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD), SensorManager.SENSOR_DELAY_NORMAL);// SensorManager.SENSOR_DELAY_NORMAL);
// 		sensorManager.registerListener(this, sensorManager.getDefaultSensor(Sensor.TYPE_GYROSCOPE), 	 SensorManager.SENSOR_DELAY_NORMAL);// SensorManager.SENSOR_DELAY_NORMAL);
	}

	/** Stop the updates when Activity is paused */
	@Override
	protected void onPause() {
		super.onPause();
//		stopTimer ();
		sensorManager.unregisterListener(this);
		if (arView != null) {
			rl.removeView(arView);
			arView.destroy ();
		}
		if (dv     != null) rl.removeView(dv);
		Log.d(TAG, String.format("AR_Activity_Class.onPause, save FoV %1.0f °, Auto: "  +ARView.autoViewAngle, ARView.horizontalViewAngle));
		// Save settings
	    SharedPreferences.Editor editor = getSharedPreferences(prefsName, 0).edit();
	    editor.putInt    ("arSUM_SIZE", 	SUM_SIZE);
	    editor.putFloat  ("shiftFactor", (float) shiftFactor);
	    editor.putFloat  ("horizontalViewAngle", (float) ARView.horizontalViewAngle);
	    editor.putBoolean("getViewAngle",   ARView.autoViewAngle); // Delete this line in Ver. 5.0
	    editor.putBoolean("autoViewAngle",  ARView.autoViewAngle);
//	    editor.remove("getViewAngle"); // Delete this line in Ver. 5.0
	    editor.commit();
	}	
	
	public void calcPlanets (){ // called by onResume() or if necessary by timer event
		// Calculate Ecliptic 
		try { if (drawEcliptic) {
			double sternzeit = AstroUtils.sternzeit(tdt, obsLocation.getLongitude()) // Here NullPointer Exception?!?
			*Math.PI/12.;
			double obliquity = AstroOps.meanObliquity( AstroOps.toMillenia( tdt ) ); 
			for (int i = 0; i < ECLIPTIC_SIZE; i ++) {
				double [] lonLat = new double [2];
				double [] raDe  = new double [2];
				lonLat [0] = i * Math.PI*2/ECLIPTIC_SIZE;
				lonLat [1] = 0;
				AstroUtils.eclipticalToEquatorial(lonLat, obliquity, raDe);
				raDe [0] = sternzeit - raDe [0];
				double [] aziEle = new double [2];
				AstroUtils.equatorialToAzimutal(raDe, 
						Math.toRadians(obsLocation.getLatitude()), aziEle);
				eclAzi [i] = aziEle [0] + Math.PI;
				eclEle [i] = aziEle [1];
			}	
		} // if (drawEcliptic) 
		
   		PlanetoidData pde;
   		ObsInfo obsInfo = new ObsInfo (
   				new Latitude (obsLocation.getLatitude()), 
   				new Longitude (obsLocation.getLongitude()), 
   				(int) Math.round(timeZone));
//   		tdt = PlanetDroid.tdt;
//   		double deltaT = 0;
   		
//   		CharSequence[] planetsList = getResources().getTextArray(R.array.planets_array);

   		// Calculate planets, order: SUN, MERCURY...SATURN, LUNA, selected Object
   		for (int i = Planets.SUN; i <= MAX_PLANET; i ++) {
   			int planetNr = i;
   			if (i >= Planets.EARTH) {planetNr = i+1;}
   			if (i == OBJ_MOON) planetNr = Planets.LUNA;
   			if (i == MAX_PLANET  ) planetNr = planet; // = selected Object
   			pde = new PlanetoidData (planetNr, orbElem, tdt, 60/86400. /*PlanetDroid.deltaT*/, obsInfo, false);
   			objectAziRad  [i] = Math.PI-pde.getAltAzLon();
   			objectElevRad [i] = pde.getAltAzLat();

   			if (planetNr == Planets.SUN) {objectName [i] = (String) planetsList [0];}
   			else if (planetNr == Planets.PLANETOID) {objectName [i] = orbElem.getName();}
   			else {
   				if (planetNr == Planets.LUNA) {objectName [i] = (String) planetsList [1];}
   				else objectName [i] = (String) planetsList [planetNr < Planets.EARTH ? planetNr+1 : planetNr];
   			}   		}
   		} catch (Exception e) { e.printStackTrace();}
	}
	
	// Spare method to override
	public void drawRectExtension (Canvas canvas, Paint paint){		
	}
	
	// Spare method to override
	public void drawInclinedExtension (Canvas canvas, Paint paint){		
	}
	
	// Spare method to override
	public void drawPlanets (Canvas canvas, Paint paint){
    	for (int planetPtr = 0; planetPtr <= MAX_PLANET; planetPtr ++) { 
    		try {
    		// for performance reason do not draw objects like sun, moon and bright planets twice
    		if (planetPtr < MAX_PLANET && objectName[planetPtr].equals(objectName[MAX_PLANET])) planetPtr ++;
    		drawObject (canvas, paint, planetPtr);
    		} catch (Exception e) {e.printStackTrace();}
    	}		
	}
	
    /** Calculates XY Pixel coordinates for objectAziRad with global variable aziRad, eleRad
     * 
     * @param canvas
     * @param objectAziRad
     * @param objectEleRad
     * @param xyPixels return
     * @return true, if object raw with screen dimensions
     */
	public boolean getXY (Canvas canvas, double objectAziRad, double objectEleRad, double [] xyPixels) {
		AnglePair dObj = AstroUtils.angularSeparation(aziRad, eleRad, 
				objectAziRad, objectEleRad); // Distance and position from center angle in radians
		double r = calcR (dObj.lon, canvas.getWidth(), ARView.horizontalViewAngle); // in pixels	        	
		xyPixels [0] =  (cenX + r * Math.sin(dObj.lat)); 
		xyPixels [1] =  (cenY - r * Math.cos(dObj.lat)); 
		return (dObj.lon < Math.toRadians(ARView.horizontalViewAngle)*0.8);//(Math.abs(r) < canvas.getWidth()*0.8); // object within screen?
    }   
	
	/** draws an object form objectAziRad [], objectElevRad [] list
	 * 
	 * @param canvas
	 * @param paint
	 * @param planetPtr
	 */
	public void drawObject (Canvas canvas, Paint paint, int planetPtr){
    	int squareSize = BAR_SIZE/2;
		AnglePair dObj = AstroUtils.angularSeparation(aziRad, eleRad, 
				objectAziRad [planetPtr], objectElevRad [planetPtr]); // angular distance from image center in rad	        		
		double r = calcR (dObj.lon, canvas.getWidth(), ARView.horizontalViewAngle); // in pixels	        	
		if (r < canvas.getWidth()*0.8) { // Check whether object is near frame
			if (dObj.lon < Math.toRadians(ARView.horizontalViewAngle)) 
			{ // Draw Object
				int objCenX = (int) (cenX + r * Math.sin(dObj.lat)); // -Math.toRadians(screenOrientation)));
				int objCenY = (int) (cenY - r * Math.cos(dObj.lat)); // -Math.toRadians(screenOrientation)));
				paint.setStrokeWidth((float) 1);
				if (drawCross) {
					// draw cross
					paint.setStrokeWidth((float) 1.5);
					canvas.drawLine(objCenX -squareSize, objCenY, objCenX +squareSize, objCenY, paint);
					canvas.drawLine(objCenX, objCenY -squareSize, objCenX, objCenY +squareSize, paint);
    				paint.setStrokeWidth((float) 1);
				} else {
					// Draw diagonal square
					canvas.drawLine(objCenX -squareSize, objCenY, objCenX , objCenY +squareSize, paint);
					canvas.drawLine(objCenX , objCenY +squareSize, objCenX +squareSize, objCenY, paint);
					canvas.drawLine(objCenX +squareSize, objCenY, objCenX , objCenY -squareSize, paint);
					canvas.drawLine(objCenX , objCenY -squareSize, objCenX -squareSize, objCenY, paint);
				}
				paint.setTextAlign(Paint.Align.LEFT);
				canvas.drawText(objectName [planetPtr], objCenX +BAR_SIZE/2, objCenY +paint.getTextSize()/2, paint);	        			
			}			
		}
		
		//draw object directionArrow for planetPtr == MAX_PLANET
		double posAng = dObj.lat;
		if (planetPtr == MAX_PLANET) {
			paint.setStrokeWidth((float) 2);
			// Elevation Priority: avoid arrow pointing up if object has lower elevation and vice versa
			double dir = AstroUtils.latAngle(objectElevRad [planetPtr] - eleRad); // Direction up or down
			if ((dir > 0 && Math.cos(posAng) < 0) || (dir < 0 && Math.cos(posAng) > 0)) {
				posAng = Math.PI *(1 -Math.signum(Math.sin(posAng))/2.); 
				//Log.d(TAG, String.format("Correct arrow PA: %.0f°, sElev: %.0f", Math.toDegrees(posAng), Math.signum(Math.sin (eleRad))));
				}

			double arrowSize = BAR_SIZE * 2;
			if (dObj.lon < Math.toRadians(ARView.horizontalViewAngle)) 
			{   
				if (r < arrowSize *2) arrowSize = r/2;
			}
			float arrTipX = (float) (cenX + arrowSize * Math.sin(posAng));
			float arrTipY = (float) (cenY - arrowSize * Math.cos(posAng));
			canvas.drawLine(cenX, cenY, arrTipX, arrTipY, paint);
			canvas.drawLine(arrTipX, arrTipY, 
					(float) (arrTipX + arrowSize * Math.sin(posAng-Math.PI*0.8)*0.4), 
					(float) (arrTipY - arrowSize * Math.cos(posAng-Math.PI*0.8)*0.4), 
					paint);	        				
			canvas.drawLine(arrTipX, arrTipY, 
					(float) (arrTipX + arrowSize * Math.sin(posAng+Math.PI*0.8)*0.4), 
					(float) (arrTipY - arrowSize * Math.cos(posAng+Math.PI*0.8)*0.4), 
					paint);	    
		}        	       			
	}
	
	protected boolean isDrawing = false;
	public static int BAR_SIZE;
	public static int cenX;
	public static int cenY;
	protected static float yt;  
	
	public class DrawView extends View {
	        public DrawView(Context context) {
	            super(context);
	        }

	        @Override
	        public void onDraw(Canvas canvas) { //TODO Method for drawing the content
	        	super.onDraw(canvas);
	        	if (isDrawing) {
	        		Log.d(TAG, "onDraw refused, busy");
	        		return;
	        	}
	        	isDrawing = true;
	        	BAR_SIZE = canvas.getHeight() /10;
	        	cenX	   = canvas.getWidth()  / 2;
	        	cenY	   = canvas.getHeight() / 2;
	        		        	
	        	Paint paint = new Paint();
	        	paint.setStyle(Paint.Style.FILL);
	        	paint.setAntiAlias(true);
	        	paint.setColor(Color.RED);
	        	paint.setTextSize(TEXT_SIZE);
	        	paint.setTextAlign(Paint.Align.CENTER);
	    	        	
	        	// rotate canvas in 90 deg grid according to screen orientation
	        	float rot = (float) (90 *Math.round(Math.toDegrees(AstroUtils.longAngle(-oriRad))/90.));
	        	canvas.rotate (rot, cenX, cenY);
	        	if (rot == 90. || rot == 270.){
	        		yt = (float) (cenY+cenX-paint.getTextSize()*1.2); // in portrait canvas rotation, y = 0 is somewhat below upper margin!!
	        	} else  yt = canvas.getHeight()-paint.getTextSize();
	        
	        	// Print calibration reminder for the first 10 seconds after first start
	        	if (calibReminderStart == 0) calibReminderStart = System.currentTimeMillis();
	        	if (System.currentTimeMillis() -calibReminderStart < 10000){
	        		int x = cenX, y = (int) (cenY +BAR_SIZE +paint.getTextSize());
	        		for(String line: calibToast.split("\n")){
	        		      canvas.drawText(line, x, y, paint);
	        		      y += paint.getTextSize();
	        		}
	        	}
	        	
	        	// Print azimuth and elevation
	        	canvas.drawText(String.format(AZI_ELE_STRING, 
	        			Math.toDegrees(azimFromNorth ? aziRad : AstroUtils.latAngle(aziRad-Math.PI)), Math.toDegrees(eleRad)),
	        			cenX, yt, paint);
	        	canvas.drawText(FormatterUtils.zeitStrg(tdt, timeZone, dateTimePattern, locale),
	        			cenX, yt+paint.getTextSize(), paint);	
        		drawRectExtension (canvas, paint);
        		//canvas.drawText(String.format("FoV: %.0f° x %.0f°", ARView.horizontalViewAngle, ARView.verticalViewAngle), cenX, yt-2*paint.getTextSize(), paint);

	        	// Set rotation vector to real screen inclination
	        	rot = (float) -Math.toDegrees(oriRad) -rot;  // this will rotate the whole screen output counter screen orientation angle
	        	canvas.rotate (rot, cenX, cenY); // adds to previous roation

	        	//draw central cross
	        	paint.setColor(Color.RED);
	        	paint.setStrokeWidth((float) 2);
	        	canvas.drawLine(cenX -BAR_SIZE, cenY, cenX +BAR_SIZE, cenY, paint);
	        	canvas.drawLine(cenX, cenY -BAR_SIZE, cenX, cenY +BAR_SIZE, paint);
	        	
	        	// Draw horizon:
	        	paint.setColor(0xFF009000); // draw horizon in dark green
	        	paint.setStrokeWidth((float) 4);
        		double r = calcR (eleRad, canvas.getWidth(), ARView.horizontalViewAngle); // in pixels
        		int xMax = (int) (0.8*canvas.getWidth());
        		
        		// Check whether horizon is roughly inside image frame
        		if (Math.abs(r) < xMax) {	  
	        		int ym    = (int) (cenY +r); //(int) (r * Math.cos(soRad)) +cenY;
	        		// Draw Horizon
	        		canvas.drawLine(-xMax +cenX, ym,   xMax +cenX, ym, paint);
	        		
	        		// Draw Compass marks
	        		paint.setStrokeWidth((float) 2);
	        		for (int i = (int) (Math.round((Math.toDegrees(aziRad) -ARView.horizontalViewAngle/1.5)/10)*10);
	        			i <= (int) (Math.toDegrees(aziRad) +ARView.horizontalViewAngle/1.5);
	        			i+=10) {
	        				// Draw 10 deg marks
	        				int x = (int) calcR (Math.toRadians(i) -aziRad, canvas.getWidth(), ARView.horizontalViewAngle);
	    	        		paint.setStrokeWidth((float) 2);
	        				canvas.drawLine(x +cenX, ym -BAR_SIZE*(float)(0.3), x +cenX, ym +BAR_SIZE*(float)(0.3), paint);
	    	        		paint.setStrokeWidth((float) 1);
	        				canvas.drawText(String.format("%1.0f°",
	        						azimFromNorth ? AstroUtils.realMod(i,360): AstroUtils.realMod(i,360) -180), 
	        						x +cenX, (float) (ym +BAR_SIZE *0.3 +paint.getTextSize()), paint);
	        				paint.setTextSize(TEXT_SIZE *2);
	        				if (i == 0 || i == 360) canvas.drawText(ShortNorth, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				if (i == 90) canvas.drawText(ShortEast, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				if (i == 180) canvas.drawText(ShortSouth, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				if (i == 270) canvas.drawText(ShortWest, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				
	        				// Draw 5 DegMarks
	    	        		x = (int) calcR (Math.toRadians(i+5) -aziRad, canvas.getWidth(), ARView.horizontalViewAngle);
	        				canvas.drawLine(x +cenX, ym -BAR_SIZE*(float)(0.3), x +cenX, ym +BAR_SIZE*(float)(0.3), paint);
	        				if (i == 40) canvas.drawText(ShortNE, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				if (i == 130) canvas.drawText(ShortSE, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				if (i == 220) canvas.drawText(ShortSW, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				if (i == 310) canvas.drawText(ShortNW, x +cenX, 
	        						(float) (ym -BAR_SIZE *0.3), paint);
	        				paint.setTextSize((float) (TEXT_SIZE));
	        			}
	        	}
        		
        		// draw ecliptic
    			paint.setStrokeWidth((float) 1);
    			paint.setStyle(Paint.Style.FILL_AND_STROKE);
        		if (drawEcliptic) {
        			paint.setColor(Color.BLUE);
        			for (int i = 0; i < ECLIPTIC_SIZE; i ++) {
        				// search fitting segment
        				if (Math.toDegrees(Math.abs(AstroUtils.latAngle (eclAzi [i] -aziRad))) < ARView.horizontalViewAngle) {
        					AnglePair ecl = AstroUtils.angularSeparation (aziRad, eleRad, eclAzi [i], eclEle [i]);
        					r = calcR (ecl.lon, canvas.getWidth(), ARView.horizontalViewAngle);
        					int x1 = (int) (cenX + r * Math.sin(ecl.lat));
        					int y1 = (int) (cenY - r * Math.cos(ecl.lat));
        					ecl = AstroUtils.angularSeparation (aziRad, eleRad, eclAzi [(i+1)%ECLIPTIC_SIZE], eclEle [(i+1)%ECLIPTIC_SIZE]);
        					r = calcR (ecl.lon, canvas.getWidth(), ARView.horizontalViewAngle);
        					int x2 = (int) (cenX + r * Math.sin(ecl.lat));
        					int y2 = (int) (cenY - r * Math.cos(ecl.lat));
        					canvas.drawLine(x1, y1, x2, y2, paint);
        				}	        			
        			}
        		}

	        	// draw objects
            	paint.setColor(Color.YELLOW);
        		drawPlanets (canvas, paint);
	        	
    			//draw Rise arrow
        		paint.setTextAlign(Paint.Align.CENTER);
    			paint.setTextSize((float) (TEXT_SIZE*0.7));
    			if (azimutRise > -9){
    				double xy [] = new double [2];
    				if (getXY (canvas, azimutRise, 0, xy)){
    					paint.setStrokeWidth(2);
    					drawArrow (canvas, xy[0], xy[1] +BAR_SIZE/1.5, xy[0], xy[1] -BAR_SIZE/1.5, 
    							NO_TIP, ARROW_TIP, BAR_SIZE*0.3, paint);
    					paint.setStrokeWidth(0);
    					canvas.drawText(riseStrg, (float) xy[0], (float) (xy[1] -BAR_SIZE/1.5 -TEXT_SIZE*0.1), paint);
    				}
    			}
    			
    			//draw Set arrow
    			if (azimutSet > -9){
    				double xy [] = new double [2];
    				if (getXY (canvas, azimutSet, 0, xy)){
    					paint.setStrokeWidth(2);
    					drawArrow (canvas, xy[0], xy[1] -BAR_SIZE/1.5, xy[0], xy[1] +BAR_SIZE/1.5, 
    							NO_TIP, ARROW_TIP, BAR_SIZE*0.3, paint);
    					paint.setStrokeWidth(0);
    					canvas.drawText(setStrg, (float) xy[0], (float) (xy[1] +BAR_SIZE/1.5  +TEXT_SIZE), paint);

//    					AnglePair setPoint = AstroUtils.angularSeparation (aziRad, eleRad, azimutSet, 0);
//    				r = calcR (setPoint.lon, canvas.getWidth(), ARView.horizontalViewAngle); // in pixels
//    				if (setPoint.lon < Math.toRadians(ARView.horizontalViewAngle)) { // Check whether object is near frame
//    					paint.setStrokeWidth(2);
//    					float arrTipX = (float) (cenX + r * Math.sin(setPoint.lat));
//    					float arrTipY = (float) (cenY - r * Math.cos(setPoint.lat) + BAR_SIZE/1.5);
//    					canvas.drawLine(arrTipX, (float) (cenY - r * Math.cos(setPoint.lat) - BAR_SIZE/1.5), arrTipX, arrTipY, paint);
//    					canvas.drawLine(arrTipX, arrTipY, 
//    							(float) (arrTipX + BAR_SIZE*0.3), 
//    							(float) (arrTipY - BAR_SIZE*0.3), 
//    							paint);	        				
//    					canvas.drawLine(arrTipX, arrTipY, 
//    							(float) (arrTipX - BAR_SIZE*0.3), 
//    							(float) (arrTipY - BAR_SIZE*0.3), 
//    							paint);	    
//    					paint.setStrokeWidth((float)0);
//    					canvas.drawText(setStrg, arrTipX, arrTipY +TEXT_SIZE, paint);
    				}
    			}	       
        		drawInclinedExtension (canvas, paint);
	        	isDrawing = false;
	        }

	        
	} // DrawView

	public static final int NO_TIP  = 0;
	public static final int BAR_TIP = 1;
	public static final int ARROW_TIP = 2;
	//public static final int FILLED_ARROW_TIP = 3;
	public static final int CIRCLE_TIP = 4;
	
	protected void drawArrow (Canvas canvas, double xStart, double yStart, double xEnd, double yEnd, int tipStart, int tipEnd, double tipSize, Paint paint){
		double dx = xEnd -xStart;
		double dy = yEnd -yStart;
		double angle = Math.atan2(dy, dx);
		double r = Math.sqrt(dx*dx +dy*dy)*2;
		double r2 = r; // avoid painting Lines within two overlapping circles
		
		// draws circle tips
		if (tipStart == CIRCLE_TIP) {
			canvas.drawCircle((float) xStart, (float) yStart, (float) (tipSize/2), paint);
			xStart += tipSize/2*Math.cos(angle);
			yStart += tipSize/2*Math.sin(angle);
			r2 -= tipSize/2;
		}
		
		if (tipEnd   == CIRCLE_TIP) {
			canvas.drawCircle((float) xEnd,   (float) yEnd,   (float) (tipSize/2), paint);
			xEnd -= tipSize/2*Math.cos(angle);
			yEnd -= tipSize/2*Math.sin(angle);
			r2 -= tipSize/2;
		}
		
		if (r2 > 0)	canvas.drawLine((float) xStart, (float) yStart, (float) xEnd, (float) yEnd, paint);
		
		// Draw Bars
		if ((tipStart == BAR_TIP || tipEnd == BAR_TIP) && Math.abs(dx)+Math.abs(dy) > 0){
			if (tipStart == BAR_TIP)
				canvas.drawLine((float) (xStart +dy*tipSize/r), (float) (yStart -dx*tipSize/r), 
						(float) (xStart -dy*tipSize/r), (float) (yStart +dx*tipSize/r), paint);			
			if (tipEnd == BAR_TIP)
				canvas.drawLine((float) (xEnd +dy*tipSize/r), (float) (yEnd -dx*tipSize/r), 
						(float) (xEnd -dy*tipSize/r), (float) (yEnd +dx*tipSize/r), paint);			
		}
		
		// Draw Arrow
		if ((tipStart == ARROW_TIP || tipEnd == ARROW_TIP) && Math.abs(dx)+Math.abs(dy) > 0){
			if (tipStart == ARROW_TIP) {
				canvas.drawLine((float) xStart, (float) yStart,
						(float) (xStart +tipSize*Math.cos(angle-Math.PI/4)), 
						(float) (yStart  +tipSize*Math.sin(angle-Math.PI/4)),
						paint);
				canvas.drawLine((float) xStart, (float) yStart,
						(float) (xStart  +tipSize*Math.cos(angle+Math.PI/4)), 
						(float) (yStart  +tipSize*Math.sin(angle+Math.PI/4)),
						paint);
				}
			if (tipEnd == ARROW_TIP) {
				canvas.drawLine((float) xEnd, (float) yEnd,
						(float) (xEnd -tipSize*Math.cos(angle-Math.PI/4)), 
						(float) (yEnd -tipSize*Math.sin(angle-Math.PI/4)),
						paint);
				canvas.drawLine((float) xEnd, (float) yEnd,
						(float) (xEnd  -tipSize*Math.cos(angle+Math.PI/4)), 
						(float) (yEnd  -tipSize*Math.sin(angle+Math.PI/4)),
						paint);
				}
		}
		
	}

	
	/** calculates radius in pixels for an angular separation of angleRad
	 * 
	 * @param angleRad  radians
	 * @param imgWidth  pixels
	 * @param horizontalViewAngle degrees
	 * @return
	 */
    public static double calcR (double angleRad, int imgWidth, double horizontalViewAngle) {
		return Math.tan(angleRad) * imgWidth
		/(2.*Math.tan(Math.toRadians(horizontalViewAngle/2.))); // in pixels
	}
  
    
    // Sensor handling methods code below
	
	public static volatile float direction = (float) 0;
	//public static volatile float inclination;
	public static volatile float rollingZ = (float)0;
	public static volatile float kFilteringFactor = (float)0.05;
	public static float 		 aboveOrBelow = (float)0;
	public static final double 	 LIMIT_ANGLE = Math.toRadians(0.5);  // minimal change for forcing redraw
	
    private static final int 	matrix_size 	= 16;
    private float[] 		 	mags;
    private float[] 			accels;
    //private float[] 		 	gyros;
    float[] 					R1     	  = new float[matrix_size];
    float[] 					outR 	  = new float[matrix_size];
    float[] 					I 		  = new float[matrix_size];
    float[] 					values 	  = new float[3];
    
	double[] 					xArr      = new double [MAX_SUM_SIZE];
	double[] 					yArr      = new double [MAX_SUM_SIZE];
	double[] 					incArr 	  = new double [MAX_SUM_SIZE];
	double[] 					accArr0   = new double [MAX_SUM_SIZE];
	double[] 					accArr1   = new double [MAX_SUM_SIZE];
	int      					arrPtr	  = -1;
    private double  			oldAzimuth;
    private double  			oldElevation;
    //private boolean calibratingMeasureArray = true;
    
    final int 					maxTiltCorr = 30;
    float [] []					tiltCorr  = new float [maxTiltCorr] [];
    int 						tiltCorrPtr = -1;
    
    
    // Calibration parameters
    double  					sumAccX, sumAccY, sumAccZ; // values for accelerometer bias 
    int     					nAcc = -1;  // number of values, if set to zero or greater zero do new calibration!
    long    					calibStart   = 0;    // Calibration start time millis
    final long 					CALIB_MILLIS = 4000; // Calibration duration time

    // from https://gist.github.com/322555, pith = phi, roll = theta
    public static double magnetometer_readings_to_tilt_compensated_heading(double bx, double by, double bz, double phi, double theta) {
        // Takes in raw magnetometer values, pitch and roll and turns it into a tilt-compensated heading value ranging from -pi to pi (everything in this function should be in radians). """
        //variation = 4.528986*(pi/180) // magnetic variation for Corpus Christi, should match your bx/by/bz and where they were measured from (a lookup table is beyond the scope of this gist)
        double Xh = bx * Math.cos(theta) + by * Math.sin(phi) * Math.sin(theta) + bz * Math.cos(phi) * Math.sin(theta);
        double Yh = by * Math.cos(phi)   - bz * Math.sin(phi);
        return AstroUtils.longAngle(Math.atan2(-Yh, Xh));
    }
        
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
//		Log.d(TAG, "Sensor: " +sensor.getName() +"  Accuracy level: " +accuracy);
		// accuracy 3 = High, 2 = medium, 1= low
	}    
    
    // from http://code.google.com/p/staaar/source/browse/trunk/src/ssi/staaar/ARActivity.java?r=368
	public void onSensorChanged(SensorEvent event) { //TODO

		// caution! Galaxy S Plus and some other devices return always unreliable!
//		if (event.accuracy == SensorManager.SENSOR_STATUS_UNRELIABLE){
//			Log.d(TAG, "   sensor unreliable: " +event.sensor.getName() + ", device: " +android.os.Build.MODEL);
//			if (!android.os.Build.MODEL.equals("GT-I9001")){ 
//				return;
//			}
//		}

		switch (event.sensor.getType()) {
//		case Sensor.TYPE_GYROSCOPE:
//			gyros = event.values.clone();  // units: rad/s
////			Log.d(TAG, String.format("Gyro: x= %5.2f, y= %5.2f, z= %5.2f", event.values [0], event.values [1], event.values [2]));
//			break;
		case Sensor.TYPE_MAGNETIC_FIELD:
			mags = event.values.clone();
			//Log.d(TAG, String.format("Mag:  x= %5.2f, y= %5.2f, z= %5.2f", event.values [0], event.values [1], event.values [2]));
			
//			// apply tilt correction
//			if (tiltCorrPtr < 0 ){
//				for (int i = 0; i<maxTiltCorr; i++){
//					tiltCorr[i] = mags.clone();	
//				}
//			}
//			tiltCorrPtr ++; 
//			Log.d(TAG, "Mag TiltCorr "+tiltCorrPtr);
//			if (tiltCorrPtr >= maxTiltCorr) tiltCorrPtr = 0;
//			tiltCorr[tiltCorrPtr] = mags.clone();
//			float sum0, sum1, sum2;
//			sum0 = sum1 = sum2 = 0;
//			for (int i = 0; i<maxTiltCorr; i++){
//				sum0 +=tiltCorr[i] [0];	
//				sum1 +=tiltCorr[i] [1];	
//				sum2 +=tiltCorr[i] [2];	
//			}
//			mags [0] +=sum0/maxTiltCorr;
//			mags [1] +=sum1/maxTiltCorr;
//			mags [2] +=sum2/maxTiltCorr;			
			break;
		case Sensor.TYPE_ACCELEROMETER:
			accels = event.values.clone();
			
			// new Accelerometer calibration
			if (nAcc >= 0) { 
				sumAccX +=accels [0]; // accelerometer x in  m/s^2
				sumAccY +=accels [1]; 
				sumAccZ +=accels [2]; 
				nAcc ++; 
				Log.d(TAG, String.format("Acc:  x= %5.2f, y= %5.2f, z= %5.2f", event.values [0], event.values [1], event.values [2]));
				//Log.d(TAG, String.format("Calibration: n = %d, xMean = %5.2f", nAcc, sumAccX/nAcc));
				
				// ? Calibration in process?
				if (System.currentTimeMillis()-calibStart > CALIB_MILLIS) {
					if (Math.abs(sumAccX/nAcc) > 8){ // This is a landscape orientation calibration
						sumAccZ /= nAcc;	
						sumAccY /= nAcc;	
						sumAccX = sumAccX/nAcc -Math.signum(sumAccX)*SensorManager.GRAVITY_EARTH;	            				
					}
					if (Math.abs(sumAccY/nAcc) > 8){ // This is a portrait mode calibration
						sumAccX /= nAcc;	
						sumAccZ /= nAcc;	
						sumAccY = sumAccY/nAcc -Math.signum(sumAccY)*SensorManager.GRAVITY_EARTH;	            				
					}
					if (Math.abs(sumAccZ/nAcc) > 8){ // This is a horizontal calibration
						sumAccX /= nAcc;	
						sumAccY /= nAcc;	
						sumAccZ = sumAccZ/nAcc -Math.signum(sumAccZ)*SensorManager.GRAVITY_EARTH;	            				
					}
					Log.d(TAG, String.format(Locale.US,"Calibration complete: n= %d, x= %5.2f, y= %5.2f, z= %5.2f",
							nAcc, sumAccX, sumAccY, sumAccZ));
					nAcc =-1; // set n to negative value => calibration finished
					saveAccelCalib ();
				}
			}
			
			// add bias calibration values to accelerometer values 
			accels [0] +=sumAccX/nAcc;
			accels [1] +=sumAccY/nAcc;
			accels [2] +=sumAccZ/nAcc;
			
			
			// apply tilt correction
//			if (tiltCorrPtr < 0 ){
//				for (int i = 0; i<maxTiltCorr; i++){
//					tiltCorr[i] = accels.clone();	
//				}
//			}
//			tiltCorrPtr ++; 
//			Log.d(TAG, "Acc TiltCorr "+tiltCorrPtr);
//			if (tiltCorrPtr >= maxTiltCorr) tiltCorrPtr = 0;
//			tiltCorr[tiltCorrPtr] = accels.clone();
//			float sum0, sum1, sum2;
//			sum0 = sum1 = sum2 = 0;
//			for (int i = 0; i<maxTiltCorr; i++){
//				sum0 +=tiltCorr[i] [0];	
//				sum1 +=tiltCorr[i] [1];	
//				sum2 +=tiltCorr[i] [2];	
//			}
//			accels [0] +=sum0/maxTiltCorr;
//			accels [1] +=sum1/maxTiltCorr;
//			accels [2] +=sum2/maxTiltCorr;			
			break;
		}
		
		//****************  do orientation calculation  **********************
		if (mags != null && accels != null) {

			//Log.d(TAG, String.format("Accels: %8.2f  %8.2f  %8.2f", accels [0], accels [1], accels [2]));
			SensorManager.getRotationMatrix(R1, I, accels, mags);
			SensorManager.remapCoordinateSystem(R1, SensorManager.AXIS_X,
					SensorManager.AXIS_Z, outR);
			SensorManager.getOrientation(outR, values); // values [0] Azimuth, -values [1] = elevation
//			values [0] = (float) magnetometer_readings_to_tilt_compensated_heading(
//					mags[0], mags[1], mags[2],
//					Math.atan2(accels [2], accels [0]), Math.atan2(accels [2], accels [1]));
			
			// averaging sensor jitter
			// fill array with same values at first call
			if (arrPtr < 0) {				
				for (int i = 0; i < SUM_SIZE; i++) {
					xArr[i]      =  Math.cos(values[0]);	// Split into x and y to avoid averaging over -PI -> +PI
					yArr[i]      =  Math.sin(values[0]);	
					incArr[i] 	 = -values[1];
					accArr0  [i] =  accels[0];
					accArr1  [i] =  accels[1];
				}
			}

			// inc array pointer or reset to zero
			if (arrPtr < SUM_SIZE -1) {arrPtr ++; } else {
				arrPtr = 0;
			}
			
			
			// apply compass tilt compensation
			//if (Math.abs(-values[1] -oldElevation) > 0.005) 
				values[0] += (-values[1] -oldElevation) *shiftFactor *Math.sin(values[0]); 
			
			// save values for averaging in Arrays			
			xArr      [arrPtr] =  Math.cos(values[0]); // values[0] range -PI ... +PI
			yArr      [arrPtr] =  Math.sin(values[0]); // values[0] range -PI ... +PI
			incArr    [arrPtr] = -values[1];
			accArr0   [arrPtr] =  accels[0];
			accArr1   [arrPtr] =  accels[1];

			//Log.d(TAG, "Array ptr " + arrPtr); // +String.format("  Orient: %.0f°", Math.toDegrees(rawOriArr [arrPtr])));

			// calculate mean values to smoothen values
			double sumX    = 0.; 
			double sumY    = 0.;
			double sumInc  = 0.;  
			double sumOri0 = 0.;
			double sumOri1 = 0.;

			for (int i = 0; i < SUM_SIZE; i++) {
				sumX    += xArr[i];	
				sumY    += yArr[i];
				sumInc  += incArr[i];
				sumOri0 += accArr0[i];
				sumOri1 += accArr1[i];
			}

			aziRad =  AstroUtils.longAngle(Math.atan2 (sumY/SUM_SIZE, sumX/SUM_SIZE) +geomagDec);  // Correct to true north
			eleRad =  sumInc/SUM_SIZE; 
			oriRad =  Math.atan2 (sumOri1/SUM_SIZE, sumOri0/SUM_SIZE)					
					+(getWindowManager().getDefaultDisplay().getRotation()-1)*Math.PI/2.;  
			// Orientation = Rotation is 0 for Phones Portrait, 0 for Tablets Landscape, 
			//  Turn Left: Roation = 1, Turn right: Rotation = 3
//					+(getWindowManager().getDefaultDisplay().getOrientation()-1)*Math.PI/2.;  // Orientation will be 1 for Phones, 0 for Tablets
			
			if (Math.abs(aziRad - oldAzimuth) > LIMIT_ANGLE || Math.abs(eleRad - oldElevation) > LIMIT_ANGLE) {
//				Log.d(TAG, String.format(Locale.US,"dElevation:  %.5f", eleRad -oldElevation));
//				if (Math.abs(eleRad -oldElevation) > 0.005) aziRad -= (eleRad -oldElevation) *shiftFactor; 
				oldAzimuth = aziRad;
				oldElevation = eleRad ;
				dv.invalidate();
			} 
		}
	}

	// save accelerator calibration values
	public void saveAccelCalib (){
		SharedPreferences.Editor editor = getSharedPreferences(prefsName, 0).edit();
		editor.putFloat("AccelsX", (float) (-sumAccX/nAcc));
		editor.putFloat("AccelsY", (float) (-sumAccY/nAcc));
		editor.putFloat("AccelsZ", (float) (-sumAccZ/nAcc));
		editor.commit();
		SystemUtils.saveString(
				String.format(Locale.US, "Accelerator bias [m/s^2]\n%.5f\n%.5f\n%.5f",
						-sumAccX/nAcc, -sumAccY/nAcc, -sumAccZ/nAcc),
						ACC_CALIB_FILE_NAME);
	}

	//load accelarator calibration values
	public void loadAccelCalib () {
		nAcc = -1;
		// read from preferences
		SharedPreferences prefs = getSharedPreferences(prefsName, 0);
		prefs.getFloat ("AccelsX", 0);
		prefs.getFloat ("AccelsY", 0);
		prefs.getFloat ("AccelsZ", 0);

		// overwrite with saved values in file
		if (SystemUtils.fileExists(ACC_CALIB_FILE_NAME)) {
			try{
				BufferedReader in = new BufferedReader(new FileReader(ACC_CALIB_FILE_NAME));
				in.readLine();  // drop first line (Headline)
				try{
					sumAccX = Double.parseDouble(in.readLine());
					sumAccY = Double.parseDouble(in.readLine());
					sumAccZ = Double.parseDouble(in.readLine());
				} catch (Exception e) {e.printStackTrace();}
				in.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				Log.d(TAG, "ReadAccelCalib: FileNotFound Exception: " +ACC_CALIB_FILE_NAME);
			} catch (IOException e) {
				e.printStackTrace();
				Log.d(TAG, "ReadAccelCalib: IO Exception");
			}
		}
		Log.d(TAG, String.format(Locale.US,"Calibration loaded: n= %d, x= %5.2f, y= %5.2f, z= %5.2f",
				nAcc, sumAccX, sumAccY, sumAccZ));
	}


	public static final int Menu_Settings 		= 1;
	public static final int Menu_CalibPortrait = 2;


	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case Menu_Settings:
			Intent intent = new Intent(this, ARSettings.class);
			startActivity (intent); 
			return true;

		case Menu_CalibPortrait:			
    		AlertDialog.Builder builder = new AlertDialog.Builder(this);
    		builder.setMessage(calibDialogStrg)
    		.setCancelable(true)
    		.setPositiveButton(okStrg, new DialogInterface.OnClickListener() {
    			public void onClick(DialogInterface dialog, int id) {
    				doCalibPortrait ();
    			}
    		})
    		.setNegativeButton(cancelStrg, new DialogInterface.OnClickListener() {
        			public void onClick(DialogInterface dialog, int id) {
        				dialog.cancel();
        				//Log.d(TAG, "NO clicked");
        			}
        		});

    		AlertDialog dialog = builder.create(); 
    		dialog.show();	    
//    		DisplayMetrics metrics = getResources().getDisplayMetrics();
//    		int width = metrics.widthPixels;
//    		int height = metrics.heightPixels;
//    		dialog.getWindow().setLayout(width, height);
			return true;
		}
		return false;
	}

	private ProgressDialog progressBar;

	void doCalibPortrait () {

		progressBar = new ProgressDialog(this);

		//progressBar.setCancelable(true);
		progressBar.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
		progressBar.setMessage(doNotMove);
		progressBar.setProgress(0);
		//		progressBar.setMax(100);
		progressBar.show();
//		progressBar.getWindow().setLayout(500, 350); // call after .show!

		new Thread(new Runnable() {
			public void run() {
				// start the calibration
				sumAccX = sumAccY = sumAccZ = 0;
				nAcc = 0; // start the calibration with n >= 0 (Calibration is done by onSensorChanged)
				calibStart = System.currentTimeMillis();
				while (nAcc >= 0) {
					try {
						Thread.sleep(100);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					progressBar.setProgress((int) ((System.currentTimeMillis()-calibStart) /(CALIB_MILLIS/100)));
				}
				progressBar.dismiss(); 
			}
		}).start();    	
	}

    static String dateLine = "";
    // see http://developer.android.com/resources/samples/ApiDemos/src/com/example/android/apis/app/AlertDialogSamples.html
//    protected Dialog onCreateDialog(int id) { // will be called only once, on first call!!
//    	Log.d(TAG, "onCreateDialog");
//    	Dialog dialog = null;
//    	switch(id) {
//     	case CALIB_CONFIRM: 
//    		AlertDialog.Builder builder = new AlertDialog.Builder(this);
//    		builder.setMessage(calibDialogStrg)
//    		.setCancelable(true)
//    		.setPositiveButton(okStrg, new DialogInterface.OnClickListener() {
//    			public void onClick(DialogInterface dialog, int id) {
//    				doCalibPortrait ();
//    			}
//    		})
//    		.setNegativeButton(cancelStrg, new DialogInterface.OnClickListener() {
//        			public void onClick(DialogInterface dialog, int id) {
//        				dialog.cancel();
//        				//Log.d(TAG, "NO clicked");
//        			}
//        		});
//
//    		dialog = builder.create();        	
//    		return dialog;   
//    	
//        default:
//            dialog = null;
//        }
//        return dialog;
//    }
	
    public static boolean sensorsAvailable (Context c) {
    	SensorManager mSensorManager;
    	mSensorManager = (SensorManager) c.getSystemService(Context.SENSOR_SERVICE);
    	List<Sensor> ac = mSensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
    	List<Sensor> ma = mSensorManager.getSensorList(Sensor.TYPE_MAGNETIC_FIELD);
    	return (ac.size()> 0 && ma.size() > 0);
    }
    
	public void onClick(View v) {
		// On Click listener for Menu Button
		openOptionsMenu();  // call ARSettings.java
	}
}
