package strickling.forms;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.util.Locale;
import strickling.utils.ARView;
import strickling.utils.AstroUtils;
import strickling.utils.SystemUtils;
import android.app.Activity;
import android.content.res.Configuration;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.text.method.DigitsKeyListener;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;

public class ARSettings extends Activity {
	
	public static String TAG = "ARSettings"; 
	
	// variables to be initialized
	public static String settings_name, Settings_ARAverage, Settings_ARAvComm, Settings_Ecliptic; 
	public static String Settings_ViewAngle, Settings_GetViewAngle, shiftFactorTxt; 
	
	EditText sum_SizeEdit, viewAngleEdit, shiftFactorEdit;
	CheckBox eclipticChk, viewAngleChk;
	TextView viewAngleTxt;
	
    @Override
    public void onCreate(Bundle bundle) {
		super.onCreate(bundle);
//		if (//! PlanetDroid.currentLanguage.equals("0")	&& ! AstroUtils.locale.getLanguage().equals(getString(R.string.language))) {
		if(ARActivity_Class.locale != Locale.getDefault()) {
//			Log.d(TAG, "New Locale: " +locale.getLanguage());
			Configuration config = new Configuration();
			config.locale = ARActivity_Class.locale;
			getBaseContext().getResources().updateConfiguration(
					config, getBaseContext().getResources().getDisplayMetrics());
		}

		setTitle (settings_name);  // Sometimes no language update?!?
		// construct views and layout from Java code without XML
		LinearLayout ll1 = new LinearLayout(this);
		ll1.setOrientation(LinearLayout.VERTICAL);
		setContentView(ll1);
		
		ScrollView sv = new ScrollView (this);
		ll1.addView (sv);
		
		LinearLayout ll = new LinearLayout(this);
		ll.setOrientation(LinearLayout.VERTICAL);
		sv.addView (ll);
		
		// Ecliptic checkBox
		eclipticChk = new CheckBox (this);
		eclipticChk.setText(Settings_Ecliptic);
		eclipticChk.setChecked(ARActivity_Class.drawEcliptic);
//		eclipticChk.setTextColor(Color.WHITE);
//		eclipticChk.setTextSize (TypedValue.COMPLEX_UNIT_PX, 24);
		ll.addView (eclipticChk);
		addHrLine(ll);
		
		
		// SUM_SIZE
		LinearLayout ll2 = new LinearLayout(this);
		ll2.setOrientation(LinearLayout.HORIZONTAL);
		ll.addView (ll2);
				
		TextView SUM_SIZETxt = new TextView (this);
		SUM_SIZETxt.setText(Settings_ARAverage +"  ");
		SUM_SIZETxt.setTextColor(Color.WHITE);
//		SUM_SIZETxt.setTextSize (TypedValue.COMPLEX_UNIT_SP, 14);
		ll2.addView (SUM_SIZETxt);
		
		sum_SizeEdit = new EditText (this);
		// set numeric format, see http://androidblogger.blogspot.com/2009/01/numeric-edittext-and-edittext-with-max.html
		DigitsKeyListener MyDigitKeyListener =
			new DigitsKeyListener(false, false); // first true : is signed, second one : is decimal 
		sum_SizeEdit.setKeyListener( MyDigitKeyListener );
		sum_SizeEdit.setText("" + ARActivity_Class.SUM_SIZE);
		ll2.addView (sum_SizeEdit);
		
		TextView SUM_SIZETxt2 = new TextView (this);
		SUM_SIZETxt2.setText(Settings_ARAvComm);
		ll.addView (SUM_SIZETxt2);
		
		addHrLine(ll);
		
//	    Log.d(PlanetDroid.TAG, "version " +Build.VERSION.RELEASE + "   "
//	    		+Build.VERSION.SDK_INT);//
//	    		//+AstroUtils.firstVal(Build.VERSION.RELEASE));//+tm.getDeviceSoftwareVersion());

		viewAngleChk = new CheckBox (this);
		viewAngleChk.setText(Settings_GetViewAngle);
		if (Build.VERSION.SDK_INT < 8) {
			ARView.autoViewAngle = false; // available for SDK 8 and higher
			viewAngleChk.setVisibility(View.GONE); // Hide for older versions - Input is always manual
		}
		Log.d(TAG, "ARSettings.onCreate: Chk visibility: " +(viewAngleChk.getVisibility()== View.VISIBLE) +", auto: " +ARView.autoViewAngle);
		viewAngleChk.setChecked(ARView.autoViewAngle);
		ll.addView (viewAngleChk);			

		LinearLayout ll3 = new LinearLayout(this);
		ll3.setOrientation(LinearLayout.HORIZONTAL);
		ll.addView (ll3);

		viewAngleTxt = new TextView (this);
		viewAngleTxt.setText(Settings_ViewAngle);
		viewAngleTxt.setTextColor(Color.WHITE);
//		viewAngleTxt.setTextSize (TypedValue.COMPLEX_UNIT_SP, 14);
		ll3.addView (viewAngleTxt);

		viewAngleEdit = new EditText (this);
		// set numeric format, see http://androidblogger.blogspot.com/2009/01/numeric-edittext-and-edittext-with-max.html
		viewAngleEdit.setKeyListener( MyDigitKeyListener );
		viewAngleEdit.setText("" +(int) (ARView.horizontalViewAngle));
		ll3.addView (viewAngleEdit);
		Log.d (TAG, "Automatic: " +viewAngleChk.isChecked());
		if (viewAngleChk.isChecked()/*&& Build.VERSION.SDK_INT > 8*/ && viewAngleChk.getVisibility()== View.VISIBLE) { // automatic: do not ask for focal length
			viewAngleTxt. setVisibility(View.GONE);
			viewAngleEdit.setVisibility(View.GONE);
		}			
		viewAngleChk.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				if (viewAngleChk.isChecked()) { 
					viewAngleTxt. setVisibility(View.GONE);
					viewAngleEdit.setVisibility(View.GONE);
				}	else { // ask for focal length
					viewAngleTxt. setVisibility(View.VISIBLE);
					viewAngleEdit.setVisibility(View.VISIBLE);					
				}
			}			
		});
		addHrLine(ll);	    	


//		Shift Factor
//		LinearLayout ll4 = new LinearLayout(this);
//		ll4.setOrientation(LinearLayout.HORIZONTAL);
//		ll.addView (ll4);
//
//		TextView shiftFactorTxtV = new TextView (this);
//		shiftFactorTxtV.setText(shiftFactorTxt);
//		shiftFactorTxtV.setTextColor(Color.WHITE);
//		shiftFactorTxtV.setTextSize (TypedValue.COMPLEX_UNIT_PX, 24);
//		ll4.addView (shiftFactorTxtV);
//				
//		
//		shiftFactorEdit = new EditText (this);
//		DigitsKeyListener SignedDigitKeyListener =
//			new DigitsKeyListener(true, false); // first true : is signed, second one : is decimal 
//		shiftFactorEdit.setKeyListener( SignedDigitKeyListener );
//		shiftFactorEdit.setText("" +(int) (ARActivity_Class.shiftFactor));
//		ll4.addView (shiftFactorEdit);
		
//		addHrLine(ll);
		
        // Set blue actionBar with icon, replaces android:theme="@style/ThemeDialogWhenLarge" in manifest activity
        SystemUtils.configureActionBar (this); 		
		
		
    }
    
	protected void addHrLine(LinearLayout linLay) {
		View hrLine1 = new View (this);
		hrLine1.setBackgroundColor(Color.GRAY);
		hrLine1.setLayoutParams(new LayoutParams (getWindowManager().getDefaultDisplay().getWidth(), 1));
		linLay.addView(hrLine1);
	}
	
    
    protected void onPause() {
       	super.onPause();
//		Log.d(TAG, "ARSettings onPause");
       	int SUM_SIZE = (int) AstroUtils.getNumeric(sum_SizeEdit);
       	if (SUM_SIZE <= 0) SUM_SIZE = 1;
       	if (SUM_SIZE >= ARActivity_Class.MAX_SUM_SIZE) SUM_SIZE = ARActivity_Class.MAX_SUM_SIZE;
       	ARActivity_Class.SUM_SIZE = SUM_SIZE;
//       	ARActivity_Class.shiftFactor = AstroUtils.getNumeric(shiftFactorEdit);
       	if (viewAngleEdit != null && !viewAngleChk.isChecked()) {
       		ARView.horizontalViewAngle = AstroUtils.getNumeric (viewAngleEdit);
       		if (ARView.horizontalViewAngle < 4) ARView.horizontalViewAngle = 10;
       		if (ARView.horizontalViewAngle > 150) ARView.horizontalViewAngle = 80;
       	}
       	ARView.autoViewAngle = viewAngleChk.isChecked();
       	ARActivity_Class.drawEcliptic = eclipticChk.isChecked();
       	Log.d(TAG, String.format("ARSettings.onPause, set FoV %1.0f deg, Auto: "  +ARView.autoViewAngle, ARView.horizontalViewAngle));
    }
    

}
