package strickling.forms;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 
import org.osmdroid.bonuspack.overlays.InfoWindow;
import org.osmdroid.bonuspack.overlays.Marker;
import org.osmdroid.compatibility.BmpPolyline;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapView;
import strickling.utils.OsmTools;
import com.strickling.forms.R;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

public class ClickableInfoWindow extends InfoWindow {
	
	public static   String   TAG = "ClickableInfoWindow";
	public GeoPoint position = null;
	public Marker	marker	 = null;
	public boolean  oneInstance = true;
	public boolean  noButton = false;
	public boolean  noOffset = false; // if true, set offset to  (0, 0), marker tip is at marker position
	public String 	degreeFormat = "d5"; // example: "dms0", "dm1", "d4" (last number is decimals) 
	public Handler  mHandler = null;

	public ClickableInfoWindow (int layoutResId, MapView mapView) {
		super(layoutResId, mapView);  
	}
	
	public ClickableInfoWindow (int layoutResId, MapView mapView, Handler onClick) {
		super(layoutResId, mapView); 
		mHandler = onClick;
	}
	
	public void onClose() {
		marker   = null;
		position = null;
	}

	@Override
	public void open(Object object, GeoPoint position, int offsetX, int offsetY) {
		if (noOffset)  { super.open(object, position, 0, 0); }
		else super.open(object, position, offsetX, offsetY);
	}
	
	public void onOpen(Object arg0) {
		if (oneInstance) InfoWindow.closeAllInfoWindowsOn(mMapView);		
		BmpPolyline polyline = null;
		Button btnMoreInfo = (Button) mView.findViewById(R.id.bubble_moreinfo);
		try { 
			marker   = (Marker) arg0; 
			btnMoreInfo.setVisibility(noButton ? View.GONE : View.VISIBLE);
		} catch (Exception e) {
			polyline = (BmpPolyline) arg0;
			btnMoreInfo.setVisibility(View.GONE); 
			}
		if (polyline == null) {
			position = marker.getPosition();
//			Log.d(AstroFlight.TAG,"ClickableInfoWindow.onOpen " +pos2String(position));
		}
		
		//		      Button btnMoreInfo = (Button) mView.findViewById(R.id.bubble_moreinfo); 
		try {
			TextView txtTitle = (TextView) mView.findViewById(R.id.bubble_title);
			TextView txtDescription = (TextView) mView.findViewById(R.id.bubble_description);
			TextView txtSubdescription = (TextView) mView.findViewById(R.id.bubble_subdescription);

			txtTitle.setText (polyline == null ? ((Marker) arg0).getTitle() : ((BmpPolyline) arg0).getTitle());
			String s = (polyline == null ? ((Marker) arg0).getSnippet() : ((BmpPolyline) arg0).getSnippet());
			if ((s == null || s.equals("")) && polyline == null && position != null) 
				s = OsmTools.pos2String(position, degreeFormat);
			//		Log.d (TAG, "Snippet: " +s);
			txtDescription.setText(s);
			txtSubdescription.setText(polyline == null ? (((Marker) arg0).getSubDescription()) : ((BmpPolyline) arg0).getSubDescription());
		} catch (Exception e)  {e.printStackTrace(); 
		}

        // https://mobiledevstories.wordpress.com/2014/03/01/osmdroid-bonus-pack-markers-with-clickable-infowindows/
		LinearLayout layout = (LinearLayout) mView.findViewById(R.id.bubble_layout); // in Bonuspack_bubble.xml
		if (layout !=  null) layout.setOnClickListener(new OnClickListener() {  // replaced by onClick, which fires on ButtonClick fixed in the layout.xml
			public void onClick(View v) {
				if (mHandler != null) {
					Message msg = new Message ();
					msg.obj = marker;
					mHandler.sendMessage(msg);
				}
				Log.d(TAG, "InfoWin.onClick ");
			}
		}); 
	} 

}