package strickling.forms;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Resources;
import android.os.Build;
import android.os.Handler;
import android.os.Message;
import android.renderscript.Sampler;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Toast;

import com.strickling.forms.R;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import strickling.utils.FormatterUtils;


public class LanguageSelector {

    public int selection;
    public String selectedLanguageCode;
    protected CharSequence[] languages, orgLanguages, shortLanguages;
    protected int [] languageMapper;
    protected final Activity activity;


    public LanguageSelector(final Activity mActivity, final String newLanguage, final String[] availableLanguages, final Handler handler) {
        activity = mActivity;
        selectedLanguageCode = newLanguage;

        // build list of available languages
        Resources res = mActivity.getResources();
        languages      = res.getTextArray(R.array.languages);
        orgLanguages   = res.getTextArray(R.array.orgLanguages); // Languages in their own language
        shortLanguages = res.getTextArray(R.array.shortLanguages);
        languageMapper = new int [availableLanguages.length];

        String [] languageList = new String [availableLanguages.length];

        // Build language list
        for (int i = 0; i< availableLanguages.length; i++ ) {
            if (availableLanguages [i].equals(newLanguage)) selection = i;
            if (availableLanguages [i].equals("ar") && Build.VERSION.SDK_INT < 14) // lock Arabic in Android less then 14
            {
                languageList[i] = "en";
            }
            languageList[i] = FormatterUtils.LTR +getLanguageLine(i, availableLanguages);
//            Log.d ("PlanetDroid", "Search for " +newLanguage);
//            Log.d ("PlanetDroid", "" +i +"  " +shortLanguages [i]);
//            Log.d ("PlanetDroid", "selection " +selection +" ");
        }

        // set Auto-system if newLanguage = Device language
        if (Locale.getDefault().getLanguage().equals(newLanguage)
             && shortLanguages [0].equals("0")) selection = 0;


        // build and display dialog
        AlertDialog.Builder alt_bld = new AlertDialog.Builder(mActivity);
        alt_bld.setTitle(mActivity.getString(R.string.selectLanguage));

        alt_bld.setSingleChoiceItems(languageList, selection, new DialogInterface
                .OnClickListener() {

            // on Click Listener
            public void onClick(DialogInterface dialog, int item) {
                // handle "0" and "ar" for Build.VERSION.SDK_INT < 14
                selectedLanguageCode = availableLanguages[item];
                if (selectedLanguageCode.equals("0"))
                    selectedLanguageCode = Locale.getDefault().getLanguage();
                if (selectedLanguageCode.equals("ar") && Build.VERSION.SDK_INT < 14) // lock Arabic in Android less then 14
                {
                    selectedLanguageCode = "en";
                }

                // store selection result in String array
                String [] retValue = new String [4];
                retValue [0]       = selectedLanguageCode;
                retValue [1]       = (String) languages [languageMapper[item]];
                retValue [2]       = (String) orgLanguages [languageMapper[item]];
                retValue [3]       = makeLanguageLine (retValue [2], retValue [1], retValue [0]);
                Message msg = new Message();
                msg.what = item;
                msg.obj = retValue;
                handler.sendMessage(msg);
                dialog.dismiss();// dismiss the alertbox after chose option
            }
        });
        AlertDialog alert = alt_bld.create();
        alert.show();
    }

    //
    protected String getLanguageLine (int availableLanguageId, String[] availableLanguages) {
        String entry;
        String langCode = availableLanguages [availableLanguageId];
        if (langCode.equals("0")) {
            entry =  activity.getString(R.string.lg_0);
            if (! activity.getString (R.string.language).equals("sr"))
                entry += " ("+ Locale.getDefault().getDisplayLanguage() +")";
            return entry;
        }
        int i = -1;
        do {
            i ++;
            if (langCode.equals(shortLanguages [i])) {
                // Language entry found
                languageMapper [availableLanguageId] = i;

//                entry = (String) orgLanguages [i];
//                try {Log.d("Planetdroid", "Language " +i +languages [i]+ " " +orgLanguages [i]);} catch (Exception  e) {e.printStackTrace();}
//                if (entry.equals("")) {entry = langCode;} // set languages if no orgLanguage entry
//                else if (languages.length > i && ! orgLanguages [i].equals(languages [i]) && ! languages [i].equals(""))
//                    entry += FormatterUtils.LTR +" (" +languages [i]+")";  // append language in selected language in parenthesis
//                    entry = FormatterUtils.LTR + entry;
                selectedLanguageCode = langCode;// Locale.getDefault().getLanguage();
                return makeLanguageLine ((String) orgLanguages [i], (String) languages [i], selectedLanguageCode);
            }
        } while (i < shortLanguages.length);
        return ("");
    }

    public static String makeLanguageLine (String orgLanguage, String language, String defaultOnErr) {
        if (orgLanguage == null || orgLanguage.equals(""))  orgLanguage = defaultOnErr;
        if (language    == null )  language = "";

        String entry = orgLanguage;
        if (!orgLanguage.equals(language) && !language.equals(""))
            entry += FormatterUtils.LTR + " (" + language + ")";  // append language in selected language in parenthesis
//        if (language.equals("ar")) entry = FormatterUtils.LTR + entry;
        return entry;

    }

    /**
     *
     * @param mActivity
     * @param langaugeCode eg "de"
     * @param languageList
     * @return  eg. "German"
     */
    public static String getLanguage (Activity mActivity, String langaugeCode, CharSequence[] languageList) {
        CharSequence[] shortLanguages = mActivity.getResources().getTextArray(R.array.shortLanguages);
        String longName = "";
        for (int i = 0; i < shortLanguages.length ; i++ ) {
            if (langaugeCode.equals(shortLanguages[i])) longName = (String) languageList [i];
        }
        return longName;
    }

    // handles codes like "zh_TW"
    public static Locale getLocale (String languageCode) {
        if (languageCode.contains("_")) {
            return new Locale (languageCode.substring(0, languageCode.indexOf("_")),
                    languageCode.substring(languageCode.indexOf("_")+1, languageCode.length()));
        } else return new Locale (languageCode);
    }

    // wrong value for zh_TW!
    public static String getDisplayName (String dispLanguageCode, String inLanguageCode) {
        try {
            if (dispLanguageCode == null) dispLanguageCode = "en";
            if (inLanguageCode   == null) inLanguageCode   = "en";
        //change combo "sr" ("sr") to sr ("sh"), otherwise cyrillic output!
        if (dispLanguageCode.equals("sr") && inLanguageCode.equals("sr")) inLanguageCode = "sh"; //Serbocroatic = Latin letters, not Cyrillic

            return getLocale(dispLanguageCode).getDisplayName(getLocale(inLanguageCode));
        } catch (Exception e) {
            Log.d ("LanguageSelector", "Error: " +dispLanguageCode + " in: " +inLanguageCode);
            e.printStackTrace();
            return (dispLanguageCode);
        }
    }

    /**
     *
     * @param languageCode
     * @return LanguageName in its own locale
     */
    public static String getOriginalDisplayName (String languageCode) {
        return getDisplayName (languageCode, languageCode);
    }

    /**
     *
     * @param mActivity
     * @param languageCode
     * @return  LanguageName in current Arctitvity's Locale
     */
    public static String getDisplayName (Activity mActivity, String languageCode) {
        return getDisplayName (languageCode, mActivity.getString(R.string.language));
    }
}
