package strickling.forms;

//Generic class definition for a Location input form
// extended eg. by: Location_Input_Map

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;

import com.strickling.forms.R;

import strickling.utils.AstroUtils;
import strickling.utils.GeoCoordinate;
import strickling.utils.GeoRecord;
import strickling.utils.PermissionChecker;
import strickling.utils.RequestGeoCoder;
import strickling.utils.SystemUtils;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.drawable.ColorDrawable;
import android.location.Address;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;


public class Location_Input_Class extends Activity implements TextWatcher, LocationListener {

    public static GeoCoordinate     coordBuff;                        // must be initialized by calling activity or in intent extras, returns selected position
    public static boolean           useLastPosition = true;    // can  be initialized by calling activity
    public static final String      COORD_LP_FORM[] = {"dms0", "dm2", "d5"};
    public static String            coord_form = COORD_LP_FORM[0];    // can be initialized by calling activity
    public static boolean           showAlt = false;                // can be set true to enable altitude input
    public static String            TAG = "Location_Input";    // can be initialized by calling activity
    public static String            alternativeTitle = "";                    // Will be resetted after termination
    public static String            titleExtra1 = "";                    // title of extra spinner item


    public static String PREFS_NAME = "LocationPreferences";
    public static String LOCATION_FILE_NAME; //"sdcard/location.txt";
    public static String WORLDFile;         // 762 locs, from Geonames.org	public static String		WORLDFile 			= WORLDFile0;
//    public static boolean worldContainsAlt = true;
    public static boolean resultIsGeoCoded = false;
    public static Locale locale = Locale.getDefault();


    // Values for variable mode
    public static final int     MODE_GPS = 0;
    public static final int     MODE_FAV = 1;
    public static final int     MODE_USA = 2;     // 37274 locs, from Geonames.org
    public static final int     MODE_CAN = 3;     // 265 locs, from Geonames.org
    public static final int     MODE_MEX = 4;     // 189 locs, from Geonames.org
    public static final int     MODE_BRA = 5;     // 209 locs, from Geonames.org
    public static final int     MODE_SEARCH = 6;
    public static final int     MODE_MAP = 7;     // For Map use subclass Location_Input_Map!
    public static final int     MODE_MAN = 8;     // Enter coordinates maunally
    public static final int     MODE_EXTRA1 = 9;     // Enter coordinates maunally
    public static final int     MODE_WORLD = 10;     // 762 locs, from Geonames.org
    public static final int     MODE_WORLD2 = 11;
    public static final int     MODE_WORLD3 = 12;     // this must be the last (highest) item no!!
    public static int[]         spinner1Items = {MODE_GPS, MODE_FAV, MODE_WORLD, MODE_WORLD2, MODE_WORLD3, MODE_SEARCH, MODE_MAN};
    public static int[]         modeToSpinner1 = new int[MODE_WORLD3 + 1]; // translates mode to spinner pos, initialized in BuildSpinner
    public static String        USFile, CANFile, MEXFile, BRAFile;

    //	public static  int  		defaultmode = MODE_GPS;
    protected int               mode =        MODE_GPS; // inititialized with GPS mode
    protected int               oldMode, requestedMode;
    protected String            fileSizeString;

    protected final static int  MSG_FINISH = 1;
    protected final static int  MSG_NOTFOUND = 2;
    protected final static int  MSG_DownloadComplete = 5;
    protected final static int  MSG_DownloadError = 6;

    protected final static int  DOWNLOAD_ONLY = 10;
    protected final static int  DOWNLOAD_SILENT = 11;  // Download, not showing Dialog
    protected final static int  DISP_SPINNER2 = 12;
    protected final static int  DISP_SPINNER4 = 14;
    protected final static int  DISP_LIST = 15;
    protected final static int  DOWNLOAD_FILE = 20;
    protected final static int  ACTIVATE_SPINNER3 = 23; // if too much cities for one spinner, displays ABC spinner3

    // constants for intent extras
    public static final String  LAT = "LAT";
    public static final String  LON = "LON";
    public static final String  ALT = "ALT";
    public static final String  NAME = "NAME";
    public static final String  COUNTRY = "COUNTRY";
    public static final String  USE_GPS = "USE_GPS";
    public static final String  GEOCODED = "GEOCODED";

    public static String[]      fileNames;
    protected static String     fileURLDir = "http://www.strickling.net/";

    protected List<String>      searchResult = new ArrayList<String>();
    protected static boolean    blockSpinnerSel; // Blocks spinner selection on build


    public static final String  FAV_MARK = "FAV";
    protected int               spinMode2 = -1;
    protected int               spinMode3 = -1;
    protected int               spinMode4 = -1;


    protected EditText          searchBox; //R.id.Loc_EditAddress
    protected EditText          editLon, editLat, editAlt;
    protected CheckBox          saveChkBox; //gpsCheckBox, manChkBox;
    protected TextView          lastPositionView, searchResultView, Loc_LatText, Loc_LonText, Loc_AltText, Loc_EnterAddress;
    protected Button            locSearchBtn, okBtn, cancelBtn, offlineBtn; //resetBtn,
    protected Button            editFavorites_Btn, manageFavorites_Btn; //resetBtn,

    protected ImageView         trashCan;
    protected Spinner           spinner1, spinner2, spinner3, spinner4;

    protected String            Loc_FoundHeaderActual, Loc_FoundHeader, Loc_FoundFooter, Loc_NotFound, Loc_No_Service;
    protected String            LineOLon, LineOLatNL, Loc_CurrentPos, Accuracy, Loc_actual_NET, Loc_actual_GPS;//, menuItem1;
    protected LocationManager   locationManager;
    protected Location          location;  // takes actual GPS record
    protected String            locName;      // geoCoded PGS position name
    protected int               locationRequestCounter = 0;
    protected long              locationRequestRate = 10000; // increase Request rate according to Google limitations
    protected Criteria          criteria = new Criteria();

    protected int               MIN_GPS_LOCATION_UPDATE_TIME = 10000;  // ms
    protected int               MIN_GPS_LOCATION_UPDATE_DISTANCE = 50;    // m

    protected static int        contentViewID = R.layout.location_input; // may be overwritten to chose other layout


    /**
     * on Touch listener to blockSpinnerSel = false to enable user selection
     * and for resetting Map selection to enable reselection of the same item (MAP)
     */
    private View.OnTouchListener Spinner_OnTouch = new View.OnTouchListener() {
        public boolean onTouch(View v, MotionEvent event) {
            if (event.getAction() == MotionEvent.ACTION_DOWN) {
                blockSpinnerSel = false;
                Log.d(TAG, "Spinner onTouch");
                if (mode == MODE_MAP && v.equals(spinner1)) {
                    spinner1.setSelection(0); // resets selection in order to enable reselection of the same item
                }
            }
            return false;
        }
    };


    // called by onCreate
    public void initViews() {
//		Log.d(TAG, "FindViews");
        setContentView(contentViewID);
        Loc_EnterAddress = findViewById(R.id.Loc_EnterAddress);
        searchBox = findViewById(R.id.Loc_EditAddress);
        editLat = findViewById(R.id.EditLat);
        editLon = findViewById(R.id.EditLon);
        editAlt = findViewById(R.id.EditAlt);
        saveChkBox = findViewById(R.id.Loc_Save);
        lastPositionView = findViewById(R.id.Loc_CurrentPos);
        searchResultView = findViewById(R.id.Loc_Found);
        Loc_LatText = findViewById(R.id.Loc_LatText);
        Loc_LonText = findViewById(R.id.Loc_LonText);
        Loc_AltText = findViewById(R.id.Loc_AltText);
        locSearchBtn = findViewById(R.id.Loc_Search);
        offlineBtn = findViewById(R.id.Loc_Offline);
        editFavorites_Btn = findViewById(R.id.editFavorites_Btn);
        manageFavorites_Btn = findViewById(R.id.manageFavorites_Btn);
        okBtn = findViewById(R.id.OK_Btn);
        cancelBtn = findViewById(R.id.Cancel_Btn);
        trashCan = findViewById(R.id.trashCan);

        Loc_FoundHeaderActual = getString(R.string.Loc_FoundHeaderActual);
        Loc_FoundFooter = getString(R.string.Loc_FoundFooter);
        Loc_FoundHeader = getString(R.string.Loc_FoundHeader);
        Loc_NotFound = getString(R.string.Loc_NotFound);
        Loc_No_Service = getString(R.string.No_Service);
        LineOLon = getString(R.string.LineOLon);
        LineOLatNL = getString(R.string.LineOLatNL);
        Loc_CurrentPos = getString(R.string.Loc_CurrentPos);
        if (! PermissionChecker.hasGpsPermission(this))
            Loc_CurrentPos += getString(R.string.Permission) +" " +getString(R.string.denied);
        lastPositionView.setText(Loc_CurrentPos);
        Accuracy = getString(R.string.Accuracy);
        Loc_actual_NET = getString(R.string.Loc_actual_NET);
        Loc_actual_GPS = getString(R.string.Loc_actual_GPS);
//		menuItem1		= getString(R.string.Menu_EditLocs);

        blockSpinnerSel = true;
        spinner1 = findViewById(R.id.spinner1);
        spinner1.setOnTouchListener(Spinner_OnTouch);
        spinner1.setOnItemSelectedListener(new spinnerItemSelectedListener()); // will call spinner selected (0) event

        spinner2 = findViewById(R.id.spinner2);
        spinner2.setOnTouchListener(Spinner_OnTouch);
        spinner2.setOnItemSelectedListener(new spinnerItemSelectedListener());

        spinner3 = findViewById(R.id.spinner3);
        spinner3.setOnTouchListener(Spinner_OnTouch);
        spinner3.setOnItemSelectedListener(new spinnerItemSelectedListener());

        spinner4 = findViewById(R.id.spinner4);
        spinner4.setOnTouchListener(Spinner_OnTouch);
        spinner4.setOnItemSelectedListener(new spinnerItemSelectedListener());
    }


    public void onCreate(Bundle savedInstanceState) {
        if (Build.VERSION.SDK_INT >= 11)
            setTheme(getIntent().getIntExtra("theme",
                    Build.VERSION.SDK_INT >= 21 ? android.R.style.Theme_Material : android.R.style.Theme_Holo)); // set dark theme
        super.onCreate(savedInstanceState);

        if (!SystemUtils.writeChecked)
            SystemUtils.checkSD(this); // check for writeable storage dirctory
        LOCATION_FILE_NAME = SystemUtils.writeableDir + "location.txt";
        WORLDFile = SystemUtils.writeableDir + "world_cities.csv";  // 762 locs, from Geonames.org

        initViews();
//		WORLDFile = getFilesDir().getAbsolutePath() +"/world_cities.csv"; // TODO for Emulator purpose!
        if (!alternativeTitle.equals("")) setTitle(alternativeTitle);
        if (PermissionChecker.hasGpsPermission(this))
            locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        try {        // try reading intent
            coordBuff = getIntentExtras(getIntent());
            useLastPosition = getIntent().getBooleanExtra(Location_Input_Class.USE_GPS, useLastPosition);
            Log.d(TAG, String.format(" Coord from Intent: %.3f deg %.3f deg useLastPos: " + useLastPosition, coordBuff.getLat(), coordBuff.getLon()));
        } catch (Exception e) {
            Log.d(TAG, "Intent Error ");
        }

        if (coordBuff != null) { //Capture non initialized location
            searchResultView.setText(Loc_FoundHeaderActual +
                    makeLocString(coordBuff.getLat(), coordBuff.getLon(), coordBuff.getName())
                    + Loc_FoundFooter);
            if (coordBuff.getName() != "") {
                searchBox.setText(coordBuff.getNameAndCountry());
            }
            editAlt.setText(String.format(locale, "%.0f", coordBuff.getHeight()));
        } else
            coordBuff = new GeoCoordinate(); // should be initialized by calling activity or handed by Intent

        fillLocationData(coordBuff);
//		editLat.setText(stripDMS (coordBuff.getLat()));
//		editLon.setText(stripDMS (coordBuff.getLon()));


        searchBox.addTextChangedListener(this);
        editLat.addTextChangedListener(this);
        editLon.addTextChangedListener(this);
        editAlt.addTextChangedListener(this);

        if (savedInstanceState != null) {
            locName = savedInstanceState.getString("locName");
            saveChkBox.setChecked(savedInstanceState.getBoolean("saveChk"));
            showCurrentLocation();
        } else { // Usually called on first start
            locName = "";
        }

        buildSpinner1();

        if (savedInstanceState != null) {
            spinMode2 = savedInstanceState.getInt("loc_spinMode2");
            spinMode3 = savedInstanceState.getInt("loc_spinMode3");
            spinMode4 = savedInstanceState.getInt("loc_spinMode4");
            setMode(savedInstanceState.getInt("loc_mode"));
            searchBox.setText(savedInstanceState.getString("loc_searchBox"));
            editLat.setText(savedInstanceState.getString("loc_editLat"));
            editLon.setText(savedInstanceState.getString("loc_editLon"));
            editAlt.setText(savedInstanceState.getString("loc_editAlt"));
            searchResult = savedInstanceState.getStringArrayList("loc_searchResult");
            searchResultView.setText(savedInstanceState.getString("searchResultView"));
            Log.d(TAG, "Init finished, mode: " + mode + " si. saved: " + savedInstanceState.getInt("loc_mode", MODE_GPS) + " ListLen: " + searchResult.size());
        } else {
            if (useLastPosition) {
                setMode(MODE_GPS);
            } else {
                SharedPreferences sp = getSharedPreferences(PREFS_NAME, 0);
                spinMode2 = sp.getInt("loc_spinMode2", spinMode2);
                spinMode3 = sp.getInt("loc_spinMode3", spinMode3);
                spinMode4 = sp.getInt("loc_spinMode4", spinMode4);
                Log.d(TAG, "Location_Input_Class onCreate, Settings, Mode: " + sp.getInt("loc_mode", -99));
                setMode(sp.getInt("loc_mode", mode));
                if (coordBuff == null) { // restore old input if invalid coordBuff, otherwise was init by super.onCreate
                    searchBox.setText(sp.getString("loc_searchBox", ""));
                    editLat.setText(sp.getString("loc_editLat", AstroUtils.DMSStrgDeg(coordBuff.getLat(), coord_form)));
                    editLon.setText(sp.getString("loc_editLon", AstroUtils.DMSStrgDeg(coordBuff.getLon(), coord_form)));
                    if (showAlt) {
                        editAlt.setText(sp.getString("loc_editAlt", "0"));
                    } else editAlt.setText("0");
                }
                Log.d(TAG, "Init finished, mode: " + mode + " pr. saved: " + sp.getInt("loc_mode", -1) + " 4 sel:" + spinMode4);
            }
        }


        if (!new File(LOCATION_FILE_NAME).exists()) offlineBtn.setVisibility(View.GONE);
        // Remove Keyboard
        this.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);

        // Set blue actionBar with icon, replaces android:theme="@style/ThemeDialogWhenLarge" in manifest activity
        SystemUtils.configureActionBar(this);
    }  // on Create

    protected String stripDMS(double angleDeg) {
        String result = AstroUtils.DMSStrgDeg(angleDeg, coord_form);
        result = result.replace("+", "").trim().replace("- ", "-");
        result = result.replace("\u00B0", "").replace("'", "").replace("\"", "");
        return result;
    }

    protected void onResume() {
        super.onResume();
        if (PermissionChecker.hasGpsPermission(this) && locationManager != null) {
            try {
                criteria.setAccuracy(Criteria.ACCURACY_FINE);    // More accurate, GPS fix.
                locationManager.requestLocationUpdates(locationManager.getBestProvider(criteria, true),
                        MIN_GPS_LOCATION_UPDATE_TIME, MIN_GPS_LOCATION_UPDATE_DISTANCE, this);
            } catch (Exception e) {
            }
        }
        locationRequestCounter = 0;
        locationRequestRate = 10000;
        if (PermissionChecker.hasGpsPermission(this)) getCurrentLocation();
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (locationManager != null) {
            locationManager.removeUpdates(this);
        }
        saveState();
    }

    @Override
    protected void onDestroy() {
        alternativeTitle = "";
        super.onDestroy();
    }


    /**
     * Save settings to preserve after screen rotation
     */
    protected void onSaveInstanceState(Bundle savedInstanceState) {
        savedInstanceState.putInt("loc_mode", mode);
        savedInstanceState.putInt("loc_spinMode2", spinner2.getSelectedItemPosition());
        savedInstanceState.putInt("loc_spinMode3", spinner3.getSelectedItemPosition());
        savedInstanceState.putInt("loc_spinMode4", spinner4.getSelectedItemPosition());
        savedInstanceState.putString("loc_searchBox", searchBox.getText().toString());
        savedInstanceState.putString("loc_editLat", editLat.getText().toString());
        savedInstanceState.putString("loc_editLon", editLon.getText().toString());
        savedInstanceState.putString("loc_editAlt", editAlt.getText().toString());
        if (searchResult != null) {
            savedInstanceState.putStringArrayList("loc_searchResult", (ArrayList<String>) searchResult);
            savedInstanceState.putString("searchResultView", searchResultView.getText().toString());

        }
    }

    /**
     * Save settings to preserve after screen rotation
     */
    protected void saveState() {
        SharedPreferences.Editor editor = getSharedPreferences(PREFS_NAME, 0).edit();
        editor.putInt("loc_mode", mode);

        if (spinner2.getVisibility() == View.VISIBLE) {
            editor.putInt("loc_spinMode2", spinner2.getSelectedItemPosition());
        } else editor.putInt("loc_spinMode2", -1);

        if (spinner3.getVisibility() == View.VISIBLE) {
            editor.putInt("loc_spinMode3", spinner3.getSelectedItemPosition());
        } else editor.putInt("loc_spinMode3", -1);

        if (spinner4.getVisibility() == View.VISIBLE && spinner4.isEnabled()) {
            editor.putInt("loc_spinMode4", spinner4.getSelectedItemPosition());
        } else editor.putInt("loc_spinMode4", -1);

        editor.putString("loc_searchBox", searchBox.getText().toString());
        editor.putString("loc_editLat", editLat.getText().toString());
        editor.putString("loc_editLon", editLon.getText().toString());
        if (showAlt) {
            editor.putString("loc_editAlt", editAlt.getText().toString());
        } else editor.putString("loc_editAlt", "0");
        editor.commit();
        Log.d(TAG, String.format("saveState: %d, %d, %d, %d , Name: %s", mode,
                getSharedPreferences(PREFS_NAME, 0).getInt("loc_spinMode2", -9),
                getSharedPreferences(PREFS_NAME, 0).getInt("loc_spinMode3", -9),
                getSharedPreferences(PREFS_NAME, 0).getInt("loc_spinMode4", -9),
                PREFS_NAME
        ));
    } //saveState

    /**
     * adds coordBuff values and useLastPosition to intent
     */
    protected void putIntentExtras(Intent intent) {
        putIntentExtras(intent, coordBuff, useLastPosition);
    }

    /**
     * adds coordBuff values and useLastPosition to intent
     */
    public static void putIntentExtras(Intent intent, double lat, double lon, double alt, String name, String country, boolean useGPS) {
        intent.putExtra(LAT, lat);
        intent.putExtra(LON, lon);
        intent.putExtra(ALT, alt);
        intent.putExtra(NAME, name);
        intent.putExtra(COUNTRY, country);
        intent.putExtra(USE_GPS, useGPS);
//		Log.d(TAG, String.format(Locale.US, "putIntentExtras, Name: %s, %.3f deg, %.3f deg", name +", " +country, lat, lon));
//		Log.d(TAG, String.format(Locale.US, "Test:          , Name: %s, %.3f deg, %.3f deg", intent.getStringExtra(NAME) ,
//				intent.getDoubleExtra(LAT, 0), intent.getDoubleExtra(LON, 0)));
    }

    public static void putIntentExtras(Intent intent, Location loc, String name, String country, boolean useGPS) {
        putIntentExtras(intent, loc.getLatitude(), loc.getLongitude(), loc.getAltitude(), name, country, useGPS);
    }

    public static void putIntentExtras(Intent intent, GeoCoordinate geoCoord, boolean useGPS) {
        putIntentExtras(intent, geoCoord.location.getLatitude(), geoCoord.location.getLongitude(), geoCoord.location.getAltitude(),
                geoCoord.name, geoCoord.countryName, useGPS);
        intent.putExtra(GEOCODED, geoCoord.isGeocoded);
    }

    public static GeoCoordinate getIntentExtras(Intent data) {
        GeoCoordinate geoCoord = new GeoCoordinate(
                data.getDoubleExtra(LAT, 0), data.getDoubleExtra(LON, 0),
                data.getDoubleExtra(ALT, 0),
                data.getStringExtra(NAME), data.getStringExtra(COUNTRY));
        geoCoord.isGeocoded = data.getBooleanExtra(GEOCODED, false);
//    	Log.d(TAG, String.format(Locale.US, "**** getIntentExtras, Name: %s, %.3f deg, %.3f deg", geoCoord.name , geoCoord.getLat(), geoCoord.getLon()));
        return geoCoord;
    }


    // TODO Spinner methods ********************************


    public void buildSpinner1() {
//		Log.d(TAG, "buildSpinner1 Start");
        if (mode != 0)
            blockSpinnerSel = true;

        spinner1.setPrompt(getString(R.string.selectMode));
        ArrayAdapter<CharSequence> adapter = new ArrayAdapter<CharSequence>(
                this, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.select_dialog_singlechoice);

        boolean splitWorld = false;
        // check, whether world countries are splitted into more then one spinner
        for (int i = 0; i < spinner1Items.length; i++) {
            if (spinner1Items[i] == MODE_WORLD2 || spinner1Items[i] == MODE_WORLD3)
                splitWorld = true;
        }


        // Fill Adapter
        for (int i = 0; i < spinner1Items.length; i++) {
            int n = spinner1Items[i];
            modeToSpinner1[n] = i;
            switch (n) {
                case MODE_GPS:
                    if (PermissionChecker.hasGpsPermission(this)) {
                        adapter.add(getString(R.string.SPINNER_GPS));
                    } else adapter.add("(" +getString(R.string.SPINNER_GPS) +")"); // add text in brackets
                    break;
                case MODE_FAV:
                    adapter.add(getString(R.string.SPINNER_FAV));
                    break;
                case MODE_MAP:
                    adapter.add(getString(R.string.SPINNER_MAP));
                    break;
                case MODE_SEARCH:
                    adapter.add(getString(R.string.SPINNER_SEARCH));
                    break;
                case MODE_EXTRA1:
                    adapter.add(titleExtra1);
                    break;
                case MODE_MAN:
                    adapter.add(getString(R.string.SPINNER_MAN));
                    break;

                case MODE_WORLD:
                    if (!splitWorld) {
                        adapter.add(getString(R.string.SPINNER_WORLD));
                    } else adapter.add(getString(R.string.SPINNER_WORLD) + " A - "
                            + (getResources().getTextArray(R.array.countryGroups)[0]));
                    break;
                case MODE_WORLD2:
                    adapter.add(getString(R.string.SPINNER_WORLD)
                            + " " + (getResources().getTextArray(R.array.countryGroups)[1])
                            + " - " + (getResources().getTextArray(R.array.countryGroups)[2]));
                    break;
                case MODE_WORLD3:
                    adapter.add(getString(R.string.SPINNER_WORLD)
                            + " " + (getResources().getTextArray(R.array.countryGroups)[3])
                            + " - Z");
                    break;

                // only for Eclipse2017:
                case MODE_USA:
                    adapter.add(getString(R.string.SPINNER_USA));
                    break;
                case MODE_CAN:
                    adapter.add(getString(R.string.SPINNER_CAN));
                    break;
                case MODE_MEX:
                    adapter.add(getString(R.string.SPINNER_MEX));
                    break;
                case MODE_BRA:
                    adapter.add(getString(R.string.SPINNER_BRA));
                    break;
            }
        }
        NothingSelectedSpinnerAdapter nsa = new NothingSelectedSpinnerAdapter(adapter,
                R.layout.nothing_selected, this);
        spinner1.setAdapter(nsa);  // app crash, if target is Android 21 or higher! (Spinner adapter view type count must be 1)
//		Log.d(TAG, "buildSpinner1 finished");
    }

    /**
     * builds spinner (2) with states, provinces, countries
     */
    public void buildSpinner(Spinner spinner, int selectPos) {
//		Log.d(TAG, "buildSpinner Class Start");
        spinner.setVisibility(View.VISIBLE);
        ArrayAdapter<CharSequence> adapter = new ArrayAdapter<CharSequence>(
                this, android.R.layout.simple_spinner_item);

        adapter.setDropDownViewResource(android.R.layout.select_dialog_singlechoice);
        CharSequence[] spinnerList = null;
        spinner.setAdapter(adapter);
        switch (mode) {
            case MODE_WORLD:
            case MODE_WORLD2:
            case MODE_WORLD3:
                spinner.setPromptId(R.string.selectCountry);
                CharSequence[] cl = null;
                switch (mode) {
                    case MODE_WORLD:
                        cl = getResources().getTextArray(R.array.countriesList1);
                        break;
                    case MODE_WORLD2:
                        cl = getResources().getTextArray(R.array.countriesList2);
                        break;
                    case MODE_WORLD3:
                        cl = getResources().getTextArray(R.array.countriesList3);
                        break;
                }
//			Log.d(TAG, "locale " +locale.getLanguage());
                String lang = locale.getLanguage();
                // Build Cities list
                for (int i = 0; i < cl.length; i++) {
                    if (cl[i].length() == 2) {
                        // 2-digit ISO codes: insert translated name
                        Locale l = new Locale(lang, (String) cl[i]);
                        String s = getState(l.getDisplayName(locale));
                        if (s.equals(cl[i])) s = "";
                        cl[i] = "(" + cl[i] + ") " + s;
                    }
                }
                spinnerList = cl;
                break;
        }
        // Fill Adapter
        for (int i = 0; i < spinnerList.length; i++) {
            adapter.add(spinnerList[i]);
        }
        blockSpinnerSel = true;
        NothingSelectedSpinnerAdapter nsa = new NothingSelectedSpinnerAdapter(adapter,
                R.layout.nothing_selected, this);
        spinner.setAdapter(nsa);
        try {
            if (selectPos >= 0) spinner.setSelection(selectPos);
        } catch (Exception e) {
            e.printStackTrace();
        }
//		Log.d(TAG, "buildSpinner finished, block: " +blockSpinnerSel);
    }


    /**
     * Builds a spinner to display alphabetic start letters
     *
     * @param spinner
     */
    public void buildSpinnerABC(Spinner spinner, int selectPos) {
//		Log.d(TAG, "buildSpinnerABC Start");
        blockSpinnerSel = true;
        spinner.setVisibility(View.VISIBLE);
        ArrayAdapter<CharSequence> adapter = new ArrayAdapter<CharSequence>(
                this, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.select_dialog_singlechoice);
        CharSequence[] spinnerList = null;
        spinnerList = new CharSequence[26];
        for (int i = (int) 'A'; i <= (int) 'Z'; i++) {
            spinnerList[i - (int) 'A'] = new String("" + (char) i);
        }
        // Fill Adapter
        for (int i = 0; i < spinnerList.length; i++) {
            adapter.add(spinnerList[i]);
        }
        spinner.setAdapter(adapter);
        try {
            if (selectPos >= 0) spinner.setSelection(selectPos);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * file must be read before and stored in the searchResult array
     *
     * @param spinner
     */
    public void buildSpinnerList(Spinner spinner, int selectPos) {
//		Log.d(TAG, "buildSpinnerList Start, list len: " +searchResult.size());
        spinner.setVisibility(View.VISIBLE);

        ArrayAdapter<String> adapter = new ArrayAdapter<String>(
                this, android.R.layout.simple_spinner_item);

        // add searchResult list to adapter
        for (int i = 0; i < searchResult.size(); i++) {
            // display get only  city name, not coordinates etc
            // example line:    "NY;New York;-74.005978;40.714275;-5;Y"
            try {
                String item = searchResult.get(i);
                if (!item.equals("") && item.contains(";")) {
                    item = item.substring(item.indexOf(";") + 1);
                    item = item.substring(0, item.indexOf(";"));
                    adapter.add(item);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }  // wrong internal structure of record
        }

        // set adapter to NothingSelectedSpinnerAdapter
        adapter.setDropDownViewResource(android.R.layout.select_dialog_singlechoice);
        NothingSelectedSpinnerAdapter nsa = new NothingSelectedSpinnerAdapter(adapter,
                R.layout.nothing_selected, this);
        spinner.setAdapter(nsa);

        if (adapter.isEmpty()) {
            spinner.setEnabled(false);
            //			if (mode == MODE_FAV) Toast.makeText(this, getString(R.string.noFav), Toast.LENGTH_LONG).show();
        } else try {
            spinner.setEnabled(true);
            if (selectPos >= 0) spinner.setSelection(selectPos);
        } catch (Exception e) {
            e.printStackTrace();
        }
//		Log.d(TAG, "buildSpinnerList finished, block: " +blockSpinnerSel);
    }

    /**
     * Clears spinner and fills with empty adapter
     *
     * @param spinner
     */
    public void clearSpinner(Spinner spinner) {
        spinner.setVisibility(View.VISIBLE);
        spinner.setAdapter(new ArrayAdapter<CharSequence>(
                this, android.R.layout.simple_spinner_item));
    }


    public class spinnerItemSelectedListener implements OnItemSelectedListener {

        public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
            //    		Log.d(TAG, "spinnerItemSelectedListener, pos: " + pos +"  block: " +blockSpinnerSel);
            if (blockSpinnerSel || parent.getItemAtPosition(pos) == null) {
                if (parent.equals(spinner4) && spinMode4 > 0) {
                    pos = spinMode4;
                    spinMode4 = 0;
                } else {
                    //    				Log.d(TAG, "onItemSelected: return without doing anything...");
                    //    	    	blockSpinnerSel = false;
                    return;
                }
            }

            clearModes();

            if (parent.equals(spinner1)) {
                // Selection in Spinner 1 ******************************

                Log.d(TAG, "spinner1 ItemSelectedListener: "
                        + parent.getItemAtPosition(pos).toString() + " Pos: " + pos + ", " + " ****");
                //    			spinner1.setSelected(true);
                setMode(spinner1Items[pos - 1]);

            } else if (parent.equals(spinner2)) {

                // Selection in Spinner 2 (State, Province, Country ABC) ******************************
                Log.d(TAG, "spinner2 ItemSelectedListener: " + parent.getItemAtPosition(pos).toString() + " Pos: " + pos);
                if (spinner2.getTag() != null) Log.d(TAG, "  TAG:  " + spinner2.getTag());
                switch (mode) {
                    case MODE_USA:
                        clearSpinner(spinner4);
                        spinner4.setEnabled(false);
                        readFileInThread(USFile, getState(parent.getItemAtPosition(pos).toString()) + ";", ACTIVATE_SPINNER3);
                        break;
                    case MODE_CAN:
                        clearSpinner(spinner4);
                        spinner4.setEnabled(false);
                        readFileInThread(CANFile, getState(parent.getItemAtPosition(pos).toString()) + ";", ACTIVATE_SPINNER3);
                        break;
                    case MODE_MEX:
                        readFileInThread(MEXFile, parent.getItemAtPosition(pos).toString(), DISP_SPINNER4);
                        break;
                    case MODE_BRA:
                        readFileInThread(BRAFile, parent.getItemAtPosition(pos).toString(), DISP_SPINNER4);
                        break;
                    case MODE_WORLD:
                    case MODE_WORLD2:
                    case MODE_WORLD3:
                        clearSpinner(spinner4);
                        spinner4.setEnabled(false);
                        readFileInThread(WORLDFile, getState(parent.getItemAtPosition(pos).toString()) + ";", ACTIVATE_SPINNER3);
                        break;
                }

            } else if (parent.equals(spinner3)) {
//				 ABC spinner cities    ****************************** 			
                Log.d(TAG, "spinner3 ItemSelectedListener: "
                        + parent.getItemAtPosition(pos).toString() + " Pos: " + pos);
                switch (mode) {
                    case MODE_USA:
                        readFileInThread(USFile, getState((String) spinner2.getSelectedItem())
                                + ";" + parent.getItemAtPosition(pos).toString(), DISP_SPINNER4);
                        break;
                    case MODE_CAN:
                        readFileInThread(CANFile, getState((String) spinner2.getSelectedItem())
                                + ";" + parent.getItemAtPosition(pos).toString(), DISP_SPINNER4);
                        break;
                    case MODE_WORLD:
                    case MODE_WORLD2:
                    case MODE_WORLD3:
                        readFileInThread(WORLDFile, getState((String) spinner2.getSelectedItem())
                                + ";" + parent.getItemAtPosition(pos).toString(), DISP_SPINNER4);
                        break;
                }

            } else if (parent.equals(spinner4)) {
                Log.d(TAG, "spinner4 ItemSelectedListener: "
                        + parent.getItemAtPosition(pos).toString() + " Pos: " + pos);
                searchResultView.setVisibility(View.VISIBLE);
                GeoCoordinate gc = getGeoLocation(pos - 1);
                if (gc.isValid()) {
                    coordBuff = gc;
                    fillLocationData(coordBuff);
                }
                displayLocation("");

            }
        }

        public void onNothingSelected(AdapterView<?> parent) {
            Log.d(TAG, "spinnerItemSelectedListener:  Nothing selected"); // is never firing?!?
        }
    }

    /**
     * Clears spinner modes 2-4
     */
    protected void clearModes() {
        spinMode2 = -1;
        spinMode3 = -1;
        spinMode4 = -1;
    }

    protected void initSpinner2_4() {
        hideSearchBtns();
        spinner3.setVisibility(View.GONE);
        clearSpinner(spinner4);
        spinner4.setVisibility(View.VISIBLE);
        spinner4.setEnabled(false);
        spinner4.setPromptId(R.string.selectCity);

        if (!SystemUtils.fileExists(fileNames[0])) {
            startDownload(fileNames[0]);
//			Log.d(TAG, "initSpinner2_4 start Downloading " +fileNames [0]);
        }


        if (mode == MODE_MEX || mode == MODE_BRA) {  // ABC is not in spinner3 but in spinner2
            spinner3.setVisibility(View.GONE);
            buildSpinnerABC(spinner2, spinMode2);
            if (spinMode4 > 0 && spinMode2 >= 0)
                readFileInThread(spinner2.getItemAtPosition(spinMode2).toString(), DISP_SPINNER4);
        } else {
            buildSpinner(spinner2, spinMode2);  // display states, countries...

            if (spinMode3 > 0) {  // many cities in state
                buildSpinnerABC(spinner3, spinMode3);
                if (spinMode4 > 0) {
                    readFileInThread(getState((String) spinner2.getSelectedItem())
                            + ";" + spinner3.getItemAtPosition(spinMode3).toString(), DISP_SPINNER4);
                }
            } else if (spinMode4 > 0) { // Few cities in state
                spinner3.setVisibility(View.GONE);
                readFileInThread(getState((String) spinner2.getSelectedItem()) + ";", DISP_SPINNER4);
            }
        }
    }

    /**
     * Sets location input mode                     TODO mark
     *
     * @param mode
     */
    public void setMode(int mode) {
        oldMode = this.mode;
        requestedMode = mode;
        this.mode = mode;
        useLastPosition = mode == MODE_GPS;
        Log.d(TAG, "Location_Input_Class.setMode " + mode);
        if (spinner1.getSelectedItemPosition() != modeToSpinner1[mode] + 1) {
            //			Log.d(TAG, "Set spinner selection to " +mode);
            spinner1.setSelection(modeToSpinner1[mode] + 1);
        }

        if (mode == MODE_FAV && SystemUtils.fileExists(LOCATION_FILE_NAME)) {
            editFavorites_Btn.setVisibility(View.VISIBLE);
            manageFavorites_Btn.setVisibility(View.VISIBLE);
        } else {
            editFavorites_Btn.setVisibility(View.GONE);
            manageFavorites_Btn.setVisibility(View.GONE);
        }

        switch (mode) {
            case MODE_FAV:
                fileNames = new String[1];
                fileNames[0] = LOCATION_FILE_NAME;
                hideSearchBtns();
                spinner2.setVisibility(View.GONE);
                spinner3.setVisibility(View.GONE);
                spinner4.setVisibility(View.VISIBLE);
                spinner4.setPromptId(R.string.SPINNER_FAV);
                if (spinMode4 < 0 || searchResult != null) {
                    readFileInThread(LOCATION_FILE_NAME, "", DISP_SPINNER4);
                }
                buildSpinnerList(spinner4, spinMode4);
                break;
            case MODE_WORLD:
            case MODE_WORLD2:
            case MODE_WORLD3:
                if (SystemUtils.fileExists(WORLDFile) || oldMode == mode) {
                    fileNames = new String[1];
                    fileNames[0] = WORLDFile;
                    spinner2.setPromptId(R.string.selectCountry);
                    initSpinner2_4();
                } else {
                    this.mode = oldMode;
//				if (WORLDFile.equals(WORLDFile0))
                    fileSizeString = getString(R.string.size_world_cities);
                    removeDialog(DOWNLOAD_FILE);
                    showDialog(DOWNLOAD_FILE);
                }
                break;
            case MODE_EXTRA1:
            case MODE_SEARCH:
                showSearchBtns();
                spinMode4 = -1;
                break;
            case MODE_GPS:
//			Log.d (TAG, "GPS");
                useLastPosition = true;
                hideSearchBtns();
                searchBox.setText(coordBuff.getName());
                spinner2.setVisibility(View.GONE);
                spinner3.setVisibility(View.GONE);
                spinner4.setVisibility(View.GONE);
                break;
            case MODE_MAN:
                Log.d(TAG, String.format("CoordBuff: %s, %7.3f %7.3f", coordBuff.getName(), coordBuff.getLat(), coordBuff.getLon()));
                hideSearchBtns();
                searchBox.setText(coordBuff.getName());
                spinner2.setVisibility(View.GONE);
                spinner3.setVisibility(View.GONE);
                spinner4.setVisibility(View.GONE);
                break;
        }
    } // setMode

    /**
     * Gets US or CAN state from line (eg from: "Alaska (AL)")
     *
     * @param selItem
     * @return
     */
    protected String getState(String selItem) {
        try {
//			Log.d(TAG, "getState: " +selItem 
//					+":  " +selItem.substring(selItem.indexOf("(") +1, selItem.indexOf(")")));
            if (selItem.contains("(")) {
                return selItem.substring(selItem.indexOf("(") + 1, selItem.indexOf(")"));
            } else return selItem.trim();
        } catch (Exception e) {
            return "";
        }
    }

    /**
     * enters name and coordinates into EditText fields
     *
     * @param gc
     */
    protected void fillLocationData(GeoCoordinate gc) { //TODO
        editLat.setText(stripDMS(gc.getLat()));
        editLon.setText(stripDMS(gc.getLon()));
        editAlt.setText(String.format(locale, "%.0f", coordBuff.getHeight()));
        searchBox.setText(gc.getName());
    }


    protected void hideShowGPSMan() {
        searchBox.setVisibility(/*mode == MODE_GPS ||*/ mode == MODE_MAN ? View.VISIBLE : View.GONE);  // this is the EditText
        trashCan.setVisibility(searchBox.getVisibility());
        Loc_EnterAddress.setVisibility(searchBox.getVisibility()); //(mode == MODE_MAN ? View.VISIBLE: View.INVISIBLE);//

        lastPositionView.setVisibility(mode == MODE_GPS ? View.VISIBLE : View.GONE);
        editLat.setVisibility(mode == MODE_MAN ? View.VISIBLE : View.GONE);
//		Log.d(TAG, "HideShow coordBurr: " +(coordBuff != null) +" mode " +(mode == MODE_MAN));
        if (coordBuff != null && mode == MODE_MAN) fillLocationData(coordBuff);
//			Log.d(TAG, String.format("Coord: %.3f deg %.3f deg", coordBuff.getLat(), coordBuff.getLon()));
//			editLat.setText(stripDMS (coordBuff.getLat()));
//			editLon.setText(stripDMS (coordBuff.getLon()));
//		}
        editLon.setVisibility(editLat.getVisibility());
        Loc_LatText.setVisibility(editLat.getVisibility());
        Loc_LonText.setVisibility(editLat.getVisibility());
        if (showAlt) {
            editAlt.setVisibility(editLat.getVisibility());
            Loc_AltText.setVisibility(editLat.getVisibility());
        } else {
            editAlt.setVisibility(View.GONE);
            Loc_AltText.setVisibility(View.GONE);
        }
    }

    protected void hideSearchBtns() {
        searchResultView.setVisibility(View.GONE);
        locSearchBtn.setVisibility(View.GONE);
        offlineBtn.setVisibility(View.GONE);
        hideShowGPSMan();
//		searchBox.setVisibility	(saveChkBox.isChecked() ? View.VISIBLE : View.GONE);
        Loc_EnterAddress.setVisibility(searchBox.getVisibility());
        trashCan.setVisibility(searchBox.getVisibility());
    }

    protected void showSearchBtns() {
        spinner2.setVisibility(View.GONE);
        spinner3.setVisibility(View.GONE);
        spinner4.setVisibility(View.GONE);

        searchResultView.setVisibility(View.VISIBLE);
        searchResultView.setText("");//"\n\n\n");
        locSearchBtn.setVisibility(View.VISIBLE);
        offlineBtn.setVisibility(View.VISIBLE);
        //    	resetBtn.setVisibility	(View.VISIBLE);
        hideShowGPSMan(); // will hide searchBox!
        searchBox.setVisibility(View.VISIBLE);
        trashCan.setVisibility(View.VISIBLE);
        Loc_EnterAddress.setVisibility(View.VISIBLE);
    }


    // end new spinner management


    static final int MENU_EDIT = 1;
    static final int MENU_MANAGE = 2;

    public boolean onPrepareOptionsMenu(Menu menu) {
        if (new File(LOCATION_FILE_NAME).exists()) {
            menu.clear();
            menu.add(0, MENU_EDIT, 0, R.string.Menu_EditLocs);
            menu.getItem(menu.size() - 1).setIcon(android.R.drawable.ic_menu_edit);
            menu.add(0, MENU_MANAGE, 0, R.string.manageFavorites);
            menu.getItem(menu.size() - 1).setIcon(android.R.drawable.ic_delete);
        }
        return super.onPrepareOptionsMenu(menu);
    }

    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case MENU_EDIT:
                Intent intent = new Intent(Intent.ACTION_VIEW); //ECTION_EDIT funzt nicht!
                intent.setDataAndType(Uri.fromFile(new File(LOCATION_FILE_NAME)), "text/plain");
                //	            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                try {
                    startActivity(intent);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return true;
            case MENU_MANAGE:
                removeDialog(MENU_MANAGE);
                showDialog(MENU_MANAGE);
        }
        return false;
    }

    // Button clicks   TODO
    public void btnClick(View view) {

        // click on Google-Search
        if (view == locSearchBtn) {
//				@SuppressWarnings("unused")
//				RequestGeoCoder rgc = 
            Log.d(TAG, " Start  RequestGeoCoder: " + searchBox.getText().toString());
            new RequestGeoCoder(
                    this, searchBox.getText().toString(), locDisplayHandler);
            SystemUtils.showHourglass(this, getString(R.string.wait));
            // Remove Keyboard
            try {
                InputMethodManager inputManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (inputManager != null)
                    inputManager.hideSoftInputFromWindow(this.getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

        if (view == offlineBtn) { // offline search
            searchResultView.setText(Loc_NotFound + Loc_FoundFooter);
            // Build fileNames array
            fileNames = new String[2];
            fileNames[0] = LOCATION_FILE_NAME;
            int n = 1;
            if (!SystemUtils.fileExists(LOCATION_FILE_NAME)) n = 0;
            if (SystemUtils.fileExists(WORLDFile)) fileNames[n] = WORLDFile;
            readFileInThread(searchBox.getText().toString(), DISP_LIST);
            // hide keypad
            try {
                InputMethodManager inputManager = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                if (inputManager != null)
                    inputManager.hideSoftInputFromWindow(this.getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
            } catch (Exception e) {
                e.printStackTrace();
            }


        }
        if (view.equals(saveChkBox)) {
            if (saveChkBox.isChecked() || mode == MODE_SEARCH || mode == MODE_MAN) {
                searchBox.setVisibility(View.VISIBLE);  // EditText
            } else searchBox.setVisibility(View.GONE);
            trashCan.setVisibility(searchBox.getVisibility());
            Loc_EnterAddress.setVisibility(searchBox.getVisibility()); // TextView

            // Remove Keyboard
            SystemUtils.hideKeyboard(this);
        }

        if (view.equals(trashCan)) {
            // Click on Reset or Trashcan
            searchBox.setText("");
            SystemUtils.showKeyboard(this, searchBox);
        }

        if (view.equals(editFavorites_Btn)) {
            Intent intent = new Intent(Intent.ACTION_VIEW); //ECTION_EDIT funzt nicht!
            intent.setDataAndType(Uri.fromFile(new File(LOCATION_FILE_NAME)), "text/plain");
            //	            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            try {
                startActivity(intent);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        if (view.equals(manageFavorites_Btn)) {
            removeDialog(MENU_MANAGE);
            showDialog(MENU_MANAGE);
        }

        //case R.id.OK_Btn:
        if (view == okBtn) {
            //Log.d(TAG, "OK pressed:  " +coordBuff.getName() +" " +coordBuff.getLat() + " " +coordBuff.getLon());
            if (!(searchBox.getVisibility() == View.VISIBLE))
                searchBox.setText(""); // Clear searchBox if invisible
            if (saveChkBox.isChecked()) {
                coordBuff.name = searchBox.getText().toString();
                // saves String in favorites Location
                saveLocation(makeFavString(coordBuff));
            }
            useLastPosition = mode == MODE_GPS;
            Intent outputIntent = new Intent();
            putIntentExtras(outputIntent);
            setResult(Activity.RESULT_OK, outputIntent);
            finish();
        }

        // Click on Cancel
        if (view == cancelBtn) {
            setResult(Activity.RESULT_CANCELED);
            finish();
        }
    }

    public static String makeFavString(GeoCoordinate gc) {
        return String.format(Locale.US, "%s;%.7f;%.7f;%.0f;%s",
                gc.getName().replace(";", ","),
                gc.getLat(),
                gc.getLon(),
                gc.getHeight(),
                gc.timeZoneStrg);
    }

    /**
     * saves location in favorites
     *
     * @param locToSave
     */
    protected void saveLocation(String locToSave) {
        File file = new File(LOCATION_FILE_NAME);
        if (!file.exists())
            try {
                File dir = new File(SystemUtils.stripFileDir(LOCATION_FILE_NAME));
                dir.mkdirs(); // Create Directory
                file.createNewFile();
//				Log.d(TAG, "File created: " +LOCATION_FILE_NAME);
            } catch (IOException e) {
                e.printStackTrace();
//				Log.d(TAG, "**** " +e.toString());
            }

        FileOutputStream fileOutputStream = null;
        try {
            fileOutputStream = new FileOutputStream(file, true);
        } catch (Exception e) {
            e.printStackTrace();
            Log.d(TAG, "**** " + e.toString());
        } finally {
            try {
                locToSave = locToSave + "\r\n";
                fileOutputStream.write(locToSave.getBytes(), 0, locToSave.getBytes().length);
                fileOutputStream.flush();
                fileOutputStream.close();
                fileOutputStream = null;
            } catch (Exception e) {
                e.printStackTrace();
                Log.d(TAG, "**** " + e.toString());
            }

        }
    }

    @SuppressWarnings("finally")
    protected double getDegInput(TextView tv) {
        double r = 0;
        String s = tv.getText().toString();
        s = s.replace(",", ".");       // avoids problems with decimal comma (e.g. German language)
        try {
            StringTokenizer tok = new StringTokenizer(s, " ");
            double deg = Double.parseDouble(tok.nextToken());
            r = deg;
            double min = Double.parseDouble(tok.nextToken());
            r = (Math.abs(deg) + min / 60.);
            if (s.contains("-")) r *= -1.0;  // do not use sign (deg) here, as deg may be zero!
            double sec = Double.parseDouble(tok.nextToken());
            r = (Math.abs(deg) + min / 60.0 + sec / 3600.);
            if (s.contains("-")) r *= -1.0;
            return r;
        } finally {
            return r;
        }
    }

    public void afterTextChanged(Editable s) {
        if (mode == MODE_MAN) {
            coordBuff = new GeoCoordinate(getDegInput(editLat), getDegInput(editLon), getDegInput(editAlt), searchBox.getText().toString());
            searchResultView.setText(makeLocString(coordBuff.getLat(), coordBuff.getLon(), coordBuff.getName()));
        }

    }


    /**
     * @param lat  in degrees
     * @param lon  in degrees
     * @param name as string
     * @return string like "Haltern\nBr: 51
     */
    public String makeLocString(double lat, double lon, String name) {
        if (lat > GeoRecord.ERROR_LAT) {
            return String.format
                    (name + LineOLatNL + AstroUtils.DMSStrg(Math.toRadians(lat), coord_form))
                    + String.format(LineOLon + AstroUtils.DMSStrg(Math.toRadians(lon), coord_form));
        } else return Loc_CurrentPos.replace("\n", "");
    }

    // Callback Handler
    Handler locDisplayHandler = new Handler() {
        public void handleMessage(Message msg) {
            Log.d(TAG, "locDisplayHandler Response: " + msg.what);
            SystemUtils.stopHourglass();
            switch (msg.what) {
                case RequestGeoCoder.MSG_ADDRESS:
                    Address address = (Address) msg.obj;
                    if (address != null) {
                        locName = RequestGeoCoder.getAddress(address, ", ");
                        resultIsGeoCoded = true;
                        if (coordBuff != null && coordBuff.name == "") {
                            coordBuff.countryName = address.getCountryName();
                            coordBuff.name = address.getLocality();
                        }
                        showCurrentLocation();
                    }
                    break;
                case RequestGeoCoder.MSG_COORDINATE:
                    coordBuff = (GeoCoordinate) msg.obj;
                    String locName = coordBuff.getName();
                    if (coordBuff.countryName != null && !coordBuff.countryName.equals("") && !coordBuff.countryName.equals(coordBuff.getName()))
                        locName += ", " + coordBuff.countryName;
                    searchResultView.setText(Loc_FoundHeader
                            + makeLocString(coordBuff.getLat(), coordBuff.getLon(), locName)
                            + Loc_FoundFooter);
                    useLastPosition = false;
                    resultIsGeoCoded = true;
                    coordBuff.isGeocoded = true;
                    break;
                case RequestGeoCoder.MSG_ERROR:
                    try {
                        searchResultView.setText(Loc_No_Service + Loc_FoundFooter);  // Error
                    } catch (RuntimeException e) {
                        e.printStackTrace();
                        Log.d("Location_Input_Class", "btnClick error: " + e.toString());
                    }

                case RequestGeoCoder.MSG_NO_ADDRESS:
                    searchResultView.setText(Loc_NotFound + Loc_FoundFooter);
                    break;
            }
        }
    };


    @SuppressLint("MissingPermission")
    protected void getCurrentLocation() {
        Criteria criteria = new Criteria();
        //location = null;
        //locName 		  = "";
        criteria.setAccuracy(Criteria.ACCURACY_FINE);
        Log.d(TAG, "getCurrentLocation " + locationManager.getBestProvider(criteria, true));
        try {
            location = locationManager.getLastKnownLocation(locationManager.getBestProvider(criteria, true));
            processLocation(location);
        } catch (Exception e) {
        }
    }

    protected void processLocation(Location loc) {
//		Criteria criteria = new Criteria ();
//		criteria.setAccuracy(Criteria.ACCURACY_FINE); 
        try {
            location = new Location(loc);
            Log.d(TAG, "processLocation " + location.getProvider());

            // Limit Geocoder Requests
            if (System.currentTimeMillis() - RequestGeoCoder.t0 > locationRequestRate) {
                locationRequestCounter++;
                if (locationRequestCounter % 10 == 0 && locationRequestRate < RequestGeoCoder.GOOGLE_RATE_MILLIS) {
                    locationRequestRate += 5000;
                }

                resultIsGeoCoded = false;
                @SuppressWarnings("unused")
                RequestGeoCoder rgc = new RequestGeoCoder(
                        this, location.getLatitude(), location.getLongitude(),
                        locDisplayHandler);
            }

            // set this coordinate to Coordbuff, if coordBuff is invalid
            if (coordBuff.getLat() == GeoRecord.ERROR_LAT || mode == MODE_GPS) {
                coordBuff = new GeoCoordinate(location);
                coordBuff.name = "";
                coordBuff.countryName = "";
            }
        } catch (RuntimeException e) {
        }
        showCurrentLocation();
    }

    protected void showCurrentLocation() {
        if (location != null) {
            // show valid location coordinates
            lastPositionView.setText(makeLocString(
                    location.getLatitude(), location.getLongitude(),
                    (location.getProvider().equals(LocationManager.GPS_PROVIDER) ?
                            Loc_actual_GPS : Loc_actual_NET)
                            + String.format(AstroUtils.locale, Accuracy, location.getAccuracy()))
                    + "\n");
            if (locName != "") {
                lastPositionView.append("\t\t("
                        + locName
                        //	        			+GeoUtils.getAddressForCoordinate(location.getLatitude(), location.getLongitude())
                        + ")");
            }
        } // location == null
        else lastPositionView.setText(Loc_CurrentPos);
    }

    /**
     * displays coordBuff in searchResultView
     *
     * @param header
     */
    protected void displayLocation(String header) {
        searchResultView.setVisibility(View.VISIBLE);
        searchResultView.setText(header
                        + makeLocString(coordBuff.getLat(), coordBuff.getLon(), coordBuff.getName() + ", " + coordBuff.countryName)
//				+Loc_FoundFooter
        );
        if (coordBuff.getAlt() > 0) searchResultView.append(
                "  " + String.format(getString(R.string.alt) + " %.0f" + getString(R.string.m), coordBuff.getAlt()));
        searchResultView.setVisibility(View.VISIBLE);
//		lastPositionView.setVisibility(View.GONE);
        searchBox.setText(coordBuff.getName());
//		gpsCheckBox.setChecked(false); 
    }


    /**
     * gets a GeoCoordinate from searchResult at pos index
     *
     * @param index
     * @return
     */
    public GeoCoordinate getGeoLocation(int index) {
//		Log.d(TAG, "getGeoLocation  " +index +":  " +searchResult.get(index));
        StringTokenizer st = new StringTokenizer(searchResult.get(index), ";");
        GeoCoordinate coordIn = new GeoCoordinate();
        coordIn.countryName = (st.nextToken()); // is state or Country or "FAV"
        boolean isFavorite = coordIn.countryName.equals(FAV_MARK) || coordIn.countryName.startsWith("*");
        if (isFavorite) coordIn.countryName = "";
        if (mode == MODE_MEX) coordIn.countryName = "Mexico"; // else will be "A" or similar
        if (mode == MODE_BRA) coordIn.countryName = "Brazil"; // else will be "A" or similar
        coordIn.name = (st.nextToken());
        if (mode >= MODE_WORLD && mode <= MODE_WORLD3
                && coordIn.name.contains(",")) {
            // add region name to country for world
            int p = coordIn.name.indexOf(",");
            coordIn.countryName += coordIn.name.substring(p);
            coordIn.name = coordIn.name.substring(0, p);
        }

        String value = "";
        if (st.hasMoreTokens()) {
            value = st.nextToken().trim();
        }
        boolean error = value.equals("");
        coordIn.setLat(AstroUtils.firstVal(value));
        if (st.hasMoreTokens()) {
            value = st.nextToken().trim();
        } else value = "";
        if (value.equals("")) error = true;
        coordIn.setLon(AstroUtils.firstVal(value));

        if (st.hasMoreTokens()) {// && isFavorite) {  // Read altitude
            coordIn.setAlt(AstroUtils.firstVal(st.nextToken()));
        }
        if (isFavorite && st.hasMoreTokens()) {// Read Olson TimeZoneID
            coordIn.timeZoneStrg = st.nextToken();
        }
        if (coordIn.isValid() && !error) {
            if (!coordIn.name.equals("")) coordIn.isGeocoded = true;
            coordBuff = coordIn;
            displayLocation(Loc_FoundHeader);
            useLastPosition = false;
        }
        return coordIn;
    }


    protected Thread myThread;
    protected ProgressDialog progDialog = null;

    //	protected static String threadFileName = "";
    protected static String threadSearch;
    protected static int threadReturnOption;

    protected Handler classHandler = new Handler() {
        public void handleMessage(Message msg) {
//			Log.d(EclipseApp.TAG, "Super MsgReceived: " +msg.what);
            if (progDialog != null &&
                    (msg.what != MSG_DownloadComplete
                            || threadReturnOption == DOWNLOAD_ONLY)) {
                progDialog.dismiss();
                progDialog = null;
            }

            switch (msg.what) {
                case DISP_SPINNER2:
                    buildSpinnerList(spinner2, spinMode2);
                    break;
                case DISP_SPINNER4:
                    if (searchResult.size() == 0) {
                        spinner4.setEnabled(false);
                        if (mode == MODE_FAV)
                            Toast.makeText(getApplicationContext(), getString(R.string.noFav), Toast.LENGTH_LONG).show();
                    } else buildSpinnerList(spinner4, spinMode4);
                    break;
                case ACTIVATE_SPINNER3:
                    if (searchResult.size() > 70) {
                        // display ABC if too much cities in selected state
                        buildSpinnerABC(spinner3, spinMode3);
                        if (spinMode4 > 0) {
                            readFileInThread(getState((String) spinner2.getSelectedItem())
                                    + ";" + spinner3.getItemAtPosition(spinMode3).toString(), DISP_SPINNER4);
                        }
                    } else { // Few cities in state
                        spinner3.setVisibility(View.GONE);
                        buildSpinnerList(spinner4, spinMode4);
                    }
                    break;
                case DISP_LIST:
                    // if one found display directly, else show list dialog to select
                    if (searchResult.size() > 1) {
                        removeDialog(DISP_LIST);
                        try {
                            showDialog(DISP_LIST);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    } else if (searchResult.size() == 1) {
                        GeoCoordinate gc = getGeoLocation(0);
                        //				Log.d(TAG, "Selected " +gc.isValid() +":  " +gc.name);
                        if (gc.isValid()) {
                            coordBuff = gc;
                            displayLocation(Loc_FoundHeader);
                        }
                    } else
                        Toast.makeText(getApplicationContext(), getString(R.string.Loc_NotFound).replace("\n", ""), Toast.LENGTH_LONG).show();
                    break;
                case MSG_NOTFOUND:
                    spinner4.setEnabled(false);
                    if (mode == MODE_FAV)
                        Toast.makeText(getApplicationContext(), getString(R.string.noFav), Toast.LENGTH_LONG).show();
                    break;
                case MSG_DownloadComplete:
                    Toast.makeText(getApplicationContext(), getString(R.string.downloadFinish), Toast.LENGTH_LONG).show();
                    break;
                case MSG_DownloadError:
                    Toast.makeText(getApplicationContext(), getString(R.string.downloadError), Toast.LENGTH_LONG).show();
                    break;
            }
        }
    };

    public Handler handler = classHandler;  // may be overwritten


    /**
     * @param fileName
     * @param search
     * @param returnOption
     */
    public void readFileInThread(String fileName, String search, int returnOption) {
        fileNames = new String[1];
        fileNames[0] = fileName;
        readFileInThread(search, returnOption);
    }

    public void readFileInThread(String search, int returnOption) {

        if (progDialog == null && returnOption != DOWNLOAD_SILENT)
            try {
                progDialog = new ProgressDialog(this);
                if (SystemUtils.fileExists(fileNames[0]) || mode == MODE_FAV) {
                    progDialog.setMessage(getString(R.string.wait));
                } else progDialog.setMessage(getString(R.string.downloadFile));
                progDialog.show();
                Log.d(TAG, "readFileInThread, progDialog.show()");
            } catch (Exception e) {
            }


//		threadFileName  	= fileName;
        threadSearch = search;
        threadReturnOption = returnOption;
        myThread = new Thread(myRunnable);
        myThread.start(); // file reading in separate thread
    }

    final Runnable myRunnable = new Runnable() {
        public void run() {
            Message m = new Message();
            boolean result = false;
            searchResult = new ArrayList<String>(); // clear last result
            for (int i = 0; i < fileNames.length; i++) {
                if (fileNames[i] != null && fileNames[i] != "") {
                    // first download file if necessary
                    if (!SystemUtils.fileExists(fileNames[i]) && mode != MODE_FAV) {
                        //				startDownload (threadFileName);
                        Log.d(TAG, "myRunnable: Start Download: " + fileURLDir + SystemUtils.stripFileName(fileNames[i]));
                        if (SystemUtils.downloadFromUrl(fileURLDir + SystemUtils.stripFileName(fileNames[i]),
                                fileNames[i])) {
                            Log.d(TAG, "myRunnable: Download complete");
                            m.what = MSG_DownloadComplete;
                        } else {
                            //					threadFileName = "";
                            Log.d(TAG, "myRunnable: Download error");
                            m.what = MSG_DownloadError;
                        }
                        if (threadReturnOption != DOWNLOAD_SILENT) handler.sendMessage(m);
                    }
                    if (threadReturnOption != DOWNLOAD_ONLY && threadReturnOption != DOWNLOAD_SILENT) {
                        m = new Message();
                        if (readFile(fileNames[i], threadSearch, false)) {
                            result = true;
                        }
                        //				threadFileName = ""; TODO
                    }
                }
            } // for
            if (threadReturnOption != DOWNLOAD_ONLY && threadReturnOption != DOWNLOAD_SILENT) {
                m = new Message();
                if (result) {
                    m.what = threadReturnOption;
                } else m.what = MSG_NOTFOUND;
                handler.sendMessage(m);
            }
        }
    };

    /**
     * Searches in file for SearchString
     * STATE;NAME;LATITUDE;LONGITUDE;ZONE;DST
     * example line:    "NY;New York;40.714275;-74.005978;-5;Y"
     * <p>
     * NAME;LATITUDE;LONGITUDE;ALT;ZONE
     * examplefavorite: "New York;40.714275;-74.005978;0.0;America/New_York"
     *
     * @param fileName
     * @param search
     * @param resetSearchResult
     * @return
     */
    public boolean readFile(String fileName, String search, boolean resetSearchResult) {
        return readFile(fileName, search, searchResult, resetSearchResult);
    }

    public boolean readFile(String fileName, String search, List<String> searchResult, boolean resetSearchResult) {
        BufferedReader in;
        boolean result = false;
        Log.d(TAG, "readFile: " + fileName + "  Search: " + search);
        if (fileName != null && !fileName.equals("")) try {
            if (resetSearchResult) searchResult = new ArrayList<String>();
            in = new BufferedReader(new FileReader(fileName));
            String reader = "";
            Collator usCollator = Collator.getInstance(Locale.US); // Collator to ensure finding diacritics
            usCollator.setStrength(Collator.PRIMARY); //
            while ((reader = in.readLine()) != null) {
                reader = reader.trim();
                if (fileName.equals(LOCATION_FILE_NAME))
                    reader = FAV_MARK + ";" + reader;// does not contain a country or ABC list
                // Get the Collator for US English and set its strength to PRIMARY
//					 Log.d(TAG, "search: " +search +",  readerSub: " +reader.substring(0, search.length()));
//					 Log.d(TAG, reader + ": " +usCollator.compare( reader.substring(0, search.length()), search));

                if (!reader.equals("") &&
                        (search.length() == 0  // no search criterion, return all reader records
                                || (search.length() == 1 && usCollator.compare(reader.substring(0, search.length()), search) == 0) // ABC list
                                || (search.contains(";") && usCollator.compare(reader.substring(0, search.length()), search) == 0) // eg: US, Canada: "FL;N"
                                || usCollator.compare(reader.substring(reader.indexOf(";") + 1, reader.indexOf(";") + 1 + search.length()), search) == 0
                        )) {
                    searchResult.add(reader);
                }
            } // while
            result = true;
            in.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            Log.d(TAG, "readFile: FileNotFound Exception: " + fileName);
            return false;
        } catch (IOException e) {
            e.printStackTrace();
            Log.d(TAG, "readFile: IO Exception");
            return false;
        } catch (Exception e) {
            e.printStackTrace();
            Log.d(TAG, "readFile: general Exception");
            return false;
        }
        Collections.sort(searchResult);
        Log.d(TAG, "readFile, found: " + searchResult.size());
        return result;
    } // readFile


    protected void startDownload(String fileName) {
        Log.d(TAG, "LocInpCl.startDownload: " + fileName);
        readFileInThread(fileName, "", DOWNLOAD_SILENT);
    }


    @Override
    public Dialog onCreateDialog(int id) {
        switch (id) {
            case DISP_LIST:

                for (int i = 0; i < searchResult.size(); i++) {
                    if (searchResult.get(i).startsWith(FAV_MARK))
                        searchResult.set(i, "*" + searchResult.get(i).substring(1));
                }
                Collections.sort(searchResult);


                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                ArrayAdapter<String> adapter = new ArrayAdapter<String>(
                        this, android.R.layout.simple_spinner_item);

                // add searchResult list to adapter
                for (int i = 0; i < searchResult.size(); i++) {
                    // display only  city name, not coordinates etc
                    // example line:    "NY;New York;-74.005978;40.714275;-5;Y"
                    try {
                        String item = searchResult.get(i);
                        if (!item.equals("") && item.contains(";")) {
                            boolean isFavorite = item.startsWith("*");
                            String country = item.substring(0, item.indexOf(";"));
                            // get city from item between the two semicolons
                            item = item.substring(item.indexOf(";") + 1);
                            item = item.substring(0, item.indexOf(";"));

                            // add a star to item if isFavorite, otherwise add country
                            if (isFavorite) {
                                item = "* " + item;
                            } else item = "   " + item + ", " + country;
                            adapter.add(item);
                        }
                    } catch (Exception e) {
                        Log.d(TAG, "Error SearchResult: " + searchResult.get(i));
                    }  // wrong internal structure of record
                }
                builder.setTitle(R.string.nothingSelected)
                        .setAdapter(adapter, new DialogInterface.OnClickListener() {
                            //				.setSingleChoiceItems (adapter, -1, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                // The 'which' argument contains the index position
                                // of the selected item
                                GeoCoordinate gc = getGeoLocation(which);
                                Log.d(TAG, "Selected " + gc.isValid() + ":  " + gc.name);
                                if (gc.isValid()) {
                                    coordBuff = gc;
                                    displayLocation("");
                                }
//						Toast.makeText(getApplicationContext(), "Selected: \n"+searchResult.get(which), Toast.LENGTH_LONG).show();
                            }
                        });
                return builder.create();

            case DOWNLOAD_FILE:
                builder = new AlertDialog.Builder(this);
                builder.setMessage(String.format(getString(R.string.needsDownload), fileSizeString))
                        .setCancelable(false)
                        .setPositiveButton(R.string.OK, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                mode = requestedMode;
                                setMode(requestedMode);
                            }
                        })
                        .setNegativeButton(R.string.no, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                spinner1.setSelection(modeToSpinner1[oldMode]);
                            }
                        });
                return builder.create();

            case MENU_MANAGE:
                final List<String> favList = new ArrayList<String>();    // Copy of File
                readFile(LOCATION_FILE_NAME, "", favList, false);
                final ArrayList<String> favs = new ArrayList<String>();  // List of names
                final ArrayList<String> mSelectedItems = new ArrayList<String>(); // copy of final file after edit
                for (int i = 0; i < favList.size(); i++) {
                    // add searchResult list to adapter
                    // display get only  city name, not coordinates etc
                    // example line:    "NY;New York;-74.005978;40.714275;-5;Y"
                    try {
                        String item = favList.get(i);
                        if (!item.equals("") && item.contains(";")) {
                            item = item.substring(item.indexOf(";") + 1);
                            item = item.substring(0, item.indexOf(";"));
                            favs.add(item);
                            mSelectedItems.add(favList.get(i));
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }  // wrong internal structure of record
                }
                Log.d(TAG, "  favList: " + favList.size() + " favs " + favs.size());
                builder = new AlertDialog.Builder(this);
                builder.setTitle(getString(R.string.deleteFavorites))
                        .setCancelable(true)
                        .setMultiChoiceItems(favs.toArray(new String[favs.size()]), null,
                                new DialogInterface.OnMultiChoiceClickListener() {

                                    public void onClick(DialogInterface dialog, int which,
                                                        boolean isChecked) {
                                        if (isChecked) {
                                            // If the user checked the item to delete, remove it from the selected items
                                            mSelectedItems.remove(favList.get(which));
                                            Log.d(TAG, "delete, Sum:" + mSelectedItems.size() + ": " + Integer.valueOf(which) + "  " + favList.get(which));
                                        } else if (!mSelectedItems.contains(favList.get(which))) {
                                            // Else, if the item is not in the array, add it
                                            mSelectedItems.add(favList.get(which));
                                            Log.d(TAG, "add, Sum:" + mSelectedItems.size() + ": " + favList.get(which));
                                        }
                                    }
                                })
                        .setPositiveButton(R.string.OK, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                // User clicked OK, so save the mSelectedItems results somewhere
                                // or return them to the component that opened the dialog
                                if (mSelectedItems.size() != favList.size()) {
                                    SystemUtils.bakFile(LOCATION_FILE_NAME);
                                    try {
                                        File file = new File(LOCATION_FILE_NAME);
                                        FileOutputStream fOut = new FileOutputStream(file);
                                        OutputStreamWriter osw = new OutputStreamWriter(fOut);
                                        for (int i = 0; i < mSelectedItems.size(); i++) {
                                            osw.write(mSelectedItems.get(i).substring(FAV_MARK.length() + 1) + "\r\n");
                                        }
                                        osw.flush();
                                        osw.close();
                                        Toast.makeText(getApplicationContext(), "" + (favList.size() - mSelectedItems.size())
                                                + getString(R.string.deletedFavorites), Toast.LENGTH_SHORT).show();
                                        if (mode == MODE_FAV)
                                            setMode(MODE_FAV); // force reload of List
                                    } catch (Exception e) {
                                        e.printStackTrace();
                                    }
                                }
                            }
                        });

                return builder.create();

            default:
                return null;
        }
    }


    public void beforeTextChanged(CharSequence s, int start, int count,
                                  int after) {
    }

    public void onTextChanged(CharSequence s, int start, int before, int count) {
    }


    //@Override
    public void onLocationChanged(Location location) {
        //getCurrentLocation ();
        Log.d(TAG, String.format(Locale.US, "Loc_I_C onLocationChanged: %s +%7.4f deg %7.4f deg", location.getProvider(), location.getLatitude(), location.getLongitude()));
        processLocation(location);
    }

    //@Override
    @SuppressLint("MissingPermission")
    public void onProviderDisabled(String provider) {
        Log.d(TAG, "Loc_I_C onProviderDisabled: " + provider);
        try {
            criteria.setAccuracy(Criteria.ACCURACY_COARSE);  // Faster, no GPS fix. Acc WLAN ca 70 m, CellID 2000 m
            locationManager.requestLocationUpdates(locationManager.getBestProvider(criteria, true),
                    MIN_GPS_LOCATION_UPDATE_TIME, MIN_GPS_LOCATION_UPDATE_DISTANCE, this);

        } catch (Exception e) {
        }
    }

    //@Override
    @SuppressLint("MissingPermission")
    public void onProviderEnabled(String provider) {
        Log.d(TAG, "Loc_I_C onProviderEnabled: " + provider);
        try {
            criteria.setAccuracy(Criteria.ACCURACY_FINE);    // More accurate, GPS fix.
            locationManager.requestLocationUpdates(locationManager.getBestProvider(criteria, true),
                    MIN_GPS_LOCATION_UPDATE_TIME, MIN_GPS_LOCATION_UPDATE_DISTANCE, this);
        } catch (Exception e) {
        }
    }

    //@Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
    }

} // public class Location_Input_Class

