package strickling.forms;

/** Class extension for usage of Google Maps in Location_Input
 *
 * dependencies in app build.gradle:
 *   android
 *

       useLibrary 'org.apache.http.legacy'  // for osmbonuspack

 *     sourceSets {

 main.java.srcDirs += 'D:\\Users\\Wolfgang\\Documents\\Android\\OSM_BonusPack'

 * ....

 * dependencies:
 implementation "com.android.support:support-v4:27.1.1"
 implementation 'org.apache.commons:commons-lang3:3.+'
 implementation 'com.google.code.gson:gson:2.8.0'

 * requirements in manifest:
 <activity
 android:name="strickling.forms.Osm_Loc_Select"
 android:label="@string/location_name"
 />

 *
 */

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 


import java.util.Locale;

import strickling.utils.GeoCoordinate;

//import com.google.android.gms.maps.model.LatLng;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;

import org.osmdroid.compatibility.LatLng;

public class Location_Input_OSM extends Location_Input_Class {
	
	
	public static int 	[]		spinner1ItemsMap = {MODE_GPS, MODE_FAV, MODE_MAP, MODE_WORLD, MODE_WORLD2, MODE_WORLD3, MODE_SEARCH, MODE_MAN};
	public static boolean 		useDefaultMapSpinner1 	= true; // May be overwritten to use other spinner config	
	public static boolean		useOSM					= true; // May be overwritten 
	
	public void onCreate(Bundle savedInstanceState) {
		if (useDefaultMapSpinner1) spinner1Items = spinner1ItemsMap;
		super.onCreate(savedInstanceState);
		org.osmdroid.tileprovider.constants.OpenStreetMapTileProviderConstants.setOsmPath (this);
	}	
	
	/** Sets location input mode                     TODO mark
	 * 
	 * @param mode
	 */
	@Override
	public void setMode (int mode){
//		Log.d(TAG, "Location_Input_Map.setMode: " +mode);
		super.setMode (mode);
		switch (mode) {
		case MODE_MAP:
			spinner2.setVisibility(View.GONE);
			spinner3.setVisibility(View.GONE);
			spinner4.setVisibility(View.GONE);
			hideSearchBtns ();
			if (! blockSpinnerSel) { // launch Map, if not blocked
				getSharedPreferences("LocPreferences", 0).edit().putBoolean("followGPS", useLastPosition).commit();
				Osm_Loc_Select.selLocation = new LatLng(coordBuff.getLat(), coordBuff.getLon());
				Osm_Loc_Select.locName	  = coordBuff.getNameAndCountry();
				//    		EclipseMap.coord_form  = Local_Circumstances.COORD_FORM[dmsMode];
				Intent mapIntent; 
//				if (useOSM) {mapIntent= new Intent(this, Osm_Loc_Select.class);}
//				else
					mapIntent= new Intent(this, Osm_Loc_Select.class);
				putIntentExtras(mapIntent);
				startActivityForResult (mapIntent, MODE_MAP);
			}
			break;		
		}
	}
	
	
	// called on returning from map activity
	protected void onActivityResult(int requestCode, int resultCode, Intent data){
		Log.d(TAG, "onActivityResult, requestCode = " +requestCode +",  resultCode = " +resultCode);
		switch (requestCode){
		case MODE_MAP:
			Log.d(TAG, "on ActivityResult, ok: " +(resultCode == Activity.RESULT_OK));
			if (resultCode == Activity.RESULT_OK) {
				useLastPosition = data.getBooleanExtra(USE_GPS, useLastPosition);
				coordBuff    = new GeoCoordinate (data.getDoubleExtra(LAT, 0), data.getDoubleExtra(LON, 0), 
						data.getDoubleExtra(ALT, 0), data.getStringExtra(NAME));		
				coordBuff.name 		= data.getStringExtra(Location_Input_Class.NAME);
				coordBuff.countryName 	= data.getStringExtra(Location_Input_Class.COUNTRY);	

				displayLocation ("");
				searchBox.setText(coordBuff.getName());
				Log.d(TAG, String.format(Locale.US, "gotIntentExtras, Name: %s, %.3f deg, %.3f deg", coordBuff.getName(), coordBuff.getLat(), coordBuff.getLon()));
			}
		}
	}



}
