package strickling.utils;

/* Bugs in AstroLib 1.1.4:
 * 
 * - Zeitzone wird nicht for Auf-/Untergaenge beruecksichtigt
 * 
 * - Methoden fuer Ausgabe ekliptikaler und Azimutaler Koordinaten 
 *   greifen auf Kartesische Elemente zurueck, Konversion auf Polarkoord. noetig!
 *   
 * - Konversion heliozentrische auf geozent. Koord. fehlt
 * 
 * - Sonnenkoor. sind geozentrisch, nicht topozentr. 
 * 
 * - Plutokoordinaten sinnlos
 * 
 * -  Koord. geozentrisch aeq. des Tages, TDT, ohne Nutation, ohne Lichtlaufzeit, ohne Aberration,  
 * 
 */



import java.util.Locale;
import java.util.StringTokenizer;

import android.util.Log;
import android.widget.TextView;

//import com.mhuss.AstroLib.MathOps;
//import com.mhuss.AstroLib.NoInitException;
//import com.mhuss.AstroLib.PlanetData;

public class AstroUtils {

	public static final double TWO_PI  = Math.PI *2.0;
	public static final double HALF_PI = Math.PI /2.0;
	public static final double RADFAKT = Math.PI /180.0;
	public static final double DEGFAKT = 180.0/Math.PI;

	public static final double AE           = 149597870.691;  // km
	public static final double ERDAEQ       = 6378.137;     // km aequatorradius WGS84
	public static final double ERDPOL       = 6356.7523;    // km Polradius WGS84     
	public static final double R_EARTH      = (ERDAEQ +ERDPOL)/2.0; //6368;         // km mittl Radius
	public static final double R_MOON       = 1735.0;       // km
	public static final double R_SUN        = 695500.0;     // Sonnenradius in km
	public static final double LUNATIONTIME = 29.530588861; // Synodischer Monat in Tagen
	public static final double MONDENTF     = 380000.0;     // km
	public static final double LIGHTSPEED   = 299792.458;   // km/s
	public static final double TLIGHT       = 0.577551831e-2;    // Tage pro AE  , 1/c
	public static final double KGAUSS       = 0.01720209895; // Gausssche Konstante fuer Ber. Mittl. Anomalie
	public static final double JD1950       = 2433282.423;   //JulDat von B1950.0
	public static final double JD2000       = 2451545.0  ;   //JulDat von J2000.0
	public static final double EPS2000      = 23.439291111;  // Ekliptikschiefe 2000.0 in Grad                           

	public static final String TAG 			= "AstroUtils";
	public static Locale	   locale	    = Locale.getDefault(); // may be overwritten for localization
	
	// returns angle 0..2*PI (Longitude Format)
	public static double longAngle (double angle) {
		while (angle < 0) angle += TWO_PI;
		while (angle > TWO_PI) angle -= TWO_PI;
		return angle;
	}

	// returns angle -PI..PI (Latitude Format)
	public static double latAngle (double angle) {
		while (angle < Math.PI) angle += TWO_PI;
		while (angle > Math.PI) angle -= TWO_PI;
		return angle;
	}

	// returns angle -PI/2..+PI/2 (strict Latitude Format)
	public static double latAnglePi2 (double angle) {
		while (angle < -Math.PI) angle += TWO_PI;
		while (angle >  Math.PI) angle -= TWO_PI;
		if (angle >  HALF_PI) angle  =  Math.PI -angle;
		if (angle < -HALF_PI) angle  = -Math.PI -angle;
		return angle;
	}

	/**	returns String like "-5 deg 45' 10" or "-5.1234 deg" from degree angle, 
	 * 
	 * @param angleDeg in degrees
	 * @param format eg. "dms0", "dm1", "d4" (last number is decimals) 
	 * @return String
	 */
	public static String DMSStrgDeg (double angleDeg, String format) {
		return DMSStrg(angleDeg*RADFAKT, format);
	}
	
	/**	returns String like "-5 deg 45' 10" or "-5.1234 deg" from radians angle, 
	 * 
	 * @param angleRad in radians
	 * @param format eg. "dms0", "dm1", "d4" (last number is decimals) 
	 * @return String
	 */
	public static String DMSStrg (double angleRad, String format) {
		String sign;
		if (angleRad >= 0) {
			sign = "+"; 
			if (angleRad >= Math.toRadians(100)) sign = ""; 
		}
		else sign = "-";		
		return sign +degreeStrg (Math.abs(angleRad), "\u00B0", "'", "\"", format); // degree symbol = "\u00B0"
	}
	
	/**
	 * @param angle in radians, for RA and HourAngle
	 * @param format eg "dms0", "dm1", "d4" number is decimals
	 * @return  String like "-5 deg 45' 10" or "-5.1234 deg",  
	 */
	public static String HMSStrg (double angle, String format) {
		return degreeStrg (angle/15.0, "h", "m", "s", format);
	}
	
	/**
	 * @param angle in radians, for RA and HourAngle
	 * @param format eg "dms0", "dm1", "d4" number is decimals
	 * @param u1 , u2, u3 eg. " deg", "'", "\""    or "h", "m", "s" 
	 * @return  String like "-5 deg 45' 10"       or "-5.1234 deg",  
	 */
	public static String degreeStrg (double angle, String u1, String u2, String u3, String format) {
		double d, m, s, roundFact;
		double angleDeg = Math.abs(Math.toDegrees(angle));
		
		// avoid western leading zeros in eastern Arabic numbers 
		String leadingZero = " %0";
		if (locale.getLanguage().startsWith("ar")) leadingZero = " %";
		
		//Rounding, to avoid results like "10 deg 60'"
		if (format.startsWith("dms")) {
			//double z = format.charAt(format.length()-1) -0x30;
			roundFact = 3600.0 *Math.pow (10, format.charAt(format.length()-1)-0x30); 
			angleDeg = Math.round (angleDeg*roundFact)/roundFact +0.1/roundFact;
		//	Log.d(TAG, "dms roundFact: " +format.charAt(format.length()-1) +"  "+roundFact);
		}
		else if (format.startsWith("dm")) {
			roundFact = 60.0 *Math.pow (10, format.charAt(format.length()-1)-0x30); 
			angleDeg = Math.round (angleDeg*roundFact)/roundFact +0.1/roundFact;			
		}

        // displaying values
		int decimals = format.charAt(format.length()-1) -48;
		int numLen   = (decimals == 0) ? 2 : 3+decimals; 
		if (format.startsWith("dm")) {
			d = Math.signum(angle) *Math.floor (angleDeg);
			m = frac (angleDeg)*60;
		
			if (format.startsWith("dms")) {
				s = frac (m)*60;
				m = m-s/60;
				return String.format(locale, "%2.0f" +u1 +leadingZero +"2.0f" +u2 +leadingZero +numLen +"." +decimals+"f"+u3, d, m, s);
			}
			else return String.format(locale, "%2.0f" +u1 +leadingZero +numLen +"." +decimals+"f" +u2, d, m); 
		}
		return String.format(locale, "%" +numLen +"." +decimals+"f" +u1 , Math.toDegrees(angle));
	}
	
	/** decimal fraction of number
	 * 
	 * @param number
	 * @return decimal fraction of number
	 */
	public static double frac (double number) {
		return number - Math.floor(number);
	}
	
	/** decimal fraction of number
	 * 
	 * @param number
	 * @return decimal fraction of number
	 */
	public static float frac (float number) {
		return (float) (number - Math.floor(number));
	}
	
	/**
	 * returns square of number
	 */
	public static float sqr (float number) {
		return number * number;
	}
	
	/**
	 * returns square of number
	 */
	public static double sqr (double number) {
		return (number * number);
	}
	
	/**	 floating decimal Modulo |  like integer: zahl % teiler for double
	 * 
	 * @param zahl
	 * @param teiler
	 * @return floating decimal Modulo
	 */
	public static double realMod (double zahl, double teiler) {
		return (teiler != 0) ? teiler * frac(zahl/teiler): 0;
	}
	
	/**  round to floating point divisor
	 * 
	 * @param zahl
	 * @param teiler
	 * @return
	 */
	public static double roundFloat (double zahl, double teiler) {
		return (teiler != 0) ? Math.round(zahl/teiler)*teiler: zahl;
	}
	
	public static double RadDeg (double radians){
		return radians *DEGFAKT;
	}
	
	public static double degRad (double degrees){
		return degrees *RADFAKT;
	}
	
	public static String stripHTML (String s) {
		s = s.replaceAll("<P>","\n");
		s = s.replaceAll("<p>","\n");
		s = s.replaceAll("<BR>","\n");
		s = s.replaceAll("<br>","\n");
		s = s.replaceAll("\\<.*?>","");  // Strip html Tags
		s = s.replaceAll("&nbsp;"," ");
		s = s.replaceAll("&amp;","");
		s.trim();
		return s;
	}
	
	/** Extracts Numeric value from string, surrounded by words, Delimiter is "="!
	 * 
	 * @param s
	 * @return
	 */
	public static double firstVal (String s) {
		StringTokenizer ST = new StringTokenizer (s, " =");
		double temp = 0;
		boolean success = false;
		while (ST.hasMoreElements() && !success){
			try {
				temp = Double.parseDouble (ST.nextToken());
				success = true;
				//Das war die Zahl
			} catch (Exception E){
				//Das war sie nicht
			}
		}
		return temp;
	}

	
	
    /**  Catches numeric input, returns 0 in case of errors */
	public static double getNumeric (TextView tv) {
    	return getNumeric (tv.getText().toString());
    }
    	
    @SuppressWarnings({ "finally" })
    public static double getNumeric (String s) {
    	double r = 0;
    	if (s.contains("/")) {
    		double p1 = getNumeric (s.substring(0, s.indexOf("/")));
    		double p2 = getNumeric (s.substring(s.indexOf("/")+1));
    		if (p2 != 0) {return p1/p2;} else return p1;
    	}

    	s = s.replaceAll( "[^\\d.,+-]", " " ).trim();
    	if (s.contains(" ")) s = s.substring(0, s.indexOf(" "));
    	s = s.replace(",", ".");      // avoids problems with decimal comma (e.g. German language)
    	s = s.replace("+", "");
    	try {
    	r = Double.valueOf (s);
    	return r;
    	}
    	finally {return r;}
    }
    
    /**  Catches numeric input, returns 0 in case of errors */
    public static int getNumericInt (TextView tv) {
    	return (int) getNumeric (tv);
    }

    public static int getNumericInt (String s) {
    	return (int) getNumeric (s);
    }

	
	/**ermittelt den Winkelabstand, ggf ueber 2Pi (360 Grad) hinweg
	 * 
	 * @param l1
	 * @param l2
	 * @return in Radians, range 0...pi
	 */
	double Winkelabstand (double l1, double l2) {
	//  {ermittelt den Winkelabstand, ggf ueber 2Pi (360 Grad) hinweg}
	//  {radians! range 0...pi}
		double dL = Math.abs(l1-l2);
		while (dL > Math.PI) dL = Math.abs (2*Math.PI -dL);
		return dL;
	} // {of Winkelabstand ***********************************}
	
	
	/**gets angle of l2 respective to l1
	 * 
	 * @param l1 radians
	 * @param l2 radians
	 * @return in Radians, range -pi..pi
	 */
	public static double dAngle (double l1, double l2) {
		double dL = l2-l1;
		while (dL >  Math.PI) dL -= 2*Math.PI;
		while (dL < -Math.PI) dL += 2*Math.PI;
		return dL;
	} // dAngle
	
	/**gets angle of l2 respective to l1
	 * 
	 * @param l1 degrees
	 * @param l2 degrees
	 * @return in degrees, range -180..180
	 */
	public static double dAngleDeg (double l1, double l2) {
		double dL = l2-l1;
		while (dL >  180) dL -= 360;
		while (dL < -180) dL += 360;
		return dL;
	} // dAngle
	
	
	 /** True mathematic calculation of angular separation. All in radians
     * 
     * @param l1
     * @param b1
     * @param l2
     * @param b2
     * @return separation [0..PI], positionAngle [0..2PI]
     */
    public static AnglePair angularSeparation (double l1, double b1, double l2, double b2) {    	

    	// subtract l1
    	double [] lbr = new double [3];
    	lbr [0] = l2-l1;    	lbr [1] = b2;     	lbr [2] = 1.0;
    	double [] xyz = new double [3];    	
    	AstroUtils.polarToCartesian3 (lbr, xyz);
    	
    	// rotate x, z by b1
    	double [] xz = new double [2];
    	xz [0] = xyz [0];
    	xz [1] = xyz [2];
    	double [] lr = new double[2];
    	AstroUtils.cartesianToPolar2(xz, lr);
    	lr [0] -= b1;    	
    	
    	// convert into Cartesian, copy into xyz, reconvert into polars
    	polarToCartesian2(lr, xz);
    	xyz [0] = xz[0];  xyz [2] = xz[1];
    	cartesianToPolar3(xyz, lbr);    	
    	//now lbr[0], lbr[1] represent coordinates of Point2 respective ly to Point1
    	
    	// do spherical transformation cos(c) = cos (a) cos (b) 
    	double separation    = Math.acos (Math.cos(lbr[0]) *Math.cos(lbr[1]));
    	//separation = Math.acos (Math.sin(b1)*Math.sin(b2) +Math.cos(b1)*Math.cos(b2)*Math.cos(l1-l2)); // Meeus 16.1  p. 105
    	double positionAngle;
    	if (separation == 0) {positionAngle = 0;}
    	// do spherical transformation sin (beta) = sin (b) / sin (c)
    	else positionAngle = Math.PI/2.0 -Math.asin (Math.sin(lbr[1])/Math.sin(separation));
    	if (AstroUtils.latAngle(lbr[0]) <0) {positionAngle =Math.PI*2.0 -positionAngle;} 
    	return new AnglePair (separation, longAngle(positionAngle));
    }
	
	
	/**
	 * Convert Cartesian coordinates to Polar coordinates. By W. Strickling
	 *
	 * @param x
	 * @param y
	 * @param z
	 * @param lbr  double [3]   Vector to hold the Polar coordinates
	 */
	public static void cartesianToPolar3(double x, double y, double z, double [] lbr)
	{
		lbr[0] = Math.atan2 (y, x); // lon
		lbr[1] = Math.atan2 (z, Math.sqrt(x*x +y*y));    // lat
		lbr[2] = Math.sqrt(x*x +y*y +z*z); // rad
	}

	/**
	 * Convert Cartesian coordinates to Polar coordinates. By W. Strickling
	 *
	 * @param xyz double [3]   Vector to hold the Cartesian coordinates
	 * @param lbr double [3]   Vector to hold the Polar coordinates
	 */
	public static void cartesianToPolar3(double xyz [], double lbr [])
	{
		cartesianToPolar3(xyz[0], xyz[1], xyz[2], lbr);
	}

	public static void cartesianToPolar2(double xy [], double angleRad [])
	{
		angleRad [0] = cartesianToPolarLon (xy[0], xy[1]);
		angleRad [1] = Math.sqrt(xy[0]*xy[0] + xy[1]*xy[1]);
	}

	public static double cartesianToPolarLon(double x, double y)
	{
		return Math.atan2(y, x); // lon
	}

	public static double cartesianToPolarLat(double x, double y, double z)
		{
		return Math.atan (z / Math.sqrt(x*x +y*y));    // lat
	}
	
	public static void polarToCartesian3 (double lon, double lat, double rad, double xyz []) {
		double lbr [] = new double [3];
		lbr [0] = lon;
		lbr [1] = lat;
		lbr [2] = rad;
		polarToCartesian3 (lbr, xyz);
	}
	public static void polarToCartesian3 (double lbr [], double xyz [])
	{
		xyz[0] = lbr [2] *Math.cos(lbr[1]) *Math.cos(lbr[0]);
		xyz[1] = lbr [2] *Math.cos(lbr[1]) *Math.sin(lbr[0]);
		xyz[2] = lbr [2] *Math.sin(lbr[1]);
		
	}
	
	public static void polarToCartesian2 (double angleRad [], double xy [])
	{ 
		xy[0] = angleRad [1] *Math.cos(angleRad[0]);
		xy[1] = angleRad [1] *Math.sin(angleRad[0]);
	}

	
	/** return phase angle in radians
	 * 
	 * @param entf  geocentric distance of planet 
	 * @param rPlan heliocentric distance of planet
	 * @param rSun  geocentric distance of the sun 
	 * @param elong elongation of the planet
	 * @return phase angle in radians
	 */
	

	

	/** Azimut of rise, compute AzimutOfSet = TWO_Pi -azimutOfRise
	 * 
	 * @param de       declination in radians
	 * @param latitude Latitude of observer in radians
	 * @return  Azimut of Rise in radians, if no rise then return -999
	 */
	public static double azimutOfRise (double de, double latitude) {
		//	  {Azimut des Aufgangs ist true, wenn Auf- oder Untergang. ALLES Bogenmass}
		//	  {azSet = TWO_PI -azRise}
		double azRise = -999;
		double brW = Math.sin (HALF_PI -latitude);
		if (brW != 0) {
			double quot = Math.sin (de) / brW;
			if (Math.abs (quot) <= 1.0) {
				azRise = HALF_PI - Math.asin (quot);
			}
		}
		return azRise;
	}	
	
	/** converts ecliptical to equatorial coordinates
	 * 
	 * @param eclLEclB  in radians, ecl longitue, ecl. latitude
	 * @param obliqRad  obliquity in radians  {double obliquity = AstroOps.meanObliquity( AstroOps.toMillenia(julDat));} 
	 * @param raDe
	 */
	public static void eclipticalToEquatorial (double [] eclLEclB, double obliqRad, double [] raDe) {
		//procedure EklAeqDat (eklL, eklB: hpNumTyp;  var RA, DE: hpNumTyp; AequinDatum: JDTyp);
		  //  Umwandlung ekliptikal (eklL, eklB) > aequatorial (RA, DE) mit Ekliptikschiefe vom Datum
		  //  ALLE Werte in Bogenmass!!
		  raDe [1] = Math.asin( Math.sin(eclLEclB [1])*Math.cos(obliqRad) +Math.cos(eclLEclB [1])*Math.sin(obliqRad)*Math.sin(eclLEclB [0]) );
		  double a = -Math.sin(eclLEclB [1])*Math.sin(obliqRad) +Math.cos(eclLEclB [1])*Math.cos(obliqRad)*Math.sin(eclLEclB [0]);
		  double b = Math.cos(raDe [1]) +Math.cos(eclLEclB [1])*Math.cos(eclLEclB [0]);
		  if (b == 0) { raDe [0] = Math.PI;} 
		  else raDe [0] = 2.* Math.atan (a/b);
	}
	
	
	/**  equatorialToAzimutal from hour angle. 0 = South! Pi/2 = West!
	 * 
	 * @param hrAng_DE    double [2] or double [3] radians,
	 * @param latitudeRad 
	 * @param azimElev    double [2] or double [3], radius will not be transferred
	 */
	public static void equatorialToAzimutal (double [] hrAng_DE, double latitudeRad, double [] azimElev) {
	   	// Umwandlung Aequatorial -Azimutal: Wepner S. 159
    	// Elevation
		azimElev [1] = Math.asin (Math.sin (latitudeRad) * Math.sin (hrAng_DE [1])
				+ Math.cos (latitudeRad) * Math.cos (hrAng_DE [1]) * Math.cos (hrAng_DE [0]));

		// Azimut
		azimElev [0] = 2.0 * Math.atan ( (Math.cos (azimElev [1]) +Math.cos (latitudeRad) * Math.sin (hrAng_DE [1])
				- Math.sin (latitudeRad) * Math.cos (hrAng_DE [1]) * Math.cos (hrAng_DE [0]))
				/ (Math.cos (hrAng_DE [1]) *Math.sin (hrAng_DE [0])) );
		while (azimElev [0] < 0.0) { azimElev [0] += 2.0*Math.PI;}
	}
	
	
	/** Sternzeit returns local Sidereal Time in Hours
	 * 
	 * @param julDat Julian Date
	 * @param longitude_degrees 
	 * @return local Sidereal Time in Hours
	 */
	public static double sternzeit (double julDat, double longitude_degrees) {
		  return realMod (6.604441111 +0.06570982237*(julDat-2443873.5)
		                +longitude_degrees/15 +24*frac(julDat +0.50001)  , 24.0); // Wepner S. 99
//		return (RadDeg (AstroOps.greenwichSiderealTime (julDat)) +longitude_degrees)/15.0;
	}// of Sternzeit *******************************************************
		
	
	public static void topozReduktion (double raDeR [], // Ra, De, Entf // Bogenmass, AE
    		double julDat,
    		double lon, double lat)  // degrees!
		//Reduces Koordinates do Topocentric
	{
		//double x, y, z, x2, y2, z2;
	    double tmpVec1[] = new double[3];
	    double tmpVec2[] = new double[3];
//	    MathOps.polarToCartesian (tmpVec1, args);
//	    MathOps.polarToCartesian (tmpVec2,
//				Math.toRadians(Sternzeit(julDat, lon)*15.0), Math.toRadians(lat), R_EARTH/AE);  
	    
	    polarToCartesian3 (raDeR, tmpVec1);
	    polarToCartesian3 (Math.toRadians(sternzeit(julDat, lon)*15.0), Math.toRadians(lat), R_EARTH/AE, tmpVec2);  
	    tmpVec1 [0] -= tmpVec2 [0];
	    tmpVec1 [1] -= tmpVec2 [1];
	    tmpVec1 [2] -= tmpVec2 [2];
		cartesianToPolar3  (tmpVec1, raDeR);
	} //TopozReduktion **************************************
	

	/** reduction ecliptical coordinates Approved with SOLEX
	 * 
	 * @param lbr0 old equinoxe
	 * @param lbr  new equinoxe
	 * @param JD0  julian date of old equ
	 * @param JD   julian date of new equ
	 */
	public static void redEcl (double [] lbr0, double [] lbr, double JD0, double JD) {
		AnglePair lb = new AnglePair ();
		lb = redEcl (lbr0[0], lbr0[1], JD0, JD);
		lbr [0] = lb.lon;
		lbr [1] = lb.lat;
		lbr [2] = lbr0[2];
	}
	
	
	/** reduction ecliptical coordinates 
	 * 
	 * @param eklL  old equinoxe radians
	 * @param eklB  old equinoxe radians
	 * @param JD0   date old equinoxe
	 * @param JD    date new old equinoxe
	 * @return new ecliptical coordinate pair 
	 */
	public static AnglePair redEcl (double eklL, double eklB, double JD0, double JD)

	/* REDUKTION EKLKIPTIKAL Montenbruck p. 20, Meeus p. 128
      old Aequinoct: JD0, new: JD
      all angles in radians              */
	{
		double  t1, dt, tt, pia, pa, ben, c;

		t1 = (JD0 -2451545.0) /36525.0;
		dt = (JD -JD0) /36525.0;
		
		// Meeus 20.5 p. 128
		final double secrad = 4.8481368111e-6; // arcsec -> radians
		
//		pia= Math.toRadians(  ((47.0029  -0.06603*t1)*dt -0.033*sqr(dt))/3600.0 ); // eta
//		tt = Math.toRadians((629554.98 +3289.48*t1 +0.61*sqr(t1)
//				-(869.81 +0.5*t1)*dt +0.04*sqr(dt))/3600.0);  // CAPITAL PI
//		pa = Math.toRadians( ((5029.097 +2.222*t1)*dt +1.111*sqr(dt))/3600.0 ); // p
    	pia =  ((47.0029-(0.06603-0.000598*t1)*t1) +
 		       ((-0.03302+0.000598*t1)+0.00006*dt)*dt)*dt * secrad;  // eta
    	tt = Math.toRadians(174.876383889) + (((3289.4789+0.60622*t1)*t1) +
 	           ((-869.8089-0.50491*t1)+ 0.03536*dt)*dt) *secrad;
    	pa = ( (5029.0966+(2.22226-0.000042*t1)*t1) +
 	          ((1.11113-0.000042*t1)-0.000006*dt)*dt)*dt * secrad;

		// Montpfleger p 20 f
//		double C1, C2, C3, S1, S2, S3, U, V, W, X, Y, Z;
//		Z = Math.sin (eklB);
//        X = Math.cos (eklL) * Math.cos(eklB);
//        Y = Math.sin (eklL) * Math.cos(eklB);
//		double [] [] A = new double [4] [4];
//		C1 = Math.cos(tt+pa);  C2 = Math.cos(pia);  C3 = Math.cos(tt);
//		S1 = Math.sin(tt+pa);  S2 = Math.sin(pia);  S3 = Math.sin(tt);
//		A[1][1] = +C1*C3+S1*C2*S3; A[1][2] = +C1*S3-S1*C2*C3; A[1][3] = -S1*S2;
//		A[2][1] = +S1*C3-C1*C2*S3; A[2][2] = +S1*S3+C1*C2*C3; A[2][3] = +C1*S2;
//		A[3][1] = +S2*S3;          A[3][2] = -S2*C3;          A[3][3] = +C2;
//
//		U  =  A[1][1]*X+A[1][2]*Y+A[1][3]*Z;
//		V  =  A[2][1]*X+A[2][2]*Y+A[2][3]*Z;
//		W  =  A[3][1]*X+A[3][2]*Y+A[3][3]*Z;
//		X = U; Y = V; Z = W;
//		return new AnglePair (Math.atan2(Y, X), Math.asin(Z));

		ben= Math.asin (Math.cos(pia)*Math.sin(eklB) +Math.sin(pia)*Math.cos(eklB)*Math.sin(tt-eklL));  // new eklB, r can be assumed as = 1
		
		//Meeus  p. 129
		double a_y = Math.cos(pia)*Math.cos(eklB) *Math.sin(tt-eklL) -Math.sin(pia)*Math.sin(eklB);
		double b_x = Math.cos(eklB)*Math.cos(tt-eklL);
		c = Math.atan2(a_y, b_x);
		
//		c  = ( Math.acos((Math.cos(eklB)*Math.cos(tt-eklL))/Math.cos(ben)) );
//		if ( Math.asin(( Math.cos(pia)*Math.cos(eklB)*Math.sin(tt-eklL) -Math.sin(pia)*Math.sin(eklB) )
//				/Math.cos(ben)) <0)
//			c = 2.0*Math.PI -c;

		//eklL = tt +pa -c ;
		//eklB = ben;

		return new AnglePair (tt +pa -c, ben);
	} //of RedEkl ***************************************************


	/** reduction equatorial coordinates 
	 * 
	 * @param Ra  old equinoxe radians
	 * @param De  old equinoxe radians
	 * @param JD0 date old equinoxe
	 * @param JD  date new old equinoxe 
	 * @return new equatorial coordinate pair 
	 */
	public static AnglePair redEqu (double Ra, double De, double JD0, double JD)
	/* REDUKTION AEQUATORIAL Montenbruck S. 32
      Altes Aequinokt: JD0, neues: JD
      Alle Winkel in Bogenmass              */
	{            
		double  T0, T, zeta, za, Theta, Ra0, De0,  x, y, z;
		Ra0 = Ra;   De0 = De;
		T0 = (JD0 -2451545.0 ) /36525.0;
		T  = (JD  -JD0 )/36525.0;

		zeta  = (2306.218 +1.397*T0) *T +0.302*sqr(T) +0.018*T*T*T;
		za    = Math.toRadians ( (zeta +0.793*sqr(T)) /3600.0);
		Theta = (2004.311 -0.853*T0)*T -0.427*sqr(T) -0.042*T*T*T;
		zeta  = Math.toRadians (zeta/3600.0);   
		Theta = Math.toRadians (Theta/3600.0);

		x = Math.cos(Theta)*Math.cos(De0)*Math.cos(Ra0+zeta) -Math.sin(Theta)*Math.sin(De0);
		y = Math.cos(De0)  *Math.sin(Ra0+zeta);
		z = Math.sin(Theta)*Math.cos(De0)*Math.cos(Ra0+zeta) +Math.cos(Theta)*Math.sin(De0);
		
		double [] l_b_rVec = new double[3];
		cartesianToPolar3 (x, y, z, l_b_rVec);
		//Ra = Ra +za;
		return new AnglePair (l_b_rVec [0] +za, l_b_rVec[1]);
	}  //of RedAeq **************************************************************


	
	
	//---------------------------------------------------------------------------
	// NUTEQU: Transformation von mittleren in wahre aequatoriale                
	//	         Koordinaten (Terme >0.1" nach IAU 1980)                           
	//	         Nach Montenbruck NUTEQU                                           
	//---------------------------------------------------------------------------
	
	/** Nutation transformation for mean to true equatorial coordinates
	 * 
	 * @param julDat Julian date
	 * @param args RA, DE in radians, (Radius)
	 */
	public static void nutat_Aeq (double julDat, double args[]) {

		final double ARC =206264.8062;        // Bogensekunden pro radian = 3600*180/pi 
		double   LS,D,F,N,EPS ;
		double   DPSI,DEPS,C,S;
		double   DX,DY,DZ     ;
		double   T            ;

		T   = (julDat-2451545.0)/36525.0;
		
		double xyzVec[] = new double[3];
//		MathOps.polarToCartesian (xyzVec, args[0], args[1], args [2]);
		polarToCartesian3 (args, xyzVec);
		LS  = TWO_PI * frac(0.993133+  99.997306*T);   // mittl. Anomalie Sonne    
		D   = TWO_PI * frac(0.827362+1236.853087*T);   // Diff. Laenge Mond-Sonne  
		F   = TWO_PI * frac(0.259089+1342.227826*T);   // Knotenabstand            
		N   = TWO_PI * frac(0.347346-   5.372447*T);   // Laenge des aufst.Knotens 
		EPS = 0.4090928-2.2696E-4*T;             	   // Ekliptikschiefe          
		DPSI = ( -17.200*Math.sin(N)   - 1.319*Math.sin(2*(F-D+N)) - 0.227*Math.sin(2*(F+N))
				 + 0.206*Math.sin(2*N) + 0.143*Math.sin(LS) ) / ARC;
		DEPS = ( + 9.203*Math.cos(N)   + 0.574*Math.cos(2*(F-D+N)) + 0.098*Math.cos(2*(F+N))
				 - 0.090*Math.cos(2*N)                 ) / ARC;
		C = DPSI*Math.cos(EPS);  S = DPSI*Math.sin(EPS);
		DX = -(C*xyzVec[1]+S*xyzVec[1]); DY = (C*xyzVec[0]-DEPS*xyzVec[1]); DZ = (S*xyzVec[0]+DEPS*xyzVec[1]);
		xyzVec[0]  =  xyzVec[0] + DX;        xyzVec[1]  = xyzVec[1] + DY;       xyzVec[2]  = xyzVec[2] + DZ;
		cartesianToPolar3 (xyzVec, args);
	}


	//---------------------------------------------------------------------------
	// NUTEQU: Transformation von mittleren in wahre aequatoriale                
	//	         Koordinaten (Terme >0.1" nach IAU 1980)                           
	//	         Nach Montenbruck NUTEQU                                           
	//---------------------------------------------------------------------------
	/** Nutation transformation for mean to true ecliptical coordinates
	 * 
	 * @param julDat Julian date
	 * @param eklL ecliptical longitude in radians,
	 * @return ecliptical longitude in radians, (no change in ecliptical latitude)
	 */
	public static double nutat_Ekl (double julDat, double   eklL) { // EklB bleibt unveraendert
	  final double ARC=206264.8062;        // Bogensekunden pro radian = 3600*180/pi 
	        //P2 =TWO_PI;             // 2*pi                                   
	  double   LS,D,F,N;
	  double   dPsi;
	  double   T            ;
	    T   = (julDat-2451545.0)/36525.0;
	    LS  = TWO_PI * frac(0.993133+  99.997306*T);   // mittl. Anomalie Sonne    
	    D   = TWO_PI * frac(0.827362+1236.853087*T);   // Diff. Laenge Mond-Sonne  
	    F   = TWO_PI * frac(0.259089+1342.227826*T);   // Knotenabstand            
	    N   = TWO_PI * frac(0.347346-   5.372447*T);   // Laenge des aufst.Knotens 
	    dPsi = ( -17.200*Math.sin(N)   - 1.319*Math.sin(2*(F-D+N)) - 0.227*Math.sin(2*(F+N))
	             + 0.206*Math.sin(2*N) + 0.143*Math.sin(LS) ) / ARC;
	    return eklL +dPsi;
	   }

	/** returns refraction in radians for trueElevationRad
	 *  Meeus Algorithms p.102 formula 15.4
	 * 
	 * @param trueElevationRad  Elevation in radians
	 * @return refraction in radians
	 */
	public static double refraction (double trueElevationRad) {
		return Math.PI/10800 *(1.02/Math.tan(trueElevationRad +0.003138/(trueElevationRad +0.0892)));
	}

	/** returns refraction corrected heigth in Earth radii for trueElevationRad
	 *  Explanatory supplement p. 55
	 * 
	 * @param trueElevationRad    Elevation in Radians
	 * @return refraction corrected heigth in Earth radii
	 */
	public static double refractionHeightRadii (double trueElevationRad) {
		double r 	= refraction (trueElevationRad);
		double z 	= HALF_PI - trueElevationRad;
		return Math.sin(z-r)/Math.sin(z);
	}
	/** returns refraction corrected heigth in Earth radii for trueElevationRad
	 *  Explanatory supplement p. 55
	 * 
	 * @param trueElevationRad Elevation in Radians
	 * @return refraction corrected heigth in m
	 */
	public static double refractionHeightMeters (double trueElevationRad) {
		return refractionHeightRadii (trueElevationRad) *R_EARTH *1000.0;
	}

}
