package strickling.utils;
 

/* Photography tool for instant Photographs, releases shutter after call. 
 * Screen orientation should be locked in Manifest to landscape. 
 * This prevents distorted and/or rotated preview display
 * 
 * Usage: call:
 * 
  			CameraActivity.TakePicture(this, "/sdcard/DCIM/Testaufnahme.jpg");

 * or start Activity by calling: 
  
	       	Intent camIntent = new Intent(this, CameraActivity.class);
	       	camIntent.putExtra ("fileName", "/sdcard/DCIM/Testaufnahme.jpg");
	    	startActivity(camIntent);
	    	
  			CameraActivity.TakePicture(this, "/sdcard/DCIM/Testaufnahme.jpg", 
  			    getWindowManager().getDefaultDisplay().getOrientation());	// Replace FileName    

//required entries in Manifest (screenOrientation prevents distorted preview display):

        <activity android:name="strickling.utils.CameraActivity"
                  android:label="Camera Util"
                  android:screenOrientation="landscape"/>

// or with a string based name:                  
	        <activity android:name="strickling.utils.CameraActivity"
	                  android:label="@string/camera_name"
	                  android:screenOrientation="landscape"/>

// permission required in Manifest:                  
	  	<uses-permission android:name="android.permission.CAMERA"/>
		<uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />

 * no xml layout is required!
 *  
 * from:  http://marakana.com/forums/android/examples/39.html
 */

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Timer;
import java.util.TimerTask;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.PictureCallback;
import android.hardware.Camera.ShutterCallback;
import android.location.Location;
import android.location.LocationManager;
import android.media.ExifInterface;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.widget.FrameLayout;
import android.widget.LinearLayout;

public class CameraActivity extends Activity {//implements SensorListener {
	public static final String TAG = "CameraUtil";
//	public static final String TAG = "HelloAndroid";
//	public static final String TAG = EclipseDroid.TAG;

	public static final int PORTRAIT  = 0; // only for  Phones, Tablets 0 = Landscape
	public static final int LANDSCAPE = 1;
	public static final int LANDSCAPE_ClOCKWISE = 3;
	public static final int UPSIDE_DOWN = 2;
	
	
	public static final int 	CAMERA_DELAY_MS = 1100; // ms
	public static final String 	DEFAULT_DIR = "/sdcard/DCIM/"; 
//    if (Build.VERSION.SDK_INT >= 8) {
//    	photoDir = android.os.Environment.getExternalStoragePublicDirectory (android.os.Environment.DIRECTORY_PICTURES).getPath();
//    } else {photoDir = CameraActivity.DEFAULT_DIR;}

	public static String 		fileName;  // file name AND path!
	public static int 			rotationAngle = 0;
	public static String 		camParameters  = "";  
	// Parameters like "picture-size=640x480;jpeg-quality=80;rotation=90" // Rot. 90 = Portrait
	// default is set to maximal resolution, and  jpeg-quality=100 (maximal quality)
	public static String paramError = ""; // Got from cameraPreview at shutter;
	protected LocationManager locationManager;  // for Geotagging

	static CameraPreview cameraPreview; // <1>
	private Timer timer;
	protected boolean releaseCalled = false;
	protected int timerCount	    = 0;
	public final static long DELAY_MILLIS = 500;

	public final static int CAMERA_ERROR_NO_INIT = CameraPreview.CAMERA_ERROR_NO_INIT;
	public final static int CAMERA_ERROR_NO_CAM  = CameraPreview.CAMERA_ERROR_NO_CAM;
	public final static int CAMERA_ERROR_IO_ERR  = CameraPreview.CAMERA_ERROR_IO_ERR;
	public final static int CAMERA_ERROR_RELEASE_FAIL  = CAMERA_ERROR_IO_ERR+1;
	public final static int CAMERA_ERROR_FILEIO  = 10;
	public final static int CAMERA_ERROR_FILENOTFOUND  = 11;
	public static int    	cameraError = CAMERA_ERROR_NO_INIT;
	public static final int CAMERAACTIVITYRESULT = 100;
	
	
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		//    setContentView(R.layout.camera_preview);  // old version with XML layout

		// construct the view without a resource file
		FrameLayout fl = new FrameLayout(this);
		setContentView(fl);
		locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
		paramError = "";
		Intent intent = getIntent();
		if (intent != null) {
			fileName = (String) intent.getCharSequenceExtra("fileName");
			if (fileName == null) fileName = "";
			Log.d(TAG, "fileName preset: " + intent.getCharSequenceExtra("fileName"));

			camParameters = (String) intent.getCharSequenceExtra("camParameters");
			if (camParameters == null) camParameters = "";
			if (! camParameters.equals("")) Log.d(TAG, "CameraActivity: Camera Parameters: "+ camParameters);  
			}
		else  camParameters = "";
		
		// add the Camera preview
		cameraPreview = new CameraPreview(this); // <3>
		cameraPreview.paramList = camParameters;
		cameraPreview.rotationAngle = rotationAngle;
		
		
		//	    ((FrameLayout) findViewById(R.id.preview)).addView(cameraPreview); // <4> old version with XML layout
		fl.addView(cameraPreview); // <4>
		cameraError = CAMERA_ERROR_NO_INIT;
		setResult (cameraError);
		Log.d(TAG, "CameraActivity onCreate'd");
	}

	@Override
	public void onStart() {
		super.onStart();
		Log.d(TAG, "CameraActivity onStart");
		
		// get orientation and prepare cameraPreview.rotationAngle
//		sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
//		sensorManager.registerListener(this, sensorA);
		//		  releaseShutter ();    // Crashes the App
		timer = new Timer("CameraActivityTimer");
		releaseCalled = false;
		timerCount = 0;
		
		
		final Runnable updateTask = new Runnable() {
			public void run() {		
				int maxTrials = 10; // set longer time for Autofocus mode
				timerCount++;  // try 10 times, then get error code, if no success
				Log.d(TAG, "    CameraActivity Timer called #" +timerCount 
						+", Camera accessible:" +(cameraPreview.camera != null)
						+", Focused: " +cameraPreview.autoFocused);
				
				if (cameraPreview.camera != null && !releaseCalled) { 
					if (cameraPreview.camera.getParameters().getFocusMode () == Camera.Parameters.FOCUS_MODE_AUTO
						|| cameraPreview.camera.getParameters().getFocusMode () == Camera.Parameters.FOCUS_MODE_MACRO)
					{maxTrials = 20;}
					if (cameraPreview.autoFocused || timerCount > maxTrials-2) {releaseShutter ();}
					}
				
				// Airbag, exit after some seconds without success
				if (timerCount > maxTrials && !releaseCalled) {
					try {
					timer.cancel();
					timer.purge();
					} catch (NullPointerException e) {
						e.printStackTrace();
						CameraPreview.errorLog += "\n" +"CameraActivity run: Error NullPointerException on Timer shutdown:" +"\n" +Log.getStackTraceString(e);
					} finally {
					timer = null;
					cameraError = CAMERA_ERROR_NO_CAM;
					setResult (cameraError);
					finish ();
					}				
				}
			}
		};
		TimerTask tt =  new TimerTask() {
			@Override
			public void run() { runOnUiThread(updateTask); }
		}; 
		//  Launch the shutter by animationTimer
		timer.schedule (tt, DELAY_MILLIS, 200);  // delay, interval
	}

	@Override
	protected void onStop() {
		super.onStop();

	}

	public void releaseShutter (){
		try{
			Log.d(TAG, "CameraActivity releaseShutter: start");
			timer.cancel();
			timer.purge();
			timer = null;
			releaseCalled = true;
			try {
				System.gc();
				cameraPreview.camera.takePicture(shutterCallback, rawCallback, jpegCallback);
				cameraError = CameraPreview.cameraError;
				Log.d(TAG, "CameraActivity releaseShutter: oK, cameraError " +cameraError);
			} catch (RuntimeException e) {
				e.printStackTrace();
				cameraError = CAMERA_ERROR_RELEASE_FAIL;
				Log.d(TAG, "CameraActivity releaseShutter: Failed!   cameraError " +cameraError);
			}
			
		} catch (NullPointerException e) {
			Log.d(TAG, "CameraActivity releaseShutter: Error NullPointerException");
			e.printStackTrace();
			cameraError = CAMERA_ERROR_NO_CAM;	
			CameraPreview.errorLog += "\n" +"CameraActivity releaseShutter: Error NullPointerException :" +"\n" +Log.getStackTraceString(e);
		} finally {
			setResult (cameraError);			
			if (cameraError != 0) {finish ();}
		}
	}


	// Called when shutter is opened
	ShutterCallback shutterCallback = new ShutterCallback() { // <6>
		public void onShutter() {
			paramError = cameraPreview.paramError;
			Log.d(TAG, "CameraActivity shutterCallback:   " +paramError);
		}
	};

	// Handles data for raw picture
	PictureCallback rawCallback = new PictureCallback() { // <7>
		public void onPictureTaken(byte[] data, Camera camera) {
			paramError = cameraPreview.paramError;
			 Log.d(TAG, "CameraActivity rawCallback:  onPictureTaken");
		}
	};

	// Handles data for jpeg picture, overwrites existing files
	// is called after ReleaseShutter as callback
	PictureCallback jpegCallback = new PictureCallback() { // <8>
		public void onPictureTaken(byte[] data, Camera camera) {
			cameraError = CameraPreview.cameraError;
			String result = " error";
			if (cameraError == 0) {
				FileOutputStream outStream = null;
				try {
					// Write to SD Card, if necessary make file name
					if (fileName == "") fileName = MakeDateTimeFileName ("");
					fileName = fileName.replace(":", "-");
					outStream = new FileOutputStream(fileName); // <9>
					outStream.write(data);
					outStream.close();
					Log.d(TAG, "CameraActivity jpegCallback: onPictureTaken " +fileName +" - wrote bytes: " + data.length);
					result = " OK";
					geoTag (fileName, locationManager);
				} catch (FileNotFoundException e) { // <10>
					cameraError = CAMERA_ERROR_FILENOTFOUND;
					e.printStackTrace();
					Log.d(TAG, "CameraActivity jpegCallback: onPictureTaken - Camera: FileNotFound Exception: " +fileName);
					CameraPreview.errorLog += "\n" +"CameraActivity jpegCallback: onPictureTaken - Camera: FileNotFound Exception: " +fileName +"\n" +Log.getStackTraceString(e);
				} catch (IOException e) {
					e.printStackTrace();
					cameraError = CAMERA_ERROR_FILEIO;
					Log.d(TAG, "CameraActivity jpegCallback: onPictureTaken - Camera: IO Exception");
					CameraPreview.errorLog += "\n" +"CameraActivity jpegCallback: onPictureTaken - Camera: IO Exception:" +"\n" +Log.getStackTraceString(e);
				} finally {			}
			}
			Log.d(TAG, "CameraActivity jpegCallback: onPictureTaken - jpeg " +fileName +result);
			setResult (cameraError);
			finish ();
		}
	};
	
	/**
	 * 
	 * @param packageContext,  usually handle "this"
	 * @param fileName
	 * @param camParameters
	 * @param orientation get by: getWindowManager().getDefaultDisplay().getOrientation()
	 */
	public static boolean TakePicture (Context packageContext, String fileName, String camParameters, int orientation) {

		/* Orientation must be handled vom calling activity, 
		 * because screeen orientation for Camera should be locked to lnadscape in Manifest.
		 * otherwise preview will be turned or distorted, if device is not in landscape. 
		 */
		boolean result = false;
		rotationAngle = calcRotationAngle (orientation); // handled to Preview in onStart
       	Intent camIntent = new Intent(packageContext, CameraActivity.class);
       	camIntent.putExtra ("fileName", fileName);
       	camIntent.putExtra ("camParameters", camParameters);
       	try{
//       		packageContext.startActivity(camIntent);
       		((Activity) packageContext).startActivityForResult(camIntent, CAMERAACTIVITYRESULT);
       		Log.d(TAG, "CameraActivity TakePicture Intent started "); // cameraError is now still CAMERA_ERROR_NO_INIT
       		result = true;
       	} catch (RuntimeException e) {
			// Open Camera error
			e.printStackTrace();
			Log.d(TAG, "CameraActivity TakePicture intent Error");
			result = false;
			CameraPreview.errorLog += "\n" +"CameraActivity TakePicture intent Error:" +"\n" +Log.getStackTraceString(e);
		}
       	return result;
	}

	public static int calcRotationAngle (int orientation) {
		int rotationAngle;
		switch (orientation){
		case PORTRAIT: // Portrait 0
			rotationAngle = 90;
			break;
		case LANDSCAPE: // Landscape 1 counterclockwise, "normal" photo position
			rotationAngle = 0;
			break;
		case UPSIDE_DOWN: // Portrait reverse 2, not defined?
			rotationAngle = 270;
			break;
		case LANDSCAPE_ClOCKWISE: // Landscape clockwise 3 
			rotationAngle = 180;
			break;
		default: rotationAngle = 0;		
		}
		Log.d(TAG, "CameraActivity: Orientation: " +orientation +" Angle: " +rotationAngle);
		return rotationAngle;
	}
	
	public static String MakeDateTimeFileName (String prefix) {
		return MakeDateTimeFileName (prefix, DateTimeUtils.GetCurrentTimeZone());
	}

	/** Makes a File name with prefix + "yyyy-mm-dd h-mm-ss" formatted String of actual TimeZone. 
	 * Prefix MUST contain a valid path!
	 * 
	 * @param prefix 
	 * @param timeZone in hrs
	 * @return
	 */
	public static String MakeDateTimeFileName (String prefix, double timeZone) {
		String fileName;
		if (! prefix.equals ("")) {fileName = prefix;}
		else fileName = DEFAULT_DIR +"CameraUtil";
		fileName = fileName +" " +DateTimeUtils.zeitStrg(
				DateTimeUtils.nowJulDat () +CAMERA_DELAY_MS/86400000.0, 
				timeZone) +".jpg"; 
		// is now Current Date & Time: yyyy - mm - dd  h:mm:ss
		fileName = fileName.replace(" - ", "-"); 
		fileName = fileName.replace("  ", " ");
		fileName = fileName.replace(":", "-");
		return fileName;
	}

	/** Returns String like 7  30  0.00 from 7.5
	 * 
	 * @param angleDeg angle in degrees
	 * @return String like 7  30  0.00 from 7.5
	 */
	public static String gpsDMSStrg (double angleDeg) {
		String result = AstroUtils.DMSStrg(angleDeg *AstroUtils.RADFAKT, "dms2");
		result = result.replace(",", ".");
		result = result.replace("\u00B0 ", "/1,0");
		result = result.replace("' ", "/1,0");
		result = result.replace("\"", "/1");
		result = result.replace("+", "");
		result = result.replace(" ", "0");
		return result;
	}

	
	public static void geoTag (String filename, LocationManager locationManager) {
		Location location = null;
        try {
            location = GeoUtils.getLocation(locationManager);
            geoTag (filename, location);
           	} catch (RuntimeException e) {
           		e.printStackTrace();
				CameraPreview.errorLog += "\n" +"CameraActivity GeoTag Error 1:" +"\n" +Log.getStackTraceString(e);
           	}
	}
	
	/** Geotags a file
	 * 
	 * @param filename String
	 * @param location Location
	 */
	public static void geoTag (String filename, Location location) {
		if (location != null) {
			if (Build.VERSION.SDK_INT >= 5) try {
				ExifInterface exif = new ExifInterface(filename);
				if (location.getLatitude() >= 0) {exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE_REF, "N");}
				else {exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE_REF, "S");}
//			    exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE,  "51.300000/1,");
				exif.setAttribute(ExifInterface.TAG_GPS_LATITUDE,  gpsDMSStrg (Math.abs(location.getLatitude()) ));
				Log.d(TAG, "Geotag Lat: " +exif.getAttribute(ExifInterface.TAG_GPS_LATITUDE));

			
				if (location.getLongitude() >= 0) {exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE_REF, "E");}
				else {exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE_REF, "W");}
//				exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE, "0.0");
				exif.setAttribute(ExifInterface.TAG_GPS_LONGITUDE, gpsDMSStrg (Math.abs(location.getLongitude())));
//				exif.saveAttributes(); // save up to now, then try more
				Log.d(TAG, "Geotag Lon: " +exif.getAttribute(ExifInterface.TAG_GPS_LONGITUDE));
				
				// only available since API level 9 = Android 2.3
				if (location.hasAltitude()) {
					if (location.getAltitude() >= 0) {exif.setAttribute("GPSAltitudeRef", "1");} // 0!
					else {exif.setAttribute("GPSAltitudeRef", "1");}
					//exif.setAttribute("GPSAltitude", "087/1");
					exif.setAttribute("GPSAltitude", String.format("%03.0f/1", Math.abs(location.getAltitude())));
				} 
				exif.setAttribute("GPSTimeStamp", 
						DateTimeUtils.HMSZeitStrg(DateTimeUtils.millis2JulDat(location.getTime()), 0).replace(":","/1,"));
				 //"12/1,20/1,30/1");
				exif.setAttribute("GPSDateStamp", 
						DateTimeUtils.YMDStrg(DateTimeUtils.millis2JulDat(location.getTime()), 0).replace(" - ",":").replace(" ", "0"));
						//"2010:07:01");
				exif.saveAttributes();   
				Log.d(TAG, "Geotag finish"); //LATITUDE: " + gpsDMSStrg (location.getLatitude()));
				//Log.d(TAG, "Geotag LONGITUDE: "+ gpsDMSStrg (location.getLongitude()));
			} catch (IOException e) {
				e.printStackTrace();
				Log.d(TAG, "GeoTag Error");
				CameraPreview.errorLog += "\n" +"CameraActivity GeoTag Error 2:" +"\n" +Log.getStackTraceString(e);
			}
		} else {Log.d(TAG, "GeoTag location = null");}
	}
	
	
	/*	public static String addGPS (String parameters, LocationManager locationManager) {
    Location loc = GeoUtils.GetLocation(locationManager);  
    if (loc != null) {
   	 return (String.format ("gps-longitude=%.7f;",loc.getLongitude())
   			 +String.format("gps-latitude=%.7f;", loc.getLatitude())
   			 +String.format("gps-altitude=%.1f;", loc.getAltitude()) 
   			 +String.format("gps-timestamp=%d;",  loc.getTime()) 
   			 +parameters); 
    } else return parameters;
}
*/
	
//	public static boolean hasTorch () { // produces runtime error 
//		boolean result = false;
//		Camera camera = null;
//		try {
//			camera = Camera.open();
//			Camera.Parameters mParameters = camera.getParameters();
//			List <String> flashModes = mParameters.getSupportedFlashModes ();
//			if (flashModes != null && flashModes.contains("torch")) {
//				result = true;
//			}
//		} finally  {
//			try {
//				if (camera != null) camera.release();
//			} finally  {};
//		}
//		return result;
//	}
	
	static ARView arView = null;
	
	/** blinks torch for specified milliseconds, for longer times use child thread! <br>
	 *  needs in manifest:  	&lt;uses-permission android:name="android.permission.CAMERA"/>	
	 * @param millis
	 * @return true, if successful, false if no flash mode available
	 */
	public static boolean blinkTorch (int millis, LinearLayout view, Context context) {
		// http://stackoverflow.com/questions/8876843/led-flashlight-on-galaxy-nexus-controllable-by-what-api/9379765#9379765
		
		Log.d(TAG, "StartBlink");
		if (arView != null) {
			view.removeView(arView);
			arView.destroy();
			arView = null;
		}

		ARView.blinkFlashMillis = millis;
		arView = new ARView (context);
		view.addView(arView); 
//		try {
//			Thread.sleep (millis); //TODO may produce ANR here, put to separate thread
//		} catch (InterruptedException e) {			
//			e.printStackTrace();
//			Log.d(TAG, "CameraActivity Blink Torch: SleepError");
//			//CameraPreview.errorLog += "\n" +"CameraActivity Blink Torch: SleepError:" +"\n" +Log.getStackTraceString(e);
//		}
		Log.d(TAG, "StopBlink");
		arView.destroy();
		return true;
	}


/*		
		
		try {
//			SurfaceView sv = (SurfaceView) (view);
//			SurfaceHolder mHolder = sv.getHolder();
			Camera camera = Camera.open();
			Camera.Parameters mParameters = camera.getParameters();
//			AutoFocusCallback autoFocusCallback = new AutoFocusCallback();
//			autoFocusCallback.setHandler(handler, message);
			//Log.d(TAG, "Requesting auto-focus callback");
			//camera.autoFocus(autoFocusCallback);

			//Log.d(TAG, mParameters.flatten());
			List <String> flashModes = mParameters.getSupportedFlashModes ();
			if (flashModes != null && flashModes.contains("torch")) {

				mParameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF); // does not work on SGS2
				camera.setParameters(mParameters);
				camera.startPreview();
				try {
					// http://stackoverflow.com/questions/8876843/led-flashlight-on-galaxy-nexus-controllable-by-what-api/9379765#9379765
					camera.setPreviewDisplay(mHolder);
					Log.d(TAG, "camera.setPreviewDisplay(mHolder): OK!");
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
					Log.d(TAG, "HolderError");
				}

				//Set the torch parameter 
				mParameters.setFlashMode(Camera.Parameters.FLASH_MODE_TORCH); // does not work on SGS2
				mParameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
				//mParameters.set("flash-mode", "torch"); 
				//Commit camera parameters
				camera.setParameters(mParameters); Log.d(TAG, "Flash on, start");
//				Log.d(TAG, camera.getParameters().flatten());
				//camera.startPreview();
				
			    camera.autoFocus(new AutoFocusCallback() {
	                public void onAutoFocus(boolean success, Camera camera) {
	                }
	     });

				try {
					Thread.sleep (millis); //TODO may produce ANR here, put to separate thread
				} catch (InterruptedException e) {
					
					e.printStackTrace();
					Log.d(TAG, "CameraActivity Blink Torch: SleepError");
					CameraPreview.errorLog += "\n" +"CameraActivity Blink Torch: SleepError:" +"\n" +Log.getStackTraceString(e);
				}
				camera.setParameters(mParameters);
				mParameters.setFlashMode(Camera.Parameters.FLASH_MODE_OFF);
				camera.stopPreview();
				Log.d(TAG, "Flash off");
				camera.release ();
				return true;
			}
			else return false;
		}
		catch  (RuntimeException e) {
			// Open Camera error
			e.printStackTrace();
			Log.d(TAG, "CameraActivity blinkTorch: Open Error");
			CameraPreview.errorLog += "\n" +"CameraActivity blinkTorch: Open Error:" +"\n" +Log.getStackTraceString(e);
			return false;
		}
	}
	*/
	
	


	   static AutoFocusCallback myAutoFocusCallback = new AutoFocusCallback(){
		   //@Override
		   public void onAutoFocus(boolean arg0, Camera arg1) {
			   //super.onAutoFocus(arg0, arg1);
			   Log.d(TAG, "Autofocus callback");
		   }};

}


