package strickling.utils;

//from:  http://marakana.com/forums/android/examples/39.html
// Autofocus see: http://android-er.blogspot.com/2011/01/start-camera-auto-focusing-autofocus.html

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 


import java.io.IOException;
import java.util.List;

import android.annotation.SuppressLint;
import android.content.Context;
import android.hardware.Camera;
import android.hardware.Camera.AutoFocusCallback;
import android.hardware.Camera.PreviewCallback;
import android.os.Build;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class CameraPreview extends SurfaceView implements SurfaceHolder.Callback { // <1>
	public static final String TAG = CameraActivity.TAG;

	//private static final String TAG = "CameraPreview";
	//  private static final String TAG = "HelloAndroid";

	SurfaceHolder mHolder;  // <2>
	public Camera camera; // <3>
	public String paramList = ""; // Parameters like "picture-size=640x384;jpeg-quality=100"
	public String paramError = "";
	public int 	  rotationAngle = 0;
	public boolean autoFocused  = false;
	
	public final static int CAMERA_ERROR_NO_INIT = -1;
	public final static int CAMERA_ERROR_NO_CAM  = 1;
	public final static int CAMERA_ERROR_IO_ERR  = 2;
	public static int    	cameraError = CAMERA_ERROR_NO_INIT;
	public static String 	errorLog = "";    


	CameraPreview(Context context) {
		super(context);
		cameraError = 0;
		// Install a SurfaceHolder.Callback so we get notified when the
		// underlying surface is created and destroyed.
		mHolder = getHolder();  // <4>
		mHolder.addCallback(this);  // <5>
		mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS); // <6>
	}

	@Override
	protected void onDetachedFromWindow () {
		super.onDetachedFromWindow ();
//		camera.unlock();  // => error, release noch nicht erfolgt!
//		camera = null;
	}
	
	// Called once the holder is ready
	
	@SuppressLint("NewApi")
	public void surfaceCreated(SurfaceHolder holder) {  // <7>
		// The Surface has been created, acquire the camera and tell it where
		// to draw.
		Log.d (TAG, "CameraPreview: Try Camera.open()");
		autoFocused = false;
		try {
//			Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
//			Camera.getCameraInfo(1, cameraInfo);
//			Log.d (TAG, "CameraInfo: " +cameraInfo.toString());
			int camera_id = 0;
			if (paramList.contains("camera_id=")) { // this is not an official parameter, extension by Strickling
				camera_id = (int) AstroUtils.getNumeric(paramList.subSequence(paramList.indexOf("camera_id="), paramList.length()).toString());
			}
			try {
				camera = Camera.open(camera_id); //
				Log.d (TAG, "CameraPreview: Camera.open("+camera_id +") oK, camera_id=" +camera_id);
			} catch (Exception e) { 
				camera = Camera.open(); 
				Log.d (TAG, "CameraPreview: Camera.open() oK");
				}
			try {
				Camera.Parameters parameters = camera.getParameters(); //TODO Hier ANR keyDispatchingTimedOut
//				Log.d(TAG, "Camera parameters: " +parameters.flatten()); //Puts all parameters into one String
				// setting PictureSize to full resolution
				List <Camera.Size> ps =  parameters.getSupportedPictureSizes ();
				Camera.Size   imgSize = parameters.getPictureSize ();
				for (int i=0;i<ps.size()-1;i++) {
					if (imgSize.width < ps.get(i).width) imgSize = ps.get(i);
					//  				Log.d (TAG, "sizes: " + ps.get(i).width +" x " + ps.get(i).height);
				}
				parameters.setPictureSize(imgSize.width, imgSize.height); // Correct values required, API 1
				parameters.setJpegQuality (100);// API Level 5!
				parameters.setRotation(rotationAngle);  // should be set by CameraActivity
				Log.d (TAG, "rotationAngle: " +rotationAngle);
				camera.setParameters(parameters);

				int i = 0;
				try {  // Crashes, if wrong parameters
					String[] rowData = paramList.split(";");  			
					while(i < rowData.length ) {
						if (! rowData[i].contains("camera_id=")) {  // this is not an official parameter
							String [] paramToSet = rowData[i].split("=");
							if (paramToSet.length == 2) {
								parameters.set(paramToSet[0], paramToSet[1]);
								try{
									camera.setParameters(parameters);
									Log.d (TAG, "CameraPreview: Set Camera: " +rowData[i]);  	
								} catch (RuntimeException r) {
									paramError += "ParamError: " +rowData[i]+";";
									Log.d (TAG, "CameraPreview: ParamError: " +rowData[i]); 
									errorLog += "\n" +"CameraPreview: ParamError: " +rowData[i] +"\n" +Log.getStackTraceString(r); 
								}
							}
						}
						i++;
					}
				} finally {	}  			

				try { 
					Log.d (TAG, "CameraPreview: actual Camera parameters: " +parameters.flatten());
					String params = parameters.flatten().replace(";", ";\n");
					if (Build.VERSION.SDK_INT > 8) {
						if (! params.subSequence(params.length()-1, params.length()).equals(";")) params+=";";
						params += "\nmax camera_id=" +(Camera.getNumberOfCameras() -1);
						}
					SystemUtils.saveString(params, SystemUtils.writeableDir +"CameraParameters.txt");
				} catch (Exception e) {}
                camera.setPreviewDisplay(holder);  // <9>

/*				camera.setPreviewCallback(new PreviewCallback() { // <10>
					// Called for each frame previewed
					public void onPreviewFrame(byte[] data, Camera camera) {  // <11>
						//Log.d(TAG, "CameraPreview onPreviewFrame called at [ms]: " + System.currentTimeMillis());
						CameraPreview.this.invalidate();  // <12>  results in ANR, not necessary
					}
				}); */  // Callback rausgeschmissen, nicht necessary, tut ja nix...
				
				
			} catch (IOException e) { // <13>
				e.printStackTrace();
				Log.d (TAG, "CameraPreview: IOException");
				cameraError = CAMERA_ERROR_IO_ERR;
				errorLog += "\n" +"CameraPreview: IOException" +"\n" +Log.getStackTraceString(e);
			}
		} //finally {} //Log.d(TAG, "onPreviewFrame Camera open error");}
		catch (RuntimeException e) { // <13>
			e.printStackTrace();
			camera = null;
			Log.d (TAG, "CameraPreview: RuntimeException " +e.toString());
			cameraError = CAMERA_ERROR_NO_CAM;
			errorLog += "\n" +"CameraPreview: RuntimeException :" +"\n" +Log.getStackTraceString(e);
		}
	}


	// Called when the holder is destroyed
	public void surfaceDestroyed(SurfaceHolder holder) {  // <14>
		// Surface will be destroyed when we return, so stop the preview.
		// Because the CameraDevice object is not a shared resource, it's very
		// important to release it when the activity is paused.
		try {
			camera.setPreviewCallback(null); // inserted after Heinz Crash report, see
			//   http://stackoverflow.com/questions/3592283/camera-force-close-when-returning-from-sleep-android
			camera.stopPreview();	  		
			camera.release(); 
			camera=null;     // der muss hier sein
		} catch (NullPointerException e) {
			e.printStackTrace();
			errorLog += "\n" +"CameraPreview: surfaceDestroyed :" +"\n" +Log.getStackTraceString(e);
		} catch (RuntimeException e) {
			e.printStackTrace();
			errorLog += "\n" +"CameraPreview: surfaceDestroyed :" +"\n" +Log.getStackTraceString(e);
		}
	}

	// Called when holder has changed
	public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) { // <15>
		// Now that the size is known, set up the camera parameters and begin
		// the preview.
		try{
			camera.startPreview();
			camera.autoFocus(myAutoFocusCallback);
		}catch (RuntimeException e) { // <13>
				e.printStackTrace();
				Log.d (TAG, "CameraPreview: surfaceChanged: " +e.toString());
				errorLog += "\n" +"CameraPreview: surfaceChanged :" +"\n" +Log.getStackTraceString(e);
//				cameraError = CAMERA_ERROR_NO_CAM;
			}
	}
	   AutoFocusCallback myAutoFocusCallback = new AutoFocusCallback(){

		   //@Override
		   public void onAutoFocus(boolean arg0, Camera arg1) {
		    // TODO Auto-generated method stub
			   //super.onAutoFocus(arg0, arg1);
		    autoFocused = true;
		    Log.d(TAG, "\n" +"CameraPreview: myAutoFocusCallback called, Focusing completed");
		   }};

}