package strickling.utils;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import android.location.Location;

public class GeoCoordinate {

/**
 *
 * @author Wolfgang Strickling
 *         following Coordinate by Alexander Muthmann <amuthmann at dev-eth0.de>
 * @version 10/2010 
 * 
 * Coordinate (double lat, double lon, double height, String name)
 */

	public static final double R_EQUATOR = 6378137.0;    //meters
	public static final double R_POLE    = 6356752.3142; //meters
	public static final String INVALID_PROVIDER = "void";
	public static final String DEFAULT_PROVIDER = "GeoCoordinate";
	
	public Location location;
    public String name;  			// Extendend by Strickling
    public String countryName;   	// Extendend by Strickling
    public String timeZoneStrg;     // like "Europe/Berlin" 
    public boolean isGeocoded;

    public GeoCoordinate(Location location) {
        if (location != null) {
            this.location = location;
            this.name = location.getProvider();
            this.countryName = "";
        } else {
            this.location = new Location (INVALID_PROVIDER);
            this.name = "";
            this.countryName = "";
            this.location.setLatitude(GeoRecord.ERROR_LAT);
            this.location.setLongitude(GeoRecord.ERROR_LAT);
        }
        this.isGeocoded = false;
    }
    public GeoCoordinate(Location location, String name) {
        if (location != null) {
            this.location = location;
            this.name = name;
            this.countryName = "";
        } else {
            this.location = new Location (INVALID_PROVIDER);
            this.name = name;
            this.countryName = "";
            this.location.setLatitude(GeoRecord.ERROR_LAT);
            this.location.setLongitude(GeoRecord.ERROR_LAT);
        }
        this.isGeocoded = false;
    }



    public GeoCoordinate(double lat, double lon, double alt_m, double accuracy, String name) {
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
    	this.location.setAltitude(alt_m);
    	this.location.setAccuracy((float) accuracy);
        if (name != null) {this.name = name;} else this.name ="";
        this.countryName = "";
        this.isGeocoded = false;
    }
        
    public GeoCoordinate(double lat, double lon, double alt_m, String name,  String country) {
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
    	this.location.setAltitude(alt_m);
        if (name != null) {this.name = name;} else this.name ="";
        if (country != null) {this.countryName = country;} else this.countryName = "";
        this.isGeocoded = false;
    }
    
    public GeoCoordinate(double lat, double lon, double alt_m, String name) { 
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
    	this.location.setAltitude(alt_m);
        if (name != null) {this.name = name;} else this.name ="";
        this.countryName = "";
        this.isGeocoded = false;
    }
    
    public GeoCoordinate(double lat, double lon, double alt_m, double bearing, double speed, String name) { 
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
    	this.location.setAltitude(alt_m);
    	this.location.setSpeed ((float) speed);
    	this.location.setBearing((float) bearing);

        if (name != null) {this.name = name;} else this.name ="";
        this.countryName = "";
        this.isGeocoded = false;
    }
    
    public GeoCoordinate(double lat, double lon, double alt_m) { 
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
    	this.location.setAltitude(alt_m);
        this.name = "";
        this.countryName = "";
        this.isGeocoded = false;
    }
    
    public GeoCoordinate(double lat, double lon, String name) {
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
        if (name != null) {this.name = name;} else this.name ="";
        this.countryName = "";
        this.isGeocoded = false;
        }

    public GeoCoordinate(double lat, double lon) {
    	this.location = new Location (DEFAULT_PROVIDER);
    	this.location.setLatitude(lat);
    	this.location.setLongitude(lon);
        this.name = "";
        this.countryName = "";
        this.isGeocoded = false;
    }

    public GeoCoordinate() {
		this.location = new Location (INVALID_PROVIDER);
		this.name = "";
		this.countryName = "";
		this.isGeocoded = false;
		this.location.setLatitude(GeoRecord.ERROR_LAT);
		this.location.setLongitude(GeoRecord.ERROR_LAT);
    }

//    public void setLat(double lat) {
//    	this.lat = lat;
//    }
//
//    public void setLon(double lon) {
//    	this.lon = lon;
//    }

    /** @return Latitude in Degrees  */
    public double getLat() {
        return location.getLatitude();
    }

    /**
     * @return Longitude in Degrees
     */
    public double getLon() {
//        return lon;
        return location.getLongitude();
    }
    
    public double getLatRadians() {
        return Math.toRadians(location.getLatitude());
    }

    public double getLonRadians() {
        return Math.toRadians(location.getLongitude());
    }
    
    public void setLat (double lat_degrees) {
    	location.setLatitude(lat_degrees);
    }
    
    public void setLon (double lon_degrees) {
    	location.setLongitude(lon_degrees);
    }
    
    public String getName() {
        return name;
    }
    
    public String getCountry() {
        return countryName;
    }
    
    public String getNameAndCountry() {
        return getNameAndCountry (name, countryName);
    }
    
    /** returns name and, if not null or empty, country, eg: "Washington, USA" */
    public static String getNameAndCountry (String locName, String country){
        String result = "";
        if (locName != null) result = locName;
        if (country != null && ! country.equals("") && ! country.equals(result)) {result += ", " +country;}
        return result;
    }
    
    /**
     * @return Altitude in meters
     */
    public double getHeight() {
        if (location.hasAltitude()) {
            return location.getAltitude();
        } else return 0.;
    }

    /**
     * @return Altitude in meters
     */
    public double getAlt() {
        return location.getAltitude();
    }

    public void setAlt (double alt_m) {
    	location.setAltitude(alt_m);
    }
    
    public double getAccuracy() {
        return location.getAccuracy();
    }

    public double getBearing() {
        return location.getBearing();
    }

    public double getBearingRad() {
        return Math.toRadians (location.getBearing());
    }

    public double getSpeed() {
        return location.getSpeed();
    }

    public boolean isValid () {
    	return (location.getLatitude() > GeoRecord.ERROR_LAT +1.0 && location.getLongitude() > GeoRecord.ERROR_LAT +1.0);
    }
    
    public Location toLocation (){
    	try {
    		return new Location (location);
    	} catch (Exception e) {
    		return new Location("NullGeoCoordinate");
    	}
//    	Location l = new Location("GeoCoordinate");
//    	l.setAccuracy((float) accuracy);
//    	l.setLatitude(lat);
//    	l.setLongitude(lon);
//    	l.setAltitude(alt);
//    	l.setBearing((float) bearing);
//    	l.setSpeed((float) speed);
//    	l.setTime(timeMillis);
//    	return l;
    }
    
    @Override
    public String toString() {
        return (name != "") ? ": " + location.getLongitude() + "/" + location.getLatitude() : location.getLongitude() + "/" + location.getLatitude();  
    }
    
    /**  Converts to xyz coordinates in meters
     * 
     * @param xyz  returns double [3] meters 
     */
    public void toXYZ (double [] xyz){
    	latLonH2XYZ (location.getLatitude(), location.getLongitude(), location.getAltitude(), xyz);
    }
    
    /**  Converts to xyz coordinates in meters
     * 
     * @param lat  degrees
     * @param lon  degrees
     * @param height meters
     * @param xyz  returns double [3] meters 
     */
    public static void latLonH2XYZ (double lat, double lon, double height, double [] xyz) {
    	latLonRadH2XYZ (Math.toRadians(lat), Math.toRadians(lon), height, xyz);
    }

    /**  Converts to xyz coordinates in meters
         * 
         * @param lat  radians
         * @param lon  radians
         * @param height meters
         * @param xyz  returns double [3] meters 
         */
    public static void latLonRadH2XYZ (double latRad, double lonRad, double height, double [] xyz) {
    	// http://de.wikipedia.org/wiki/Referenzellipsoid
    	double sinPhi = Math.sin(latRad);
    	final double EPSILON   = Math.sqrt(R_EQUATOR*R_EQUATOR -R_POLE*R_POLE)/R_EQUATOR; //meters
    	double nPhi   = R_EQUATOR / Math.sqrt(1-EPSILON *EPSILON*sinPhi*sinPhi);
    	
    	xyz[0] 		  = (nPhi+height) * Math.cos(latRad) * Math.cos (lonRad);
    	xyz[1] 		  = (nPhi+height) * Math.cos(latRad) * Math.sin (lonRad);
    	xyz[2] 		  = (nPhi*(1-EPSILON *EPSILON)+height) * sinPhi;
    }
    
    /** Calculates distance between two xyz-coordinates
     * 
     * @param xyz1
     * @param xyz2
     * @return distance
     */
    public static double geoDistance (double [] xyz1, double [] xyz2) {
    	double dx = xyz1 [0] -xyz2 [0];
    	double dy = xyz1 [1] -xyz2 [1];
    	double dz = xyz1 [2] -xyz2 [2];

    	return Math.sqrt(dx*dx +dy+dy +dz*dz);
    }
    
}
