package strickling.utils;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.io.StringReader;
import java.net.URL;
import java.util.Locale;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.Build;
import android.util.Log;

/**
 *
 * @author Dr. Wolfgang Strickling
 *    following Coordinate Tool by Alexander Muthmann <amuthmann at dev-eth0.de>
 * @version 10/2010
 * 
 * TestURL Z. B. "http://maps.google.com/maps/geo?q=\Drususstrasse+15,+Haltern"
 * umgekehrt: "http://maps.google.com/maps/geo?ll=51.7438,7.1773"
 * als XML: "http://maps.google.com/maps/geo?output=xml&oe=utf-8&ll=51.723610,7.145000"
 * 
 * Geoid Request:
 * http://geographiclib.sourceforge.net/cgi-bin/GeoidEval?input=51.87++7.2
 * 
 * 
 * 
 */

/* Internet-Prozeduren aus dieser Klasse nicht direkt nutzen, sondern aus Request GeoCoder!
 * 
 * z. B.:
 * 	RequestGeoCoder rgc = new RequestGeoCoder (this, searchString, locDisplayHandler); 
 *  
 */

/* Verwendung vom Location Manager:
 * 
 * Manifest:

    <uses-permission android:name="android.permission.ACCESS_FINE_LOCATION" />
	<uses-permission android:name="android.permission.ACCESS_COARSE_LOCATION" />

 	<uses-permission android:name="android.permission.ACCESS_MOCK_LOCATION" />


	// Variablen:
    protected LocationManager locationManager;  
         
    // You can specify the time and distance between location updates.
    // Both are useful for reducing power requirements.
    int MIN_GPS_LOCATION_UPDATE_TIME     = 5000;  // ms
    int MIN_GPS_LOCATION_UPDATE_DISTANCE = 10;    // m
    int MIN_NET_LOCATION_UPDATE_TIME     = 10000;  // ms
    int MIN_NET_LOCATION_UPDATE_DISTANCE = 50;    // m


		// in OnCreate einbinden (for GPS mandatory):
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 2000, 50, this); // 2000 ms, 50 m,
        
         // or optionally gps or network navigation:
    	
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        if (locationManager != null){
        	try{
    			Criteria criteria = new Criteria();
        		criteria.setAccuracy(Criteria.ACCURACY_FINE);    // More accurate, GPS fix.
        		locationManager.requestLocationUpdates(locationManager.getBestProvider(criteria, true),
        				MIN_GPS_LOCATION_UPDATE_TIME, MIN_GPS_LOCATION_UPDATE_DISTANCE, this);
        	} catch (RuntimeException e) {
        		Log.d(TAG, "onCreate, Error: No location service!");
        	}
        }



	// wenn  requestLocationUpdates
	// implements LocationListener in Klassendefinition insert and
	// in der Klasse folgende Mothoden implementieren:

    public void onLocationChanged(Location location) {
//    		location.getAltitude();  // m
//    		location.getLatitude();  // deg
//    		location.getLongitude(); // deg
//    		location.getTime();      // Millis seit 1.1.1970
//    		location.getAccuracy();  // m
//    		location.getSpeed(); 	 // in m/s
//    		location.getProvider();  // "gps" or "network"
      
        	Log.d(TAG, "GPS location changed: lat="+location.getLat()+", lon="+location.getLon());
    }
    public void onProviderDisabled(String arg0) {
    	try{
   		    Criteria criteria = new Criteria();
    		criteria.setAccuracy(Criteria.ACCURACY_COARSE);  // Faster, no GPS fix. Acc WLAN ca 70 m, CellID 2000 m
     		locationManager.requestLocationUpdates(locationManager.getBestProvider(criteria, true),
    	    	MIN_NET_LOCATION_UPDATE_TIME, MIN_NET_LOCATION_UPDATE_DISTANCE, this);    	
    
    		} catch (RuntimeException e) { }
    		Log.d(TAG, "GPS provider disabled " + arg0);
    }
    
    
    public void onProviderEnabled(String arg0) {
        	Criteria criteria = new Criteria();
        	criteria.setAccuracy(Criteria.ACCURACY_FINE);    // More accurate, GPS fix.
        	locationManager.requestLocationUpdates(locationManager.getBestProvider(criteria, true),
    	    MIN_GPS_LOCATION_UPDATE_TIME, MIN_GPS_LOCATION_UPDATE_DISTANCE, this);    	
    
    	Log.d(TAG, "GPS provider enabled " + arg0);
    }
    public void onStatusChanged(String arg0, int arg1, Bundle arg2) {
    	Log.d(TAG, "GPS status changed to " + arg0 + " [" + arg1 + "]");
    }


	//for instant location acquisition:
          try {
            location = locationManager.getLastKnownLocation(locationManager.getBestProvider(criteria, true));
           	} catch (RuntimeException e) { }
 

 */

public class GeoUtils {
	
	public static final double ERDAEQ       = 6378.137;      // km Equatorradius WGS84
	public static final double ERDAEQ_M     = ERDAEQ*1000.0; // m  Equatorradius WGS84
	public static final double ERDPOL       = 6356.7523;     // km Polarradius WGS84
	public static Location 	   defaultLoc   = createLocDeg(51.483, 0, 0);	 // Greenwich, use: location = new Location (GeoUtils.defaultLoc);
	

    private static final String googleMapsAURL = "http://maps.google.com/maps/geo?q=";
    private static final String googleMapsCURL = "http://maps.google.com/maps/geo?ll=";
    private static final String output = "&output=kml";

    public static boolean requestRunning = false; // Wait until false with requests!
    
    public static long lastGPXTime = 0; // Wait until false with requests!
    public static long newTrkSegTimeout = 3600000; // lastGPXTime difference for starting new Trkseg
    
    
    public static String ParseDoc (Document doc, String element) {
    	String result = "";
    	try{
        	NodeList n = doc.getElementsByTagName(element);
            Element nn = (Element) n.item(0);
            result = nn.getFirstChild().getNodeValue();
        	}
        	finally {return result;}
    }
    
    /** creates a location from latRad, lonRadEast, elevM
     * 
     * @param latRad
     * @param lonRadEast
     * @param elevM
     * @return
     */
    public static Location createLoc (double latRad, double lonRadEast, double elevM) {
		Location location = new Location ("manual");
    	location.setLatitude (Math.toDegrees(latRad));
    	location.setLongitude(Math.toDegrees(lonRadEast));
    	location.setAltitude (elevM);
    	return location;
    }

	public static Location createLocDeg (double latDeg, double lonDegEast, double elevM) {
    	return createLoc (Math.toRadians(latDeg), Math.toRadians(lonDegEast), elevM);
	}
    
	public static Location readLoc (SharedPreferences preferences, String locationName) {
		Location loc = new Location (preferences.getString(locationName +"Prov", GeoCoordinate.INVALID_PROVIDER));
		if (!loc.getProvider().equals(GeoCoordinate.INVALID_PROVIDER)) {
			loc.setLatitude	(preferences.getFloat (locationName +"Lat", 0));
			loc.setLongitude(preferences.getFloat (locationName +"Lon", 0));
			if (preferences.getFloat (locationName +"Alt", -999) != -999)
				loc.setAltitude	(preferences.getFloat (locationName +"Alt", 0));
			if (preferences.getFloat (locationName +"Brg", -999) != -999)
				loc.setBearing	(preferences.getFloat (locationName +"Brg", 0));
			if (preferences.getFloat (locationName +"Spd", -999) != -999)
				loc.setSpeed   	(preferences.getFloat (locationName +"Spd", 0));
			loc.setTime    	(preferences.getLong  (locationName +"Time", 0));
		}
		return loc;
	}

	/** Saves loc to editor, checks for loc == null
	 * 
	 * @param loc     // if null save Provider void
	 * @param editor
	 * @param locationName
	 */
	public static void saveLoc (Location loc, Editor editor, String locationName) {
		if (loc != null && !loc.getProvider().equals(GeoCoordinate.INVALID_PROVIDER)) {
			editor.putFloat(locationName +"Lat", (float) loc.getLatitude());
			editor.putFloat(locationName +"Lon", (float) loc.getLongitude());
			if (loc.hasAltitude()) editor.putFloat(locationName +"Alt", (float) loc.getAltitude());
			if (loc.hasBearing())  editor.putFloat(locationName +"Brg", loc.getBearing());
			if (loc.hasSpeed())    editor.putFloat(locationName +"Spd", loc.getSpeed());
			editor.putLong  (locationName +"Time", loc.getTime());
			editor.putString(locationName +"Prov", loc.getProvider());
//			Log.d(TAG, "Save: "+locName +String.format(Locale.US, "  %+.5f %+.5f %s %.0fm/s %.1f",
//					loc.getLatitude(), loc.getLongitude(),
//					DateTimeUtils.zeitStrg(DateTimeUtils.millis2JulDat(loc.getTime()), 0),
//					loc.getSpeed(), loc.getBearing()
//					));
		} else {
			editor.putString(locationName +"Prov", GeoCoordinate.INVALID_PROVIDER);
		}
		editor.commit();
	}

    
    /** Calculates Distance of a location loc from Line1 - Line 2; all in radians
     *  see http://www.movable-type.co.uk/scripts/latlong.html
     * 
     * @param loc
     * @param line1
     * @param line2
     * @param distBearing as result 1
     * @param footpoint   as result 2
     */
    public static void distanceToLine (double [] loc, double [] line1, double [] line2, 
    		double [] distBearing, double [] footpoint) {
    	Location footLocation = createLoc(0, 0, 0);
        Location locLocation  = createLoc(loc[0], loc[1], 0);
    	
    	distBearing [0] = distanceToLine (
    			locLocation,
    			createLoc(line1[0], line1[1], 0), createLoc(line2[0], line2[1], 0), 
    			footLocation);
    	footpoint [0] = Math.toRadians(footLocation.getLatitude());
    	footpoint [1] = Math.toRadians(footLocation.getLongitude());
    	distBearing [1] = Math.toRadians(locLocation.getBearing()); 
    }
 
	   /** http://forum.geoclub.de/viewtopic.php?f=54&t=37169
	    * Calculate the destination and final bearing after traveling a specified
	    * distance, and a specified starting bearing, for an initial location. This
	    * is the solution to the direct geodetic problem. for elliptic earth
	    *
	    * @param ellipsoid reference ellipsoid to use
	    * @param start starting location
	    * @param startBearing starting bearing (degrees)
	    * @param distance distance to travel (meters)
	    * @return
	    */
	   static public Location calculateEndingGlobalCoordinates(Location startLoc, double startBearing_deg, double distance_m)
	   {
		   if (! startLoc.hasAltitude()) startLoc.setAltitude (0);
	      double a = AstroUtils.ERDAEQ*1000.0 +startLoc.getAltitude();
	      double b = AstroUtils.ERDPOL*1000.0 +startLoc.getAltitude();
	      double aSquared = a * a;
	      double bSquared = b * b;
	      double f = (a-b)/a;
	      double phi1 = Math.toRadians(startLoc.getLatitude());
	      double alpha1 = (Math.toRadians(startBearing_deg));
	      double cosAlpha1 = Math.cos(alpha1);
	      double sinAlpha1 = Math.sin(alpha1);
	      double s = distance_m;
	      double tanU1 = (1.0 - f) * Math.tan(phi1);
	      double cosU1 = 1.0 / Math.sqrt(1.0 + tanU1 * tanU1);
	      double sinU1 = tanU1 * cosU1;

	      // eq. 1
	      double sigma1 = Math.atan2(tanU1, cosAlpha1);

	      // eq. 2
	      double sinAlpha = cosU1 * sinAlpha1;

	      double sin2Alpha = sinAlpha * sinAlpha;
	      double cos2Alpha = 1 - sin2Alpha;
	      double uSquared = cos2Alpha * (aSquared - bSquared) / bSquared;

	      // eq. 3
	      double A = 1 + (uSquared / 16384) * (4096 + uSquared * (-768 + uSquared * (320 - 175 * uSquared)));

	      // eq. 4
	      double B = (uSquared / 1024) * (256 + uSquared * (-128 + uSquared * (74 - 47 * uSquared)));

	      // iterate until there is a negligible change in sigma
	      double deltaSigma;
	      double sOverbA = s / (b * A);
	      double sigma = sOverbA;
	      double sinSigma;
	      double prevSigma = sOverbA;
	      double sigmaM2;
	      double cosSigmaM2;
	      double cos2SigmaM2;

	      for (;;)
	      {
	         // eq. 5
	         sigmaM2 = 2.0 * sigma1 + sigma;
	         cosSigmaM2 = Math.cos(sigmaM2);
	         cos2SigmaM2 = cosSigmaM2 * cosSigmaM2;
	         sinSigma = Math.sin(sigma);
	         double cosSignma = Math.cos(sigma);

	         // eq. 6
	         deltaSigma = B
	               * sinSigma
	               * (cosSigmaM2 + (B / 4.0)
	                     * (cosSignma * (-1 + 2 * cos2SigmaM2) - (B / 6.0) * cosSigmaM2 * (-3 + 4 * sinSigma * sinSigma) * (-3 + 4 * cos2SigmaM2)));

	         // eq. 7
	         sigma = sOverbA + deltaSigma;

	         // break after converging to tolerance
	         if (Math.abs(sigma - prevSigma) < 0.0000000000001) break;

	         prevSigma = sigma;
	      }

	      sigmaM2 = 2.0 * sigma1 + sigma;
	      cosSigmaM2 = Math.cos(sigmaM2);
	      cos2SigmaM2 = cosSigmaM2 * cosSigmaM2;

	      double cosSigma = Math.cos(sigma);
	      sinSigma = Math.sin(sigma);

	      // eq. 8
	      double phi2 = Math.atan2(sinU1 * cosSigma + cosU1 * sinSigma * cosAlpha1, (1.0 - f)
	            * Math.sqrt(sin2Alpha + Math.pow(sinU1 * sinSigma - cosU1 * cosSigma * cosAlpha1, 2.0)));

	      // eq. 9
	      double tanLambda = sinSigma * sinAlpha1 / (cosU1 * cosSigma - sinU1 * sinSigma * cosAlpha1);
	      double lambda = Math.atan(tanLambda);

	      // eq. 10
	      double C = (f / 16) * cos2Alpha * (4 + f * (4 - 3 * cos2Alpha));

	      // eq. 11
	      double L = lambda - (1 - C) * f * sinAlpha * (sigma + C * sinSigma * (cosSigmaM2 + C * cosSigma * (-1 + 2 * cos2SigmaM2)));

	      // eq. 12
	      //double alpha2 = Math.atan2(sinAlpha, -sinU1 * sinSigma + cosU1 * cosSigma * cosAlpha1);

	      // build result in degrees
	      double latitude = Math.toDegrees(phi2);
	      double longitude = startLoc.getLongitude() + Math.toDegrees(L);
	      
	      Location loc2 = new Location (startLoc); // same altitude and provider
	      loc2.setLatitude  (latitude);
	      loc2.setLongitude (longitude);
	      return loc2;
	   }
    
    
    /** Calculates Distance of a location loc from Line1 - Line 2
     * 
     * @param loc	 source Location to calc distance from, bearing to footpoint will be returned in bearing
     * @param line1  location point 1
     * @param line2  location point 2
     * @param footpoint  location of footpoint as result
     * @return distance angle in radians
     */
    public static double distanceToLine (Location loc, Location line1, Location line2, Location footpoint) { 
    	  // Calculates Distance of a location loc from Line1 - Line 2; in radians
    	  // 1st: calc trigonum distances B1, L1, B2, L2, Bo, Lo
    	  double da = line1.distanceTo (line2) / ERDAEQ_M; // convert m to radians
    	  double d1 = line1.distanceTo (loc)   / ERDAEQ_M;
    	  double d2 = line2.distanceTo (loc)   / ERDAEQ_M;
    	  double alpha = Math.acos ((Math.cos(d2)-Math.cos(d1)*Math.cos(da))/(Math.sin(d1)*Math.sin(da)));  //Angle at B1, L1
    	  double distanceAngle = Math.asin(Math.sin(alpha) * Math.sin (d1)); // Distance to Line in radians, from: sin alpha = sin a / sin c

    	// 2nd: Calculating Bearing to Foot point
    	  double bd1 = Math.toRadians(loc.bearingTo (line1));    // bearing [Bo,Lo - B1,L1]
    	

    	  double bda = Math.toRadians(line1.bearingTo (line2)); // bearing [B1,L1 - B2,L2]
    	  double b0  = Math.toRadians(line1.bearingTo (loc));   // bearing [B1,L1 - Bo,Lo]
    	 

    	  double beta  = Math.asin (Math.cos(alpha) /Math.cos (distanceAngle)); //angle B1,L1 -> Bo, Lo -> Footpoint, from cos alpha = cos a sin beta 
    	  if (Math.sin (b0-bda) < 0) beta *= -1.0;


    	  double lineBearing = bd1 +beta;
    	  if (lineBearing < -Math.PI) lineBearing +=2.0 *Math.PI;

    	  if (Math.cos (lineBearing) > 0) distanceAngle *= -1.0; //if [Bo,Lo] south of Line [B1,L1 - B2,L2]
    	  
    	  // Calculation of footpoint    	  
      	double[] latLonRad = new double [2];
      	distBear2Latlon (Math.toRadians(loc.getLatitude()), Math.toRadians(loc.getLongitude()), 
      			distanceAngle, lineBearing, latLonRad);
//      	footpoint = new Location (loc.getProvider());
      	footpoint.setLatitude (Math.toDegrees(latLonRad [0]));
      	footpoint.setLongitude(Math.toDegrees(latLonRad [1]));
    	
      	loc.setBearing((float) Math.toDegrees(lineBearing));
      	return distanceAngle;
    	} //distanceToLine 

    /** Calculate target coordinates from source coordinates and bearing and distance
     * 
     * @param locSrc
     * @param distanceKm
     * @param bearingDeg
     * @param locTarget
     */
    public static void distBear2Latlon (Location locSrc, double distanceKm, double bearingDeg, 
    		Location locTarget){
    	double[] latLonRad = new double [2];
    	distBear2Latlon (Math.toRadians(locSrc.getLatitude()), Math.toRadians(locSrc.getLongitude()), 
    			distanceKm/(ERDAEQ +locSrc.getAltitude()/1000.0), Math.toRadians(bearingDeg), latLonRad);
    	locTarget.setProvider(locSrc.getProvider()); //= new Location (locSrc.getProvider());
    	locTarget.setLatitude (Math.toDegrees(latLonRad [0]));
    	locTarget.setLongitude(Math.toDegrees(latLonRad [1]));
    	locTarget.setAltitude(locSrc.getAltitude());
    }
    
    /** Calculate target coordinates from source coordinates and bearing and distance (radians)
     * 
     * @param latlonSrc    in radians
     * @param distanceRad  in radians
     * @param bearingRad   in radians
     * @param latLonRad    in radians
     */
    public static void distBear2Latlon (double [] latlonSrc, double distanceRad, double bearingRad, 
    		double [] latLonRad){
    	distBear2Latlon (latlonSrc [0], latlonSrc[1], distanceRad, bearingRad, latLonRad);
    }
    
    /** Calculate target coordinates from source coordinates and bearing and distance (radians), spheric earth 
     * 
     * @param latRad
     * @param lonRad
     * @param distanceRad
     * @param bearingRad
     * @param latLonRad
     */
    public static void distBear2Latlon (double latRad, double lonRad, double distanceRad, double bearingRad, 
    		double [] latLonRad){
    	
    	//http://stackoverflow.com/questions/10119479/calculating-lat-and-long-from-bearing-and-distance
    		
    	if (distanceRad < 0) { 
    		distanceRad *=-1.0;      // Set Distance > 0
    		bearingRad  += Math.PI;  // Add 180�
    	}
    	while (bearingRad >= Math.PI*2.0) {
    		bearingRad -= Math.PI*2.0;
    	}
    	while (bearingRad < 0) {
    		bearingRad += Math.PI*2.0;
    	}

    	double lat2 = Math.asin( Math.sin(latRad)*Math.cos(distanceRad) + Math.cos(latRad)*Math.sin(distanceRad)*Math.cos(bearingRad) );
    	double a = Math.atan2(Math.sin(bearingRad)*Math.sin(distanceRad)*Math.cos(latRad), Math.cos(distanceRad)-Math.sin(latRad)*Math.sin(lat2));
    	double lon2 = lonRad + a;

    	lon2 = (lon2+ 3*Math.PI) % (2*Math.PI) - Math.PI;

    	latLonRad [0] = lat2;
    	latLonRad [1] = lon2;
    	
    	// from script http://www.zwanziger.de/gc_tools_projwp.html
    	// gives Error for negative latitudes!
    	
//    	  if (distanceRad < 0) { 
//    		  distanceRad *=-1.0;      // Set Distance > 0
//    		  bearingRad  += Math.PI;  // Add 180�
//    	  }
//	      while (bearingRad >= Math.PI*2.0) {
//	    	  bearingRad -= Math.PI*2.0;
//	      }
//	      while (bearingRad < 0) {
//	    	  bearingRad += Math.PI*2.0;
//	      }
//	      
          // double c = distanceRad;
//	      double a = latRad;
//	      if (latRad >=0) {a= Math.PI/2.0 - latRad;}     // 90�-Lat.
//	      double q = Math.PI*2.0 - bearingRad;
//	      double b = Math.acos(Math.cos(q) * Math.sin(a) * Math.sin(distanceRad) + Math.cos(a) * Math.cos(distanceRad));
//	      latLonRad [0] = Math.PI/2.0 - b;
//	      if (latLonRad [0] > Math.PI/2.0) {
//	    	  latLonRad [0] = Math.PI/2.0 -latLonRad [0]; //Suedhalbkugel
//	      }
//	      double g;
//	      if ((a + b) == 0) {
//	        g = 0; //Nenner unendlich
//	      } else {
//		    double cos_g =  (Math.cos(distanceRad) - Math.cos(a) * Math.cos(b)) / (Math.sin(a) * Math.sin(b));
//		    if (cos_g > 1 || cos_g < -1) {g = 0;}
//		    else g = Math.acos(cos_g);
////		    else g = Math.acos( (Math.cos(distanceRad) - Math.cos(a) * Math.cos(b)) / (Math.sin(a) * Math.sin(b)) );
//	      }
//	      if (bearingRad <= Math.PI) {
//	        g *= -1.0;
//	      }    	      
//	      latLonRad [1] = lonRad - g;
    }
    
    
    // evtl Class android.location.Geocoder verwenden? // liefert nur umgekehrten Weg!
    // example request:
    // http://maps.google.com/maps/geo?q=%27Berlin,%20Germany%27&output=kml
    // not supported any longer!!
    
/*    public static GeoCoordinate getCoordinateForAddress(String address) {
//    	List <GeoCoordinate> list = new ArrayList <GeoCoordinate> ();
//    	list = getCoordinateForAddressList (address);
//    	if (list.size() > 0) {return list.get(0);}
//    	else return new GeoCoordinate(GeoRecord.ERROR_LAT, 1, "ParserConfigurationException"); // Avoids System Crash
//    }
    
	requestRunning = true;
	String name;
    BufferedReader re = null;

    GeoCoordinate ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 1, "ParserConfigurationException"); // Avoids System Crash
    try {
        StringBuilder sb = new StringBuilder();
        sb.append(googleMapsAURL);
        sb.append("\'" +URLEncoder.encode(address.replace(' ', '+')) +"\'");
        sb.append(output);
        Log.d ("Request", sb.toString());
        URL u = new URL(sb.toString());
        re = new BufferedReader(new InputStreamReader(u.openStream()));
        sb = new StringBuilder();
        String currentLine;
        OutputStream os = new FileOutputStream("/sdcard/url.xml");  // add sdcard path
        OutputStreamWriter out = new OutputStreamWriter(os, "UTF-8");
        while ((currentLine = re.readLine()) != null) {
            sb.append(currentLine);
            out.write(currentLine);
        }
        out.close();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        StringReader reader = new StringReader(sb.toString());
//        Log.d("Planetdroid", sb.toString());
        InputSource inputSource = new InputSource(reader);
        Document doc = db.parse(inputSource);
        reader.close();
        doc.getDocumentElement().normalize();
        NodeList nl = doc.getElementsByTagName("code");
        Element n = (Element) nl.item(0);
        String st = n.getFirstChild().getNodeValue();
        if (st.equals("200")) {
                    	
        	NodeList n2 = doc.getElementsByTagName("coordinates");
            Element nn = (Element) n2.item(0);
            StringTokenizer tok = new StringTokenizer(nn.getFirstChild().getNodeValue(), ",");
            double lon = Double.parseDouble(tok.nextToken());
            double lat = Double.parseDouble(tok.nextToken());
            ret = new GeoCoordinate(lat, lon, address);
            
            // parse Name
            name = ParseDoc(doc, "LocalityName");
            if (name != "") {ret = new GeoCoordinate(lat, lon, name);}
            String countryName = ParseDoc (doc, "CountryName");
            if (countryName != "") {ret.countryName = countryName;}
            requestRunning = false;
            return ret;
        } else {
        	requestRunning = false;
        	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 0, "IllegalArgumentException");            	
            throw new IllegalArgumentException();
        }
    } catch (ParserConfigurationException ex) {
    	requestRunning = false;
       	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 1, "ParserConfigurationException");
        return ret;
//        Logger.getLogger(CoordinateTool.class.getName()).log(Level.SEVERE, null, ex);
    } catch (IOException ex) {  // wenn kein Internet verf�gbar
    	requestRunning = false;
       	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 2, "IOException");
        return ret;
//    	throw new IllegalArgumentException("2");
    } catch (SAXException ex) {
    	requestRunning = false;
       	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 3, "SAXException");
        return ret;
//        Logger.getLogger(CoordinateTool.class.getName()).log(Level.SEVERE, null, ex);
    } finally {
    	requestRunning = false;
        try {
            re.close();
            return ret;
        } finally {
            return ret;
        	}
    }
}

*/

    // will not work any longer, see RequestGeoCoder for better substitution!
/*    
    public static  List <GeoCoordinate> getCoordinateForAddressList (String address) {
    	requestRunning = true;
    	String name;
        BufferedReader re = null;
    	List <GeoCoordinate> list = new ArrayList <GeoCoordinate> ();
    	list = getCoordinateForAddressList (address);

        GeoCoordinate ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 1, "ParserConfigurationException"); // Avoids System Crash
        try {
            StringBuilder sb = new StringBuilder();
            sb.append(googleMapsAURL);
            sb.append("\'" +URLEncoder.encode(address.replace(' ', '+')) +"\'");
            sb.append(output);
            Log.d ("Request", sb.toString());
            URL u = new URL(sb.toString());
            re = new BufferedReader(new InputStreamReader(u.openStream()));
            sb = new StringBuilder();
            String currentLine;
            OutputStream os = new FileOutputStream("/sdcard/url.xml");  // add sdcard path
            OutputStreamWriter out = new OutputStreamWriter(os, "UTF-8");
            while ((currentLine = re.readLine()) != null) {
                sb.append(currentLine);
                out.write(currentLine);
            }
            out.close();
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            StringReader reader = new StringReader(sb.toString());
            InputSource inputSource = new InputSource(reader);
            Document doc = db.parse(inputSource);
            reader.close();
            doc.getDocumentElement().normalize();
            NodeList nl = doc.getElementsByTagName("code");
            Element n = (Element) nl.item(0);
            String st = n.getFirstChild().getNodeValue();
            if (st.equals("200")) {
                        	
            	NodeList n2 = doc.getElementsByTagName("coordinates");
                Element nn = (Element) n2.item(0);
                StringTokenizer tok = new StringTokenizer(nn.getFirstChild().getNodeValue(), ",");
                double lon = Double.parseDouble(tok.nextToken());
                double lat = Double.parseDouble(tok.nextToken());
                ret = new GeoCoordinate(lat, lon, address);
                
                // parse Name
                name = ParseDoc(doc, "LocalityName");
                if (name != "") {ret = new GeoCoordinate(lat, lon, name);}
                String countryName = ParseDoc (doc, "CountryName");
                if (countryName != "") {ret.countryName = countryName;}
                list.add(ret);
                requestRunning = false;                   
            } else {
            	requestRunning = false;
            	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 0, "IllegalArgumentException");            	
                throw new IllegalArgumentException();
            }
        } catch (ParserConfigurationException ex) {
        	requestRunning = false;
           	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 1, "ParserConfigurationException");
           	list.add(ret);
        	return list;
//            Logger.getLogger(CoordinateTool.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {  // wenn kein Internet verf�gbar
        	requestRunning = false;
           	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 2, "IOException");
           	list.add(ret);
        	return list;
//        	throw new IllegalArgumentException("2");
        } catch (SAXException ex) {
        	requestRunning = false;
           	ret = new GeoCoordinate(GeoRecord.ERROR_LAT, 3, "SAXException");
           	list.add(ret);
        	return list;
//            Logger.getLogger(CoordinateTool.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
        	requestRunning = false;
            try {
                re.close();
            	return list;
            } finally {
            	return list;
            	}
        }
    }
*/
    /** returns Address from Google, Caution: ANR is possible!
     * 
     * @param lat
     * @param lon
     * @return Address String
     */
    public static String getAddressForCoordinate(double lat, double lon) {
    	GeoRecord gR = new GeoRecord (lat, lon);
    	getGeoRecordForCoordinate (gR);
    	if (! gR.address.equals("")) {return gR.address;}
    	else return "";
    }
    
    /**  returns Address from Google, Caution: ANR is possible!
     * 
     * @param location
     * @return  Address String
     */
    public static String getAddressForCoordinate(Location location) {
    	try{
    		return getAddressForCoordinate (location.getLatitude(), location.getLongitude());
    	} catch (NullPointerException e) {
    		return "no location";
    	}
    }

    /**	returns most exact city or subcity name
     * 
     * @param lat
     * @param lon
     * @return
     */
    public static String getCityForCoordinate(double lat, double lon) {
    	GeoRecord gR = new GeoRecord (lat, lon);
    	getGeoRecordForCoordinate (gR);  //caution: ANR possible
    	String result;
    	if (! gR.dependentLocalityName.equals ("")) {result = gR.dependentLocalityName;}
    	else if (! gR.localityName.equals ("")) {result = gR.localityName;}
    	else if (! gR.subAdministrativeAreaName.equals ("")) {result = gR.subAdministrativeAreaName;}
    	else if (! gR.administrativeAreaName.equals ("")) {result = gR.administrativeAreaName;}
    	else result = gR.countryName;
    	return result;
    }
    
    /**	returns most exact city or subcity name
     * 
     * @param location
     * @return
     */
    public static String getCityForCoordinate(Location location) {
    	return getCityForCoordinate (location.getLatitude(), location.getLongitude());
    }


    /** returns city, country
     * 
     * @param location
     * @param defaultResult
     * @return
     */
    public static String getLocalityForCoordinate(Location location, String defaultResult) {
    	return getLocalityForCoordinate (location.getLatitude(), location.getLongitude(), defaultResult);
    }
        
    /** returns city, country
     * 
     * @param lat
     * @param lon
     * @param defaultResult
     * @return
     */
    public static String getLocalityForCoordinate(double lat, double lon, String defaultResult) {
    	GeoRecord gR = new GeoRecord (lat, lon);
    	getGeoRecordForCoordinate (gR);
    	String result = defaultResult;
    	if (! gR.dependentLocalityName.equals ("")) {result = gR.dependentLocalityName;}
    	else if (! gR.localityName.equals ("")) {result = gR.localityName;}
    	else if (! gR.subAdministrativeAreaName.equals ("")) {result = gR.subAdministrativeAreaName;}
    	else if (! gR.administrativeAreaName.equals ("")) {result = gR.administrativeAreaName;}
    	if (result.equals(defaultResult))
    	{result =  gR.countryName;}
    	else result = result + ", " + gR.countryName;
    	return result;
    }
    
    /** gets from coordinate full GeoRecord
     * 
     * @param gR
     */
    public static void getGeoRecordForCoordinate(Location location, GeoRecord gR) {
    	gR.lat = location.getLatitude();
    	gR.lon = location.getLongitude();
    	getGeoRecordForCoordinate (gR);
    }
    
    /** gets from coordinate full GeoRecord. Caution: ANR possible!
     * 
     * @param gR:  gR.lat, gR.lon are input coordinates
     */
    public static void getGeoRecordForCoordinate(GeoRecord gR) {

        BufferedReader re = null;
        try {
        	requestRunning = true;
            StringBuilder sb = new StringBuilder();
            sb.append(googleMapsCURL);
            sb.append(String.format (Locale.US, "%.5f,%.5f", gR.lat, gR.lon));
            sb.append(output);
            Log.d ("getAddressForCoordinate", "Request: " +sb.toString());
            URL u = new URL(sb.toString()); 
            re = new BufferedReader(new InputStreamReader(u.openStream()));
            sb = new StringBuilder();
            String currentLine;
            OutputStream os = new FileOutputStream("/sdcard/url.xml");  // add sdcard path
            OutputStreamWriter out = new OutputStreamWriter(os, "UTF-8");
            while ((currentLine = re.readLine()) != null) {
                sb.append(currentLine);
                out.write(currentLine);
            }
            out.close();
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            StringReader reader = new StringReader(sb.toString());
            InputSource inputSource = new InputSource(reader);
            Document doc = db.parse(inputSource);
            reader.close();
            doc.getDocumentElement().normalize();
            NodeList nl = doc.getElementsByTagName("code");
            Element n = (Element) nl.item(0);
            gR.resultCode = n.getFirstChild().getNodeValue();
            Log.d ("getAddressForCoordinate", "  Result Code: " +gR.resultCode);
            requestRunning = false;
            if (gR.resultCode.equals("200")) {
            	gR.address                  = ParseDoc (doc, "address");
            	gR.localityName 			= ParseDoc (doc, "LocalityName");
            	gR.dependentLocalityName 	= ParseDoc (doc, "DependentLocalityName");
            	gR.thoroughFareName 		= ParseDoc (doc, "ThoroughFareName");
            	gR.administrativeAreaName 	= ParseDoc (doc, "AdministrativeAreaName");
            	gR.subAdministrativeAreaName= ParseDoc (doc, "SubAdministrativeAreaName");
            	gR.countryNameCode 			= ParseDoc (doc, "CountryNameCode");
            	gR.countryName   			= ParseDoc (doc, "CountryName");           	
            }
            else Log.d ("getAddressForCoordinate", "  Error: not resultCode 200");
        } catch (ParserConfigurationException ex) {
            Log.d ("getAddressForCoordinate", "  ParserConfigurationException!");
        } catch (IOException ex) {  // wenn kein Internet verf�gbar
            Log.d ("getAddressForCoordinate", "  IOException!");
        } catch (SAXException ex) {
            Log.d ("getAddressForCoordinate", "  SAXException!");
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
        	requestRunning = false;
        	try {
        		re.close();
        	} catch (IOException e) {
        		Log.d ("getAddressForCoordinate", "  IOException!");
        		e.printStackTrace();
        	}
        	catch (NullPointerException e) {
        		Log.d ("getAddressForCoordinate", "  NullPointerException!");
        		e.printStackTrace();
        	}
        }
    }
    
    

    /** 
     * 
     * @param lat radians
     * @param lon radians
     * @param alt meters
     * @param ellipsoidA km
     * @param ellipsoidB km
     * @param xyz km
     */
    public static void ellipsoid_XYZ (double lat, double lon, double alt, 
    		double ellipsoidA, double ellipsoidB,
    		double [] xyz) {
    	//  Winkel Bogenma�; L�ngenma� egal, empfohlen wird Meter
    	double eq, n;

    	alt /= 1000.0;
    	eq =(ellipsoidA*ellipsoidA-ellipsoidB*ellipsoidB)/(ellipsoidA*ellipsoidA);
    	n = ellipsoidA/ Math.sqrt(1.0-eq*Math.sin(lat)*Math.sin(lat));

    	xyz[0] =(n+alt)*Math.cos(lat)*Math.cos(lon);
    	xyz[1] =(n+alt)*Math.cos(lat)*Math.sin(lon);
    	xyz[2] =((1.0-eq)*n +alt)*Math.sin(lat);
    }   // Ellipsoid_XYZ

    /**
     * 
     * @param lat  lat radians 
     * @param lon  lon radians 
     * @param alt  alt meters  
     * @param xyz  km
     */
    public static void wgs84_XYZ (double lat, double lon, double alt, 
    		double [] xyz) {
    	ellipsoid_XYZ (lat, lon, alt, ERDAEQ, ERDPOL, xyz);
    }

    
    
    /** Gets coordinates without Location name. May be null or network position, if no GPS available
     * 
     * @param locationManager
     * @return
     */
    public static Location getLocation (LocationManager locationManager) {	
//            Criteria criteria = new Criteria ();
//            criteria.setAccuracy(Criteria.ACCURACY_FINE); 
//            Location location = null;
//            // search the best Provider. May be network, if no GPS available
//            try {
//            	String provider = locationManager.getBestProvider(criteria, true);
//            	if (provider == null) {provider = "network";}
//            	location = locationManager.getLastKnownLocation(provider);
//           	} catch (RuntimeException e) { 
//           		e.printStackTrace();
//           	}  		
//     		return location;
    	
		Location location = null;
		try {
			locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
			if (Build.VERSION.SDK_INT >= 8) try {
				if (location == null)
					location = locationManager.getLastKnownLocation(LocationManager.PASSIVE_PROVIDER);
			} catch (Exception e) {
			}
			if (location == null)
				location = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);  //ANR here
		} catch (Exception e) {}
    	 return location;
     }  

    /** Gets coordinates without Location name. May be network position, if no GPS available
     * 
     * @param locationManager
     * @return
     */
    public static GeoCoordinate getCurrentCoordinates (LocationManager locationManager) {
    	return getCurrentCoordinates (locationManager, new GeoCoordinate());
    }

    	
    @SuppressWarnings("finally")
    /** Gets coordinates without Location name. May be network position, if no GPS available
     * 
     * @param locationManager
     * @param defaultLoc
     * @return
     */
	public static GeoCoordinate getCurrentCoordinates (LocationManager locationManager, GeoCoordinate defaultLoc) {	
  	   //GeoCoordinate defaultLoc = new GeoCoordinate();
     	try {    		
            Criteria criteria = new Criteria ();
            criteria.setAccuracy(Criteria.ACCURACY_FINE); 
            // search the best Provider. May be network, if no GPS available
            Location location = null;
            try{
                String provider = locationManager.getBestProvider(criteria, true);
                if (provider != null) {
                	location = locationManager.getLastKnownLocation(provider);
                }            
//     		Location location = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
     		if (location != null) {
     			defaultLoc = new GeoCoordinate (location);
//     			defaultLoc.name = location.getProvider();
     		}
            } catch (RuntimeException e) { }
     		return defaultLoc;
     	}
         finally { return defaultLoc;} 
     }  

    
    
    /** Gets coordinates and Location name, use better RequestGeoCoder or use in separate thread:
     *  Better:
     *  	geoCoordinate = GeoUtils.getCurrentCoordinates(locationManager);
     *  	RequestGeoCoder rgc = new RequestGeoCoder (this, geoCoordinate, locDisplayHandler);
     *  
     * @param locationManager
     * @return
     */
    public static GeoCoordinate getCurrentLocation (LocationManager locationManager) {	
    	return getCurrentLocation  (locationManager, new GeoCoordinate());
    }
    	
    /** Gets coordinates and Location name, use better RequestGeoCoder or use in separate thread:
     *  Better:
     *  	geoCoordinate = GeoUtils.getCurrentCoordinates(locationManager, geoCoordinate);
     *  	RequestGeoCoder rgc = new RequestGeoCoder (this, geoCoordinate, locDisplayHandler);
     *
     */
    public static GeoCoordinate getCurrentLocation(LocationManager locationManager, GeoCoordinate defaultLoc) {	
//  	   LocationManager locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    	//GeoCoordinate defaultLoc;
    	if (locationManager != null){
    		defaultLoc = getCurrentCoordinates (locationManager, defaultLoc);
    		if (defaultLoc.getLat() > GeoRecord.ERROR_LAT) {defaultLoc.name= GeoUtils.getCityForCoordinate (defaultLoc.getLat(), defaultLoc.getLon());}
    	}
    	else defaultLoc = new GeoCoordinate(GeoRecord.ERROR_LAT, GeoRecord.ERROR_LAT);
    	return defaultLoc; 
    }  
    
    public static String makeCoordString (GeoCoordinate loc, String dmsForm) {
    	String s = "----   ----";
    	try {
    		if (loc != null 
    				&& loc.getLat () > GeoRecord.ERROR_LAT  &&  loc.getLon () > GeoRecord.ERROR_LAT) {
    			s = AstroUtils.DMSStrg (loc.getLatRadians(), dmsForm)
    				+"  " +AstroUtils.DMSStrg (loc.getLonRadians(), dmsForm);
    		}
    	} catch (NullPointerException e) {
    		e.printStackTrace();
    	}
    	return s;

    }

    public static String makeLocString (GeoCoordinate loc, String dmsForm, boolean lineBreak){
        String s = "";
    	try{
    		if (loc.name != null && ! loc.name.equals("") && ! loc.name.equals("void")) s = loc.name + ", ";
    		if (lineBreak) s = s+ "\n";
    		s += makeCoordString (loc, dmsForm);
    	} catch (NullPointerException e) {
    		e.printStackTrace();
    		s = s + "----   ----";
    	}
    	return s;
    }

    public static boolean save2GPX (String fileName, Location location, String wptName) {
    	return save2GPX (fileName, location, wptName, "strickling.utils.GeoUtils", "", 0);
    }
    
    
    /**
     * 
     * @param fileName
     * @param location
     * @param wptName 
     * @param creator eg getPackageName()
     * @return
     */
    
    public static boolean save2GPX (String fileName, Location location, String wptName, String creator) {
    	return save2GPX (fileName, location, wptName, creator, "", 0);
    }

    /** 
         * 
         * @param fileName
         * @param location
         * @param wptName 
         * @param creator eg getPackageName()
         * @param description  optional description
         * @return
         */
        
   public static boolean save2GPX (String fileName, Location location, String wptName, String creator, String description, int gpsSatellites) {
	   return addGPX (fileName, location, wptName, false, true, creator, description, gpsSatellites);
   }
   
   public static boolean addGPX (String fileName, Location location, String wptName, boolean isTrack, boolean rewrite, String creator, String description, int gpsSatellites) {
    	boolean result = true;
		File file = new File(fileName);
		boolean fileExists = file.exists();
    	FileWriter f = null;
    	try {
    		f = new FileWriter(file, ! rewrite); // evtl UTF8 statt ANSI?, Wichtig bei Sonderzeichen!

    		//		f.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"); //Standalone refuses Geosetter reading
    		if (rewrite || !fileExists) {
    			f.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\r\n");
    			f.write("<gpx version=\"1.0\" creator=\"" + creator +"\">\r\n");
    			if (isTrack) {
    				f.write("<trk>\r\n");
    				// Add name to Track
    				double [] t = new double [6];
    				DateTimeUtils.jd2YMDHMS (DateTimeUtils.millis2JulDat(location.getTime()), 0, 0, t);
    				f.write(String.format(Locale.US, "    <name>Track%02.0f-%02.0f-%02.0f_%02.0f:%02.0f</name>\r\n",   
    	    				t [0], t [1], t [2], t [3], t [4], t [5]));
    				//    			    	f.write("<name><![CDATA[TrackExtract]]></name>\r\n");
    				//	    	f.write("<desc><![CDATA[]]></desc>\r\n");
    				//	    	f.write("<number>23</number>\r\n");
    				f.write(" <trkseg>\r\n");
    				lastGPXTime = location.getTime();
    			}
    		} 
    		if (isTrack) {
    			if (location.getTime() -lastGPXTime > newTrkSegTimeout) {
    				// start new segment
    				f.write(" </trkseg>\r\n");
    				f.write(" <trkseg>\r\n");
    			}

    			
    			f.write ("  <trkpt");} else f.write ("  <wpt"); 
    			f.write(String.format(Locale.US, "  lat=\"%.7f\" lon=\"%.7f\">\r\n",
    				location.getLatitude(), location.getLongitude()));

    		//		SimpleDateFormat formatter = new SimpleDateFormat ("yyyy-MM-ddTHH:mm:ssZ");
    		//		Date currentTime = new Date(location.getTime()); //crashes
    		//		f.write(String.format("    <time>%s</time>\r\n", formatter.format(currentTime)));
    		double [] t = new double [6];
    		DateTimeUtils.jd2YMDHMS (DateTimeUtils.millis2JulDat(location.getTime()), 0, 0, t);
    		f.write(String.format(Locale.US, "    <time>%02.0f-%02.0f-%02.0fT%02.0f:%02.0f:%02.0fZ</time>\r\n",   
    				t [0], t [1], t [2], t [3], t [4], t [5]));
    		if (! wptName.equals(""))     {f.write(String.format("    <name>%s</name>\r\n", wptName));}
    		else {f.write(String.format("    <name>%s</name>\r\n", SystemUtils.removeFileExt(SystemUtils.stripFileName(fileName))));}

    		if (! description.equals("")) {f.write(String.format("    <desc>%s</desc>\r\n", description));}
    		// getAltitude should give altitude ASL, but really gives ellipsoid alt. Correct value for this is <geoidheight>, but will not be recognized by tracking softwares, <ele> is better, but refers to ASL alt.
    		if (location.hasAltitude()) f.write(String.format(Locale.US, "    <ele>%.1f</ele>\r\n", location.getAltitude()));
    		if (location.hasAccuracy()) f.write(String.format(Locale.US, "    <hdop>%.0f</hdop>\r\n", location.getAccuracy()));
    		if (location.hasSpeed()) 	f.write(String.format(Locale.US, "    <speed>%.1f</speed>\r\n", location.getSpeed()));
    		if (location.hasBearing()) 	f.write(String.format(Locale.US, "    <course>%.0f</course>\r\n", location.getBearing()));
    		if (gpsSatellites > 0 || location.getProvider().equals(LocationManager.GPS_PROVIDER)) 
    			f.write(String.format(Locale.US, "    <sat>%1d</sat>\r\n", gpsSatellites));
    			
    		if (isTrack) {
    			f.write ("  </trkpt>\r\n");
    		} else { 
    			f.write("  </wpt>\r\n"); 
    			f.write("</gpx>\r\n");
    		}
        	lastGPXTime = location.getTime();
    	} catch (IOException e) {
    		e.printStackTrace();
    		result=false;
    	}

    	try {
    		f.close();
    	} catch (IllegalStateException e) {
    		result=false;
    		e.printStackTrace();
    	} catch (IOException e) {
    		result=false;
    		e.printStackTrace();
    	} catch (NullPointerException e) {
    		result=false;
    		e.printStackTrace();
    	}
    	return result;
    }
   
   public static void initGPXTrack (String fileName, String creator, boolean onlyTrekseg) {
	   FileWriter f = null;
	   File file = new File(fileName);
	   boolean fileExists = file.exists();
	   try {
		   f = new FileWriter(file, true); // evtl UTF8 statt ANSI?, Wichtig bei Sonderzeichen!
		   if (! fileExists) {
			   f.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\r\n");
			   f.write("<gpx version=\"1.0\" creator=\"" +creator +"\">\r\n");
		   } 
		   if (! fileExists || !onlyTrekseg) f.write("\r\n<trk>\r\n");
		   f.write("\r\n<trkseg>\r\n");
	   } catch (IOException e) {
		   e.printStackTrace();
	   }
	   try {
		   f.close();
	   } catch (Exception e) { e.printStackTrace(); }
   }
   
   
   /** a </trk> and </gpx> tags
    * 
    * @param fileName
    * @param isTrack
    */
   public static void finalizeGPX (String fileName,  boolean isTrack) {
	   FileWriter f = null;
	   File file = new File(fileName);
	   try {
		   f = new FileWriter(file, true); // evtl UTF8 statt ANSI?, Wichtig bei Sonderzeichen!
		   if (isTrack) {
			   f.write("</trkseg>\r\n");
			   f.write("</trk>\r\n");
		   } 
		   f.write("</gpx>\r\n");
	   } catch (IOException e) {
		   e.printStackTrace();
	   }

	   try {
		   f.close();
	   } catch (Exception e) { e.printStackTrace(); }
   }
   
   /** Removes </trk> and </gpx> tags
    * @param fileName
    */
   public static boolean removeGpxFooter (String fileName, boolean trk, boolean trkseg) {
	   String data = "";
	   RandomAccessFile file;
	   boolean result = false;
	   try {
		   file = new RandomAccessFile(fileName, "rw");
		   long position = file.length()-40;
		   if (position < 0) position = 0;
		   file.seek(position);
		   while (file.getFilePointer() <= file.length()-1) {
			   long lastFP = file.getFilePointer();
			   String dataRead = file.readLine();
			   data += dataRead;
			   if (file.getFilePointer() -lastFP >= dataRead.length() +2) data+= "?"; 
			   if (file.getFilePointer() -lastFP >= dataRead.length() +1) data+= "!"; 
		   }
//		   Log.d("GeoUtils", "DATA: " +data +" *** Data End ***");			    
		   long footerStart;

		   if (trkseg) {
			   footerStart = position + data.toLowerCase().indexOf("</trkseg>");			   
			   result = data.toLowerCase().indexOf("</trkseg>") > 0;
//			   Log.d("GeoUtils", "Remove /trkseg, pos: " +data.toLowerCase().indexOf("</trkseg>") );			    
		   } else if (trk) {
			   footerStart = position + data.toLowerCase().indexOf("</trk>");			   
			   result = data.toLowerCase().indexOf("</trk>") > 0;
//			   Log.d("GeoUtils", "Remove /trk, pos: " +data.toLowerCase().indexOf("</trk>") );			    
		   } else {
			   footerStart = position + data.toLowerCase().indexOf("</gpx>");			  
//			   Log.d("GeoUtils", "Remove /gpx, pos: " +data.toLowerCase().indexOf("</gpx") );		    
			   result = data.toLowerCase().indexOf("</gpx>") > 0;
		   }
		   if (footerStart > position) {
			   file.setLength(footerStart);
		   } else result = false;
		   file.close();
		   SystemUtils.appendString("<!-- RemoveFooter trakseg: " +trkseg +", trk: "  +trk +", result: " + result +" -->\r\n", fileName); //TODO
	   } catch (Exception e) {
		   e.printStackTrace();
		   result = false;
	   }
	   return result;
   }
   
	/**	returns String like "-5� 45' 10" or "-5.1234�" from degree angle, 
	 * 
	 * @param angle in degrees
	 * @param format eg. "dms0", "dm1", "d4" (last number is decimals) 
	 * @return String
	 */
	public static String pos2String (double lat, double lon, String format){
		return AstroUtils.DMSStrgDeg (lat, format) 
				+"  " +AstroUtils.DMSStrgDeg (lon, format);
	}
	
	/**	returns String like "-5� 45' 10" or "-5.1234�" from degree angle, 
	 * 
	 * @param angle in degrees
	 * @param format eg. "dms0", "dm1", "d4" (last number is decimals) 
	 * @return String
	 */
	public static String pos2String (Location loc, String format){
		return AstroUtils.DMSStrgDeg (loc.getLatitude(), format) 
				+"  " +AstroUtils.DMSStrgDeg (loc.getLongitude(), format);
	}
	


}
