package strickling.utils;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.util.ArrayList;
import org.osmdroid.api.IGeoPoint;
import org.osmdroid.compatibility.Crosshair;
import org.osmdroid.compatibility.FlexibleScaleBar;
import org.osmdroid.tileprovider.constants.OpenStreetMapTileProviderConstants;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapView;
import org.osmdroid.views.overlay.ScaleBarOverlay;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.util.Log;

public class OsmTools {
	
	final public static int ORANGE = Color.rgb(255, 160, 0);
	
	
	/**  Google map view

org.osmdroid.views.MapView mapView = (org.osmdroid.views.MapView) findViewById(R.id.map_view); //resolve the map view by id given in the layout
mapView.setTileSource(new OnlineTileSourceBase("Google Maps", ResourceProxy.string.unknown, 1, 20, 256, ".png", "http://mt3.google.com/vt/v=w2.97") {	
	@Override
	public String getTileURLString(final MapTile aTile) {
		/*
		 * GOOGLE MAPS URL looks like
		 * 	base url	    const	x	y    zoom
		 * http://mt3.google.com/vt/v=w2.97&x=74327&y=50500&z=17
		 * /
		return getBaseUrl() + "&x=" + aTile.getX() + "&y=" + aTile.getY() + "&z=" + aTile.getZoomLevel();
	}
});
mapView.setUseDataConnection(false); //this actually makes the controller use only offline tiles

Read more: http://datamoil.blogspot.com/2011/05/offline-google-maps-on-osmdroid.html#ixzz3tYA5Wam5

	 
	 */
	
	public static final String	CACHECONFIG			= OpenStreetMapTileProviderConstants.getBasePath() +"/CacheConfig.txt";	

	public static String TAG = "OsmDroid";
	
	public static boolean readCacheParams (){ 
		ArrayList <String> config = new ArrayList <String> ();
		boolean result = false;
		if (SystemUtils.fileExists (CACHECONFIG)){			
			result = SystemUtils.readText(config, CACHECONFIG);
		}
		
		if  (result) {
			long  MAX_CACHE_SIZE  = OpenStreetMapTileProviderConstants.TILE_MAX_CACHE_SIZE_BYTES/(1024*1024);
			long  TRIM_CACHE_SIZE = OpenStreetMapTileProviderConstants.TILE_TRIM_CACHE_SIZE_BYTES/(1024*1024);
			for  (int i = 0; i < config.size(); i++){
				if  (config.get(i).contains("TILE_MAX_CACHE_SIZE_BYTES")) MAX_CACHE_SIZE = (long) AstroUtils.firstVal(config.get(i));
				if  (config.get(i).contains("TILE_TRIM_CACHE_SIZE_BYTES")) TRIM_CACHE_SIZE = (long) AstroUtils.firstVal(config.get(i));
			}
			if (MAX_CACHE_SIZE <= 0)  MAX_CACHE_SIZE  = OpenStreetMapTileProviderConstants.TILE_MAX_CACHE_SIZE_BYTES/(1024*1024);
			if (TRIM_CACHE_SIZE <= 0) TRIM_CACHE_SIZE = OpenStreetMapTileProviderConstants.TILE_TRIM_CACHE_SIZE_BYTES/(1024*1024);
			if (TRIM_CACHE_SIZE > MAX_CACHE_SIZE) TRIM_CACHE_SIZE = (long) (MAX_CACHE_SIZE * 0.9);
			OpenStreetMapTileProviderConstants.setCacheSizes(MAX_CACHE_SIZE, TRIM_CACHE_SIZE);	
			Log.d(TAG, "Set Cache to max " +MAX_CACHE_SIZE + "MB, trim at: " +TRIM_CACHE_SIZE +"MB");
		}
//		Log.d(TAG, "setCacheParams Result: " +result +"\n" +org.apache.commons.lang3.StringUtils.join(config)); 
		return result;
	}
	
	public static boolean writeCacheParams (){ 
		String config = "TILE_MAX_CACHE_SIZE_BYTES = " +OpenStreetMapTileProviderConstants.TILE_MAX_CACHE_SIZE_BYTES/(1024*1024)
					+"\nTILE_TRIM_CACHE_SIZE_BYTES = " +OpenStreetMapTileProviderConstants.TILE_TRIM_CACHE_SIZE_BYTES/(1024*1024);
		boolean result = SystemUtils.saveString(config, CACHECONFIG);
		Log.d(TAG, "setCacheParams Result: " +result +"\n" +config);
		return result;
	}
	
	/** sets max cache to value, writes cache params to file */
	public static void setMaxCache (long cacheMB) {
		OpenStreetMapTileProviderConstants.setCacheSizes(cacheMB, (long) (cacheMB*0.009)*100);
		writeCacheParams ();
	}

	/** returns max  cache in MB */
	public static long getMaxCache () {
		return OpenStreetMapTileProviderConstants.TILE_MAX_CACHE_SIZE_BYTES/(1024*1024);
	}

	
//	public static void bottomScaleBar (final MapView mapView){
//       	mScaleBarOverlay = new ScaleBarOverlay(mapView);
//       	mScaleBarOverlay.setAlignBottom(true);
//    	mScaleBarOverlay.setMinZoom(1);
//    	mScaleBarOverlay.setEnableAdjustLength(true);
//    	mapView.getOverlays().add(mScaleBarOverlay);
//	}
//	
//	public static void addCrosshair (final MapView mapView, int length){
//		Crosshair crosshair = new Crosshair(mapView, 50);
//    	mapView.getOverlays().add(crosshair);
//	}
	
	public static Location makeLoc (GeoPoint mapPos) {
		Location loc = new Location ("Map");
		loc.setLatitude (mapPos.getLatitude());
		loc.setLongitude(mapPos.getLongitude());
		loc.setTime(System.currentTimeMillis ());
		return loc;
	}

	public static GeoPoint makeMapPos (Location loc) {
		return new GeoPoint(loc.getLatitude(), loc.getLongitude());
	}

	
	public static Bitmap dotBitmap (Context context, int diameter, int color){
		Bitmap bmp = Bitmap.createBitmap(diameter, diameter, Bitmap.Config.ARGB_8888);
		final Canvas canvas = new Canvas(bmp);
		Paint paint = new Paint ();		
		paint.setColor(color);
//		paint.setStrokeWidth(10.0f);
		paint.setStyle(Paint.Style.FILL_AND_STROKE);
		paint.setAntiAlias(true);
		canvas.drawCircle(diameter/2f, diameter/2f, diameter/2f, paint);
		return bmp;
	}
	
	public static Drawable dotIcon (Context context, int diameter, int color){
		return new BitmapDrawable(context.getResources(), dotBitmap (context, diameter, color));
	}

	/**	returns String like "-5 deg 45' 10" or "-5.1234 deg" from degree angle,
	 * 
	 * @param angle in degrees
	 * @param format eg. "dms0", "dm1", "d4" (last number is decimals)  Local_Circumstances.COORD_FORM[EclipseApp.dmsMode]
	 * @return String
	 */
	public static String pos2String (GeoPoint pos, String format){
		return GeoUtils.pos2String (pos.getLatitude(), pos.getLongitude(), format); 
	}
	
	public static String pos2String (GeoPoint pos){
		return pos2String (pos, "d5");
	}

	public static String pos2String (IGeoPoint pos){
		return GeoUtils.pos2String (pos.getLatitude(), pos.getLongitude(), "d5");
	}

	public static Location pos2Location (IGeoPoint pos){
		Location l = new Location ("OSM");
		l.setLatitude(pos.getLatitude());
		l.setLongitude(pos.getLongitude());
		return l;
	}



}
