package strickling.utils;

/**
 * class needing no resources. For makePermissionsOverwiev import strickling.forms.PermissionCheckerX;
 */

/** requires
 *  minSdkVersion 9
 *
 *  in dependencies
 *  implementation 'com.google.android.gms:play-services-base:8.+'  // required for request permissione, minSDK 9!
 */

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Build;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.util.Log;


import java.util.ArrayList;
import java.util.List;

public class PermissionChecker {

    public static String TAG = "PermissionChecker";

/**
    String[] permissions= new String[]{
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.CAMERA,
            Manifest.permission.ACCESS_COARSE_LOCATION,
            Manifest.permission.ACCESS_FINE_LOCATION};
*/

    public static final int WRITE_REQUEST  = 10;
    public static final int GPS_REQUEST    = 11;  // gives same result as COARSE_REQUEST
    public static final int COARSE_REQUEST = 13;  // gives same result as GPS_REQUEST
    public static final int LOCATION_REQUEST = GPS_REQUEST;
    public static final int CAMERA_REQUEST = 14;

    public static final int MULTIPLE_PERMISSIONS = 100;


    // checks without asking
    public static boolean hasPermission (Context context, int requestCode) {
        switch (requestCode) {
            case WRITE_REQUEST:  return hasPermission (context, Manifest.permission.WRITE_EXTERNAL_STORAGE);
            case GPS_REQUEST:    return hasPermission (context, Manifest.permission.ACCESS_FINE_LOCATION);
            case COARSE_REQUEST: return hasPermission (context, Manifest.permission.ACCESS_COARSE_LOCATION);
            case CAMERA_REQUEST: return hasPermission (context, Manifest.permission.CAMERA);
        }
        return false;
    }

    public static boolean hasPermission (Context context, String permission) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
//            Log.d (TAG, "hasPermission "  +permission +(activity.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED));
            return ContextCompat.checkSelfPermission(context, permission) == PackageManager.PERMISSION_GRANTED;
        }   else return true;
    }


    public static boolean hasWritePermission (Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
//            Log.d (TAG, "hasPermission "  +permission +(activity.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED));
            return ContextCompat.checkSelfPermission(context, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED;
        }   else return true;
    }


    public static boolean hasGpsPermission (Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
//            Log.d (TAG, "hasPermission "  +permission +(activity.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED));
            return ContextCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED;
        }   else return true;
    }

    public static boolean hasCameraPermission (Context context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
//            Log.d (TAG, "hasPermission "  +permission +(activity.checkSelfPermission(permission) == PackageManager.PERMISSION_GRANTED));
            return ContextCompat.checkSelfPermission(context, Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED;
        }   else return true;
    }



    /** do not ask several other permissions (e.g. askWritePermission + askGPSPermission at the same time (all in onCreate))!
     *  only first request will be handled!
     *  use this method
     *
     * @param activity
     * @param permissionList
     * @return
     */
    public static boolean askMultiplePermissions(Activity activity, String[] permissionList) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            int result;
            List<String> listPermissionsNeeded = new ArrayList<>();
            for (String per : permissionList) {
                result = ContextCompat.checkSelfPermission(activity, per);
                if (result != PackageManager.PERMISSION_GRANTED) {
                    listPermissionsNeeded.add(per);
                    Log.d   (TAG, "askMultiplePermissions Permission denied:  " + per);
                } else Log.d(TAG, "askMultiplePermissions Permission granted: " + per);
            }
            if (!listPermissionsNeeded.isEmpty()) {
                Log.d (TAG, "listPermissionsNeeded: " +listPermissionsNeeded.toString());
                ActivityCompat.requestPermissions(activity, listPermissionsNeeded.toArray(new String[listPermissionsNeeded.size()]), MULTIPLE_PERMISSIONS);
                return false;
            }
            return true;
        }
        return true;
    }

    public static boolean askPermission (Activity activity, String permission, int requestCode) {
        return askPermission (activity, permission, requestCode, true);
    }

    public static boolean askPermission(Activity activity, String permission, int requestCode, boolean forceRequest) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (ContextCompat.checkSelfPermission(activity, permission) == PackageManager.PERMISSION_GRANTED) {
                Log.v(TAG, "Permission " + permission + " is granted, forceRequest: " + forceRequest);
                if (forceRequest)
                    ActivityCompat.requestPermissions(activity, new String[]{permission}, requestCode);
                return true;
            } else {
                ActivityCompat.requestPermissions(activity, new String[]{permission}, requestCode);
                return false;
            }
        } else return true;
    }


    /** check and ask for external write permission. Call BEFORE  checkSD()! Otherwise deny without asking
     *
     * @param activity
     * @return
     */
    public static boolean askWritePermission (Activity activity, boolean forceRequest) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (activity.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED) {
                Log.d(TAG, "askWritePermission: Write Permission is granted");
                if (forceRequest) ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, WRITE_REQUEST);
                //File write logic here
                return true;
            } else {
                ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, WRITE_REQUEST);
                return false;
            }
        } else return true;
    }


    public static boolean askGpsPermission (Activity activity, boolean forceRequest) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (activity.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
                Log.d(TAG, " ACCESS_FINE_LOCATION Permission is granted");
                if (forceRequest) ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, GPS_REQUEST);
                //File write logic here
                return true;
            } else {
                Log.d(TAG, " ACCESS_FINE_LOCATION Permission not granted, ask for permission");
                ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, GPS_REQUEST);
                return false;
            }
        } else return true;
    }

    public static boolean askCoarsePermission (Activity activity, boolean forceRequest) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (activity.checkSelfPermission(Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
                Log.d(TAG, " ACCESS_COARSE_LOCATION Permission is granted");
                if (forceRequest)  ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, COARSE_REQUEST);
                //File write logic here
                return true;
            } else {
                ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, COARSE_REQUEST);
                return false;
            }
        } else return true;
    }


    public static boolean askCameraPermission (Activity activity, boolean forceRequest) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (activity.checkSelfPermission(Manifest.permission.CAMERA) == PackageManager.PERMISSION_GRANTED) {
                Log.d(TAG, " CAMERA Permission is granted");
                if (forceRequest)  ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.CAMERA}, CAMERA_REQUEST);
                //File write logic here
                return true;
            } else {
                ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.CAMERA}, CAMERA_REQUEST);
                return false;
            }
        } else return true;
    }
}
