package strickling.utils;

/* Orbital Elements from the internet: 
 * see SelectPlanetoid on  package com.planetdroid
 */


/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 


import android.util.Log;
import com.mhuss.AstroLib.Astro;
import com.mhuss.AstroLib.AstroOps;
import com.mhuss.AstroLib.LocationElements;
import com.mhuss.AstroLib.MathOps;
import com.mhuss.AstroLib.Nutation;
import com.mhuss.AstroLib.ObsInfo;
import com.mhuss.AstroLib.PlanetData;
import com.mhuss.AstroLib.Planets;
import com.mhuss.AstroLib.Vsop;
import java.util.Locale;

public class PlanetoidData extends PlanetData {
	
	public static final String TAG = "PlanetoidData";
	public OrbElem orbElem = null;
    /**
     * internal error code of kepler class.
     * @since 1.0
     */
    public int keplerErrCode = 0; // error code comes from kepler.
    /**
     * the max iteration, if convergence probelm in kepler.
     * @since 1.0
     */
    public int kepler_convergence_problem_iteration = 0; 

//    public String planetoid_name; // the name of the planetoid.
//    private ArrayList planetoid_data; // the data from database of the planetoid.
    
    double Mjd0, mjdPerihel;

    //	Time.Mjd(year, month, (new Double(day)).intValue(), 0, 0, 0.0)
    //        + (day - (new Double(day)).intValue()); // Epoch (yyyy mm dd.d)
    //double mjd = jd_TDT -2400000.5;

    // read planetoid elements
    double meanAnomaly;
    double a;   // semimajor axis
    double q=0; // Perihelion distance q in AU
    double e; // Excentricity e
    double i; // Inclination i in [rad]
    double Omega; // Longitude of ascending node in [rad]
    double omega; // Argument of perihelion in [rad]
    double T_eqx0; // Equinox of elements


	/**
     * The method <CODE>getPlanetoidName</CODE> returns the name of the planetoid.
     * @return name of planetoid.
     * @since 1.0
     */
    public String getPlanetoidName() {
        return orbElem.name;// planetoid_name;
    }

    /**
     * The method <CODE>getPlanetoidData</CODE> returns the data of the planetoids database entry.
     * @return data as list.
     * @since 1.0
     */
//    public ArrayList getPlanetoidData() {
//        return planetoid_data;
//    }

    /**
     * Default constructor. No valid initialization
     */
     public PlanetoidData () { m_planet = Planets.NAP; m_initComplete=false; }

     
     public PlanetoidData (OrbElem planetoid) {
      	this.orbElem = planetoid;   
      	//initElements (planetoid); 
      	m_initComplete=false;
   	  }

     /** Constructor for Comets and Asteroids
      * 
      * @param orbElem
      * @param jd
      * @param deltaT
      * @param oi
      */
     public PlanetoidData (OrbElem orbElem, double jd, double deltaT, ObsInfo oi ) {
//    	 Log.d(TAG, "Calc 1 PLANETOID: " +Planets.PLANETOID +", with nutation");
//    	 this.orbElem = orbElem.reduceElements(jd); 
    	 this.orbElem = orbElem; 
    	 calc (Planets.PLANETOID, this.orbElem, jd, deltaT, oi, true, true, true);
   	  }
      
     /** Constructor for full prec.
      * 
      * @param planet
      * @param orbElem
      * @param jd
      * @param deltaT
      * @param oi
      */
     public PlanetoidData (int planet, OrbElem orbElem, double jd, double deltaT, ObsInfo oi, boolean nut_aberr_Corr) {
//    	 Log.d(TAG, "Calc 2 planet: " +planet +", with nutation");
//    	 this.orbElem = orbElem.reduceElements(jd); 
    	 this.orbElem = orbElem; 
    	 calc (planet, this.orbElem, jd, deltaT, oi, true, true, nut_aberr_Corr);
	  }
     
     /** Constructor only for Planets and moon, full prec.
      * 
      * @param planet  MUST not be a planetoid
      * @param jd
      * @param oi
      */
     public PlanetoidData (int planet, double jd, ObsInfo oi ) {
		 //    	 Log.d(TAG, "Calc 3 planet: " +planet +", with nutation");
    	 calc (planet, null, jd, 0.0, oi, false, false, false);
	  }
      
     
     /**
      * This calculates all LocationElements, then returns the longitude.
      * <BR>
      * Longitude is split out because it gets used most often.
      *
      * @param planet Planet number (from <TT>Planets</TT> class)
      * @param jd Julian day number
      * @param oi Observer location
      */
      public double calcLon( int planet, double jd, ObsInfo oi ) {
    	  if ( Planets.PLANETOID != planet ) {return super.calcLon(planet, jd, oi);}
    	  else {
    		    m_jd = jd;
    		    m_planet = planet;
    		    m_centuries = AstroOps.toMillenia( jd );
        	    double [] lbr = new double [3];
        		EphemUtils.AsteroidLBR (jd , 
        				orbElem,
        				lbr);        		
        	    m_polarLEs = new LocationElements();
        	    m_polarLEs.setLongitude(lbr[0]);
        	    m_polarLEs.setLatitude (lbr[1]);
        	    m_polarLEs.setRadius   (lbr[2]);
        	    return m_polarLEs.getLongitude();
    	  }
      }


/*
     public void calcXYZ (double jd_TDT, double [] XYZ)
    {
    	m_planet = Planets.PLANETOID;
    	
        LinMath linmath = new LinMath();
        // double T_eqx = (Year - 2000.0) / 100.0;
        double T_eqx = (jd_TDT-DateTimeUtils.JD2000)/36525.0; // Auf Aequinoktium d. Datums setzen
        double Date = DateTimeUtils.jd2MJD(jd_TDT);

        PrecNut precnut = new PrecNut();
        Kepler kepler = new Kepler();
        Mat3D PQR = kepler.GaussVec(Omega, i, omega);
        Mat3D PQRtemp = precnut.PrecMatrix_Ecl(T_eqx0, T_eqx);
        PQR = linmath.mMult(PQRtemp, PQR);


            // Heliozentrische ekliptikale Koordinaten des Koerpers, Aequinoktium T_eqx
        Vec3D r_helioc = new Vec3D(); // Ortsvektor
        Vec3D v_helioc = new Vec3D(); // Geschwindigkeit
        
        kepler.Kepler(Constant.GM_SUN, mjdPerihel, Date, q, e, PQR, r_helioc, v_helioc);
        if (kepler.errCode < 0) {
        	keplerErrCode = kepler.errCode;
        	kepler_convergence_problem_iteration = kepler.convergence_problem_iteration;
//        	return "Kepler errCode " +kepler.errCode;
        }
        else {
        	XYZ [0] = r_helioc.m_Vec [0];
        	XYZ [1] = r_helioc.m_Vec [1];
        	XYZ [2] = r_helioc.m_Vec [2];
        }
    }
    */

    
    public void calc (int planet, OrbElem planetoid, double jd, double deltaT, ObsInfo oi, boolean topoc_Corr, boolean lightT_Corr, boolean nut_aberr_Corr) {
		//if (q == 0) {
//			initElements (planetoid);
		//}
			calc (planet, jd, deltaT, oi, topoc_Corr, lightT_Corr, nut_aberr_Corr);
    }
    	
    /** Calculates , for Asteroids orbElem must be initialised before!
     * 
     */
    public void calc (int planet, double jd, double deltaT, ObsInfo oi, boolean topoc_Corr, boolean lightT_Corr, boolean nut_aberr_Corr) {
		if (planet == Planets.PLANETOID) {
    		planetoidCalc (jd, deltaT, oi, topoc_Corr, lightT_Corr, nut_aberr_Corr);
    	}
    	else super.calc (planet, jd, deltaT, oi, topoc_Corr, lightT_Corr, nut_aberr_Corr); // PlanetData.calc
    }
    
    /** Calculates Asteroids,  orbElem must be initialised before!
     * 
     * @param jd
     * @param deltaT
     * @param oi
     * @param topoc_Corr
     * @param lightT_Corr
     */
    public void planetoidCalc (double jd, double deltaT, ObsInfo oi, boolean topoc_Corr, boolean lightT_Corr, boolean nut_aberr_Corr) {

    	double localSiderealTime =
    		AstroOps.greenwichSiderealTime(jd-deltaT) + oi.getLongitudeRad(); // UTC required !
    	double obliquity = AstroOps.meanObliquity( AstroOps.toMillenia( jd ) );
		final double LIGHTSPEED   = 299792.458 *86400.0/AU;   // in AU/d

//        Log.d("PlanetData", String.format("planetoidCalc.calc  epochOfOsculation = %15.6f, PerL = %8.5f rad = %10.6f°",
//                orbElem.epochOfOsculation, orbElem.argumentOfPerihelion, Math.toDegrees(orbElem.argumentOfPerihelion)));


//		if (Math.abs(orbElem.equinoxOfElements -jd) > 100) {
//		if (! m_initComplete) {
//			Log.d (TAG, "Reduce elements: " + DateTimeUtils.YMDStrg (orbElem.equinoxOfElements, 0) +" -> "+ DateTimeUtils.YMDStrg (jd, 0));
//			orbElem = orbElem.reduceElements(jd);
//		}
//		Log.d (TAG, "planetoidCalc, PlanetData elements Equinox: " + DateTimeUtils.YMDStrg (orbElem.equinoxOfElements, 0) +String.format("%15.5f", orbElem.equinoxOfElements));

    	double tmpVec[]  = new double[MathOps.VECTOR_SIZE];  // buffers planet's helioc. xyz position, subject to transformations
    	double tmpVec2[] = new double[MathOps.VECTOR_SIZE];  // buffers Earth's xyz position
    	double lightTimeCorrection = a/LIGHTSPEED; // set start value semimajor axis
    	int passNo = 1;
    	if (! lightT_Corr) passNo = 2; // already last pass, if no light time correction required
    	do {
     		//calcLon( planet, jd -lightTimeCorrection, oi );  // Heliocentric planetary coordinates
    	    double [] lbr = new double [3];
//			Log.d ("PlanetData", "Pass " +passNo +",  LightTimeCorrection (min):"+ lightTimeCorrection *24.*60. );
    		EphemUtils.AsteroidLBR (jd -lightTimeCorrection, orbElem,
    				lbr); // Procedure will reduce from ecliptical coordinates from epoch of elements to epoch of date

    	    m_polarLEs = new LocationElements();
    	    m_polarLEs.setLongitude(lbr[0]);
    	    m_polarLEs.setLatitude (lbr[1]);
    	    m_polarLEs.setRadius   (lbr[2]);
//			if (passNo == 2) Log.d("PlanetData", String.format("planetoidData.planetoidCalcpassNo %d, heliozLon = %8.3f°,  lat = %8.3f°, r = %8.5f AE, ltc [min] = %5.2f, JD-ltc: = %15.5f",
//					passNo, Math.toDegrees(m_polarLEs.getLongitude()), Math.toDegrees(m_polarLEs.getLatitude()), m_polarLEs.getRadius(),
//					lightTimeCorrection *24.*60., jd-lightTimeCorrection));

    		AstroUtils.polarToCartesian3(lbr, tmpVec);

    		// conversion Heliozentric to geocentric position

    		// calculate earth's position and convert tmpVec to geocentric
    		g_polarLEs = new LocationElements(); // for buffering earth's position
    		
    		if (passNo ==1 || ! lightT_Corr) {  // Earth position does not change, calc only first time				  
    			// Calculate heliocentric coordinates of the Earth (g_polarLEs)
				m_centuries = AstroOps.toMillenia( jd); // -lightTimeCorrection);
				Vsop.calcAllLEs( g_polarLEs,  AstroOps.toMillenia(jd), Planets.EARTH ); // Values checked with Solex, Equinox of Day, JD = TDT = GMT - DeltaT
    			MathOps.polarToCartesian( tmpVec2,  g_polarLEs.getLongitude(),
    					g_polarLEs.getLatitude(), 
    					g_polarLEs.getRadius());

//				Log.d("PlanetData", String.format("planetoidData.planetoidCalc Sun  AeqDat: lSun %9.6f°  bSun %9.6f°  rSun %9.6f, TDT= %15.6f", Math.toDegrees(g_polarLEs.getLongitude()), -Math.toDegrees(g_polarLEs.getLatitude()), g_polarLEs.getRadius(), jd));
//				double [] sunKoord = new double [3];
//				sunKoord [0] =  g_polarLEs.getLongitude();  sunKoord [1] =  g_polarLEs.getLatitude();  sunKoord [2] =  g_polarLEs.getRadius();
//				AstroUtils.redEcl(sunKoord, tmpVec2, jd, Astro.J2000);
//				Log.d("PlanetData", String.format("planetoidData.planetoidCalc Sun  JD2000: lSun %9.6f°  bSun %9.6f°  rSun %9.6f, TDT= %15.6f", AstroUtils.realMod(Math.toDegrees(tmpVec2 [0]), 360), -Math.toDegrees(tmpVec2 [1]), tmpVec2 [2], jd));
	//			Log.d("PlanetData", String.format("planetoidData.planetoidCalc Sun: X %9.6f  Y %9.6f  Z %9.6f, TDT= %15.6f", tmpVec2 [0], tmpVec2 [1], tmpVec2 [2], jd));
    		}


    		// tmpVec  = Helicoentric planet position or geocentric lunar position
    		// tmpVec2 = Helicoentric position of the Earth

    		// Planets: Change tmpVec from heliocentric to geocentric
    		tmpVec [0] -= tmpVec2 [0];
    		tmpVec [1] -= tmpVec2 [1];
    		tmpVec [2] -= tmpVec2 [2];

    		// reduce geocentric equatorial position tmpVec cartesian coordinates to topocentric
    		// Topocentric reduction in passNo 2
    		if  (passNo == 2) { // only in second pass of for Moon and Sun
//				MathOps.cartesianToPolar(tmpVec, tmpVec2); // only for debugging of following Log.d
//				Log.d("PlanetData", String.format("planetoidData.planetoidCalc geoL %9.6f  geoB %9.6f  r %9.6f", Math.toDegrees(tmpVec2 [0]), Math.toDegrees(tmpVec2 [1]), tmpVec2 [2]));
//				Log.d(TAG, String.format("planetoidData.planetoidCalc X %9.6f  Y %9.6f  Z %9.6f", tmpVec [0], tmpVec [1], tmpVec [2]));
    			if (topoc_Corr) TopocReductionEcl (tmpVec, localSiderealTime, oi.getLatitudeRad(), obliquity);
    		}
    		// Calc lightTimeCorrection in days for passNo 2
    		else {  // passNo == 1
    			lightTimeCorrection = Math.sqrt(
    					tmpVec [0]*tmpVec [0]  +tmpVec [1]*tmpVec [1]  +tmpVec [2]*tmpVec [2])
    					/LIGHTSPEED; 
    		}

    		passNo ++;
    	} while (passNo <= 2);  // end of Light time correction passes

        if (nut_aberr_Corr) {
            Nutation nutation = new Nutation( AstroOps.toMillenia( jd ));
//          Log.d ("Nutation", String.format("Nutation: %.5f sec, %.5f sec ", nutation.getDPhi (), nutation.getDEpsilon()));
            obliquity += Math.toRadians(nutation.getDEpsilon()/3600.0);
//          Log.d ("Nutation", String.format("Coord before: %.6f, %.6f, %.6f ", tmpVec[0],  tmpVec[1],  tmpVec[2]));
            MathOps.rotateVector( tmpVec, Math.toRadians(nutation.getDPhi ()/3600.0), 2 );
//          Log.d ("Nutation", String.format("Coord after : %.6f, %.6f, %.6f ", tmpVec[0],  tmpVec[1],  tmpVec[2]));

            // apply stellar aberration
            if (/*Aberration*/ true) {
                MathOps.cartesianToPolar(tmpVec, tmpVec2); // buffering angular vector in tmpVec2
//              Log.d ("Nutation", String.format("Aberr Coord before : %.6f, %.6f", tmpVec2[0],  tmpVec2[1]));
                EphemUtils.Aberration(jd, tmpVec2);
//              Log.d ("Nutation", String.format("Aberr Coord after  : %.6f, %.6f", tmpVec2[0],  tmpVec2[1]));
                MathOps.polarToCartesian(tmpVec, tmpVec2);
            }
        }
        // end Strickling modification

    	m_eclipticLEs = new LocationElements(tmpVec);

    	// At this point,  eclipticLE is a unit vector in ecliptic
    	// coords of date.  Rotate it by 'obliquity' to get
    	// a vector in equatorial coords of date:
    	//
    	MathOps.rotateVector( tmpVec, obliquity, 0 );
    	//TopocReductionEqu (tmpVec, localSiderealTime, oi.getLatitudeRad());
    	m_equatorialLEs = new LocationElements(tmpVec);  // Cartesian!!

    	// extract RA/Dec from equatorial coords of date
    	//
    	m_rightAscension = Math.atan2(
    			m_equatorialLEs.getY(), m_equatorialLEs.getX() );
    	//    m_declination = Math.asin( m_equatorialLEs.getZ() );
    	m_declination = Math.atan2( m_equatorialLEs.getZ(), 
    			Math.sqrt(m_equatorialLEs.getX()* m_equatorialLEs.getX() +
    					m_equatorialLEs.getY()*m_equatorialLEs.getY())); // Correction by Strickling

    	// The following two rotations take us from a vector in
    	// equatorial coords of date to an alt/az vector:
    	//
    	MathOps.rotateVector( tmpVec, -localSiderealTime, 2 );

    	//Strickling Change Z To X and Y to -Y
    	m_hourAngle = Math.atan2( -tmpVec[LocationElements.Y], tmpVec[LocationElements.X] ); 

    	MathOps.rotateVector( tmpVec, oi.getLatitudeRad() - Astro.PI_OVER_TWO, 1);
    	m_altAzLEs = new LocationElements(tmpVec);  // Cartesian!!

    	m_initComplete=true;
    } //planetoidCalc

    
    
    
}



