package strickling.utils;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

/* 	GeoCoder request transferred into a child thread to prevent freezing display
 *  (Request may last several seconds!)
 *  or Application Not Responding (ANR) dialog
 *  
 * usage: 

	// Implement Callback handler:
    Handler locDisplayHandler = new Handler() {
		public void handleMessage(Message msg) {
			Address address = (Address) msg.obj;
			if (address != null){
			// TODO implement your code here, example:
			 
//				locText.setText(RequestGeoCoder.getAddress(address, "\n")); 
//				if (address.getSubLocality () != null) { // Appends SubLocality
//					locText.append(address.getLocality() +" - " +address.getSubLocality ()+"\n" );
//				}	    				
			}
		}
    };



		// Call geocoder Request    // gets Location for coordinates
     	RequestGeoCoder rgc = new RequestGeoCoder (
    			this, location, 
    			locDisplayHandler); 

		// Call geocoder Request    // gets coordinates for address 
     	RequestGeoCoder rgc = new RequestGeoCoder (
				this, address, handler)

		// Caution: Google blocks IP, if > 2500 requests /24h or request rate too high!
		// Alternative? reverse ceocoding:  http://api.geonames.org/findNearby?lat=51.74381&lng=7.17705&username=geo_wolfy2
		// Ceocoding http://api.geonames.org/search?&name=Haltern%20am%20See&username=geo_wolfy2
		//           important: use URLEncoder.encode(query, "UTF-8"); to encode umlauts  (eg. " " => %20
		// Timezone: http://api.geonames.org/timezone?lat=51.743427&lng=7.17728&username=geo_wolfy2
 */


import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Locale;
import java.util.Timer;

//import com.google.android.gms.maps.model.LatLng;




import android.content.Context;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class RequestGeoCoder implements Runnable {

	public static 		String TAG = "PlanetDroid";

    private Context 	gcContext;
    private double 		gcLat, gcLon;
    private String		gcString;
    private int			gcRequestID;  // is MSG_ADDRESS for Address Requests or MSG_COORDINATE for Coordinates
    private boolean		gcRequestTimeZone = false;  // takes long  time, do not use as default
      
    private Handler 	gcHandler;
    public static boolean requestRunning = false;
    public Thread 		gcThread;
    public Timer 		timer ;
    public static long t0 = 0; // Start time of a request in Millis, last request Time
    
    public static final long GOOGLE_RATE_MILLIS = 35000; // max 2500 Requests /Day, appr. 35s
    public static final long TIMEOUT_MILLIS 	= 10000;
    public static final int MSG_ADDRESS 		= 1;
    public static final int MSG_NO_ADDRESS 		= 2;
    public static final int MSG_GC_BUSY 		= 3;
    public static final int MSG_NULL 			= 4;
    public static final int MSG_GC_TIMEOUT 		= 5;
    public static final int MSG_COORDINATE 		= 11;
    public static final int MSG_ERROR 			= 12;
    public static final int MSG_TIMEZONE 		= 13;
     
    public static final String GEONAMES_URL		= " http://api.geonames.org/";
    public static       String geoNamesUser		= "&username=geo_wolfy2";

	// GeoCoder request transferred into a child thread to prevent freezing display 
    // (Request may last several seconds!)
	// or Application Not Responding (ANR) dialog
	// from http://wiki.aquasonic.ch/android/daten_in_einem_thread_laden	
    // see also http://www.developer.com/java/j2me/article.php/10934_3762056_2/Handling-Lengthy-Operations-in-Googles-Android.htm
	
	public RequestGeoCoder (Context context, Location location, Handler handler){
    	this.gcLat 			= location.getLatitude();
    	this.gcLon 			= location.getLongitude();
    	this.gcHandler 		= handler;
    	this.gcContext 		= context;
    	this.gcRequestID 	= MSG_ADDRESS;
		Thread thread 		= new Thread (this);
		thread.start();
	}
    
//	public RequestGeoCoder (Context context, LatLng latLng, Handler handler){
//    	this.gcLat = latLng.latitude;
//    	this.gcLon = latLng.longitude;
//    	this.gcHandler = handler;
//    	this.gcContext = context;
//		Thread thread = new Thread (this);
//		thread.start();
//	}
    
	/**
	 * 
	 * @param context
	 * @param latDeg
	 * @param lonDeg
	 * @param handler
	 */
    public RequestGeoCoder (Context context, double latDeg, double lonDeg, Handler handler){
    	this.gcLat = latDeg;
    	this.gcLon = lonDeg;
    	this.gcHandler = handler;
    	this.gcContext = context;
    	this.gcRequestID 	= MSG_ADDRESS;
		Thread thread = new Thread (this);
		thread.start();
    }
	
    
    /** Requests Address for GeoCoordinate gc
     * 
     * @param context
     * @param gc
     * @param handler
     */
	public RequestGeoCoder (Context context, GeoCoordinate gc, Handler handler){
    	this.gcLat = gc.getLat();
    	this.gcLon = gc.getLon();
    	this.gcHandler = handler;
    	this.gcContext = context;
    	this.gcRequestID 	= MSG_ADDRESS;
    	// Stopping threads: http://stackoverflow.com/questions/680180/where-to-stop-destroy-threads-in-android-service-class
		if (! requestRunning || System.currentTimeMillis() -t0 > TIMEOUT_MILLIS) {  // Prevent nested calls
			requestRunning = false;
			gcThread = new Thread (this);
			gcThread.start();						
		}
		else {
			Message m = new Message ();
			m.what = MSG_GC_BUSY;
			gcHandler.sendMessage(m);
		}
    }
	
	/** Requests a Coordinate for a seach string adrress
	 * 
	 * @param context
	 * @param address String to search for
	 * @param handler
	 * 
	 * returns to handler Msg.what = MSG_COORDINATE, Msg.obj = GeoCoordinate; or MSG_NO_ADDRESS or MSG_ERROR
	 */
    public RequestGeoCoder (Context context, String address, Handler handler){
    	this.gcString = address;
    	this.gcHandler = handler;
    	this.gcContext = context;
    	this.gcRequestID 	= MSG_COORDINATE;
		gcThread = new Thread (this);
		gcThread.start();						
    }
	/** Requests a Coordinate for a seach string adrress
	 * 
	 * @param context
	 * @param address String to search for
	 * @param requestTimeZone
	 * @param handler
	 * 
	 * returns to handler Msg.what = MSG_COORDINATE, Msg.obj = GeoCoordinate; or MSG_NO_ADDRESS or MSG_ERROR
	 */
    public RequestGeoCoder (Context context, String address, boolean requestTimeZone, Handler handler){
    	this.gcString = address;
    	this.gcHandler = handler;
    	this.gcRequestTimeZone = requestTimeZone;
    	this.gcContext = context;
    	this.gcRequestID 	= MSG_COORDINATE;
		gcThread = new Thread (this);
		gcThread.start();						
    }

    /** returns for example timeZone like "Europe/Berlin" 
     * 
     * @param location
     * @param handler
     */
	public RequestGeoCoder (Context context, Location location, Handler handler, int requestID){
    	this.gcLat 			= location.getLatitude();
    	this.gcLon 			= location.getLongitude();
    	this.gcHandler 		= handler;
    	this.gcContext 		= context;
    	this.gcRequestID 	= requestID;
		Thread thread 		= new Thread (this);
		thread.start();
	}

    
    /** requests Geocoder */
    public boolean requestGeocoderCoord (String gcString, GeoCoordinate coord, Context context) {
    	boolean result = false;
		try {
			List<Address> foundGeocode = null;
			try {
				foundGeocode = new Geocoder(context).getFromLocationName(gcString, 1); // List size may be 0!
				if (foundGeocode != null && foundGeocode.size() > 0) {
//					foundGeocode.get(0).getLatitude(); //getting latitude
//					foundGeocode.get(0).getLongitude();//getting longitude
					
		    		coord.setLat (foundGeocode.get(0).getLatitude());
		    		coord.setLon (foundGeocode.get(0).getLongitude());

//					coord = new GeoCoordinate (foundGeocode.get(0).getLatitude(),
//							foundGeocode.get(0).getLongitude(),
//							gcString);
					// parse Name
					String name = foundGeocode.get(0).getLocality();
					if (name != null && ! name.equals("")) {coord.name = name;}
					else coord.name = gcString;
					String countryName = foundGeocode.get(0).getCountryName(); 
					if (countryName != null && !countryName.equals("")) {coord.countryName = countryName;}					
					Log.d(TAG, "requestGeocoderCoord, found: " +foundGeocode.size());
			   		coord.isGeocoded = true;			   	 
					result = true;
				}  	else {
					Log.d(TAG, "requestGeocoderCoord: nothing found!");
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		} catch (Exception e){
			e.printStackTrace();  // service not available or error
		}
		return result;
    }
    
    /** parses a geonames coord request answer */
    public static boolean parseGeoNamesCoord   (String record, GeoCoordinate coord) {
    	try{
    		coord.setLat (XmlTokenizer.getItemDouble("lat", record));
    		coord.setLon( XmlTokenizer.getItemDouble("lng", record));

    		// parse Name
    		String name = XmlTokenizer.getItem("name", record);
    		if (name == null || name.equals("")) {name = XmlTokenizer.getItem("toponymName", record);}
    		if (name != null && ! name.equals("")) {coord.name = name;}
    		String countryName = XmlTokenizer.getItem("countryName", record);
    		if (countryName == null || countryName.equals(""))  {countryName = XmlTokenizer.getItem("countryCode", record);}
    		if (countryName != null && !countryName.equals("")) {coord.countryName = countryName;}
    		return true;
    	} catch (Exception e) { return false;}
    }
    
    /** gets coordinates from address using GeoNames.org */
    public static boolean requestGeoNamesCoord (String gcString, GeoCoordinate coord) {
    	// Ceocoding http://api.geonames.org/search?&name=Haltern%20am%20See&username=geo_wolfy2
		//           important: use URLEncoder.encode(query, "UTF-8"); to encode umlauts  (eg. " " => %20
    	String url = gcString;
    	boolean result = false;
    	try {
    		url = URLEncoder.encode(gcString, "UTF-8");
    	} catch (UnsupportedEncodingException e) {
    		e.printStackTrace();
    	}
    	url = GEONAMES_URL+"search?&name="+url+geoNamesUser;
    	String answer = SystemUtils.downloadFile(url);
    	int totalResultsCount = XmlTokenizer.getItemInt("totalResultsCount", answer);
    	Log.d (TAG, "Rq Geonames: " +url +" , cts: " +totalResultsCount);
    	if (totalResultsCount >= 1) {
    		XmlTokenizer xt  = new XmlTokenizer (answer, "geoname");
    		if (xt.countTokens() > 0){
    			String record = xt.nextToken(); 
    			result = parseGeoNamesCoord (record, coord);
//    			Log.d (TAG, "Record: +\n" +record);
//    			Log.d (TAG, "Parsing: " +result);
    			if (coord.name.equals("")) coord.name = gcString;
    			coord.isGeocoded = true;
    		}
    	}
    	return result;
    }
    
    /** returns timezoneId like "Europe/Berlin" */
    public static String requestGeoNamesTimeZone (Location loc) {
    	return requestGeoNamesTimeZone (loc.getLatitude(), loc.getLongitude());
    }
    
    /** returns timezoneId like "Europe/Berlin" */
    public static String requestGeoNamesTimeZone (double latDeg, double lonDeg) {
    	// http://api.geonames.org/timezone?lat=51.743427&lng=7.17728&username=geo_wolfy2
    	String url = String.format(Locale.US, GEONAMES_URL +"timezone?lat=%.7f&lng=%.7f" +geoNamesUser,
    			latDeg, lonDeg);
    	String answer = SystemUtils.downloadFile(url);
    	return XmlTokenizer.getItem("timezoneId", answer);
    }


    public void run() {
		t0 = System.currentTimeMillis();
		switch (gcRequestID) {
    	case MSG_ADDRESS: {  // Request address for coordinate

    		//** Request Address for coordinates
    		if (! requestRunning && Math.abs(gcLat) <= 90 && Math.abs(gcLon) <= 360) {  // Prevent nested calls and Crashes with wrong coordinates
    			List <Address> list = null;
    			try {
    				requestRunning = true;
    				Geocoder gc = new Geocoder (gcContext);
    				list = gc.getFromLocation( // ANR caused if request lasts more than 5 sec.
    						gcLat, gcLon, 1);
    				requestRunning = false;
    				if (list.size() > 0) {// gcAddress = list.get(0); // Address found
    					Message m = new Message ();
    					m.what = MSG_ADDRESS;
    					m.obj = list.get(0);
    					gcHandler.sendMessage(m);
    				}
    				else { // no address found
    					Message m = new Message ();
    					m.what = MSG_NO_ADDRESS;
    					gcHandler.sendMessage(m);
    				}

    			} catch (Exception e) {
    				requestRunning = false;
    				Message m = new Message ();
    				m.what = MSG_NULL;
    				gcHandler.sendMessage(m);
    				e.printStackTrace();
    			}
    			if (timer != null) {
    				timer.cancel();
    				timer.purge();
    				timer = null;   
    			} 
    			requestRunning = false;
    		} // if (! requestRunning)
    		else {
    			Message m = new Message ();
    			m.what = MSG_NO_ADDRESS;
    			gcHandler.sendMessage(m);
    		}
    		break;
    	} // case MSG_ADDRESS


    	case MSG_COORDINATE: { // ** Request coordinates for Address  
    		Message m 	= new Message ();
    		GeoCoordinate coord = new GeoCoordinate();

    		// first try requestGeocoder
    		boolean result = requestGeocoderCoord (gcString, coord, gcContext);
    		Log.d(TAG, "requestGeocoderCoord: " +result +" after " +(System.currentTimeMillis()-t0) +" ms");

    		// if no result try requestGeoNames
    		if (! result) result = requestGeoNamesCoord  (gcString, coord);
    		if (result) coord.isGeocoded = true;
      		Log.d(TAG, "requestGeoNamesCoord: " +result +" after " +(System.currentTimeMillis()-t0) +" ms");
//      		Log.d(TAG, "MSG_COORDINATE Result: "+result);
    		
    		// Check for errors
    		if (result && coord != null && coord.getLat()> -999) {
    			if (gcRequestTimeZone) {
    				coord.timeZoneStrg = requestGeoNamesTimeZone (coord.location);
    				Log.d(TAG, "found Timezone: " +coord.timeZoneStrg +" after " +(System.currentTimeMillis()-t0) +" ms");
    			}
    			m.what 		= MSG_COORDINATE;
    			m.obj 		= coord;
    			//					Log.d(TAG, "MSG_COORDINATE ok");
    		} else { 
    			if (coord.getLon() == 0) 
    			{m.what = MSG_NO_ADDRESS; 
    			//					Log.d(TAG, "MSG_COORDINATE: MSG_NO_ADDRESS");
    			}     // nothing found
    			else {m.what = MSG_ERROR;	   // service not available
    			//					Log.d(TAG, "MSG_COORDINATE: MSG_ERROR");
    			}
    		}

    		Log.d(TAG, "MSG: "+m.what);
    		gcHandler.sendMessage(m);
    		break;
    	} // case MSG_COORDINATE request
    	case MSG_TIMEZONE:
    		Message m 	= new Message ();
			m.what 		= MSG_TIMEZONE;
			m.obj 		= requestGeoNamesTimeZone (gcLat, gcLon); // String like "Europe/Berlin"
			gcHandler.sendMessage(m);
    		break;   		
    	}
    }

	
	/** Generates Address like "Bahnhofstrasse 48, 45721 Haltern"
	 * 
	 * @param address Address data structure
	 * @param lineSeparator
	 * @return String address like "Bahnhofstrasse 48, 45721 Haltern"
	 */
	public static String getAddress (Address address, String lineSeparator) {
		String s = "";
		for (int i = 0; i< address.getMaxAddressLineIndex(); i++) { // Appends address lines
			s += address.getAddressLine(i);
			if ( i < address.getMaxAddressLineIndex() -1) s+= lineSeparator;
		}
		return s;
	}
	
	
	// Returns most precise Locality name, eg. Sythen
	public static String getLocName (Address address){
		String s = address.getSubLocality();
		if (s == null) {s = address.getLocality();}
		if (s == null) {s = address.getSubAdminArea ();}
		if (s == null) {s = address.getAdminArea ();}
		if (s == null) {s = address.getCountryName ();}
		if (s == null) {s = "";}		
		return s;
	}
}
