package strickling.utils;

/* GeoidHeight internet Request, 
 * uses website Online geoid calculations using the GeoidEval utility 
 * http://geographiclib.sourceforge.net/cgi-bin/GeoidEval
 * 
 * Request example:
 * http://geographiclib.sourceforge.net/cgi-bin/GeoidEval?input=51.87++7.18
 *
 * Usage: 


	// TODO Call GeoidEval Request, perhaps limit requests to max 1/10 min
    if (System.currentTimeMillis() - RequestGeoidEval.lastCall > 600000) {
   	   RequestGeoidEval rg = new RequestGeoidEval (myLocation, geoidHandler);
   	   } 
   	
	
	// TODO  Implement Callback handler:
    Handler geoidHandler = new Handler() {
		public void handleMessage(Message msg) {
			if (msg.what == RequestGeoidEval.MSG_GEOIDHEIGHT) {
				// TODO implement your code here, example:
   				double height = msg.arg2/100.;  // arg2 returns heigth in cm
			}
		}
    };

	// TODO add in Manifest:
	<uses-permission android:name="android.permission.INTERNET" />



 * 
 */


/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Locale;

import android.location.Location;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

public class RequestGeoidEval implements Runnable {
	
    private Handler rgHandler;
    private double  rgLat, rgLon;
    public static long lastCall = 0;
    public  static final int MSG_GEOIDHEIGHT = 1;
    public  static final int MSG_GEOIDERROR  = 0;

    public RequestGeoidEval (Location location, Handler handler){
    	this.rgHandler = handler;
    	this.rgLat = location.getLatitude();
    	this.rgLon = location.getLongitude();
    	Thread thread = new Thread (this);
    	thread.start();		
    }

    public RequestGeoidEval (double lat, double lon, Handler handler){
    	this.rgHandler = handler;
    	this.rgLat = lat;
    	this.rgLon = lon;
    	Thread thread = new Thread (this);
    	thread.start();		
    }

    public void run() {
    	boolean requestError = true;
    	BufferedReader re = null;
    	try {
    		StringBuilder sb = new StringBuilder();
    		sb.append("http://geographiclib.sourceforge.net/cgi-bin/GeoidEval?input=" 
    				+String.format(Locale.US, "%.5f+%+.5f", rgLat, rgLon));
    		//Log.d ("HelloAndroid", sb.toString());
    		URL u = new URL(sb.toString());
    		lastCall = System.currentTimeMillis();
    		re = new BufferedReader(new InputStreamReader(u.openStream()));
    		String currentLine;
    		while ((currentLine = re.readLine()) != null) {
    			if (currentLine.contains(">EGM2008</a>")) { 
    				int p = currentLine.indexOf("</a>");
    				double geoidHeight = AstroUtils.firstVal(AstroUtils.stripHTML(currentLine.substring(p)));
    				Message m	= new Message ();
    				m.what 	  	= MSG_GEOIDHEIGHT;
    				m.arg1		= (int)  geoidHeight;        //Height in m as integer  
    				m.arg2		= (int) (geoidHeight*100.0); //Height in cm
    				m.obj 		= geoidHeight;
    				rgHandler.sendMessage(m);     
    				requestError = false;
    			}
    		}
    	} catch (IOException ex) {  // no Internet available
    	} finally {
    		if (requestError) {
    			Message m	= new Message ();
    			m.what 	  	= MSG_GEOIDERROR;
    			rgHandler.sendMessage(m);            		
    		}
    	}       
    }


}
