package strickling.utils;

/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.HttpURLConnection;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

//import org.apache.http.util.ByteArrayBuffer;
// not available in SKD >= 23; see https://stackoverflow.com/questions/32153318/httpclient-wont-import-in-android-studio

import android.Manifest;
import android.annotation.SuppressLint;
import android.app.ActionBar;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.ComponentName;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.res.Resources;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.BatteryManager;
import android.os.Build;
import android.os.Environment;
//import android.support.v4.app.ActivityCompat;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Display;
import android.view.Menu;
import android.view.View;
import android.view.WindowManager;
import android.view.WindowManager.BadTokenException;
import android.view.inputmethod.InputMethodManager;

public class SystemUtils {

	public static String TAG = "SystemUtils";
	/** "/.com.strickling/";*/
	private static String STRICKLING_DIR		 = "/.com.strickling/";
	protected static String STRICKLING_SD_SPACE = Environment.getExternalStorageDirectory().getAbsolutePath()+STRICKLING_DIR;//"/sdcard";
//	private static String DOCUMENTS_DIR		 = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS).getAbsolutePath() +STRICKLING_DIR;
//	private static String DOWNLOAD_DIR 		 = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS).getAbsolutePath() +STRICKLING_DIR;
	public static String writeableDir  	 	 = STRICKLING_SD_SPACE; // call checkSD, will set to private or STRICKLING_SD_SPACE
	public static boolean writeChecked		 = false;

	public static final int 			ACTION_BAR_COLOR = 0xFF293d53;
	public static final int 			ACTION_BAR_ASTRO_RED = 0xFF440808;

	public static void hideKeyboard (Activity activity) {
		InputMethodManager inputManager = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
		if (inputManager != null) 
			try {
				inputManager.hideSoftInputFromWindow(activity.getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
		} catch (Exception e) {}
		try {
			activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN); 		
		} catch (Exception e) {}
	}

	/** sets Focus to v and shows keyboard
	 * 
	 * @param activity
	 * @param  v (View)
	 */
	public static void showKeyboard (Activity activity, View v) {
		InputMethodManager inputManager = (InputMethodManager) activity.getSystemService(Context.INPUT_METHOD_SERVICE);
		if (inputManager != null) {
//			activity.getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_VISIBLE);
			v.requestFocus();
			inputManager.showSoftInput(v,  InputMethodManager.SHOW_IMPLICIT ); 	
		}
	}
	
	/** returns minimum side size of the screen
	 * 
	 * @param context
	 * @return
	 */
	public static int getMinScreenSize (Context context) {
		WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
		Display display = wm.getDefaultDisplay();
		DisplayMetrics metrics = new DisplayMetrics();
		display.getMetrics(metrics);
		return Math.min(metrics.widthPixels, metrics.heightPixels);
	}

	public static boolean isOrientationPortrait (Context context) {
		WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);
		Display display = wm.getDefaultDisplay();
		DisplayMetrics metrics = new DisplayMetrics();
		display.getMetrics(metrics);
		return (metrics.widthPixels < metrics.heightPixels);
	}
	
	/**
	 * 
	 * @param c ususally "this"
	 * @return value > 0
	 */
	public static double getDisplayDensity (Context c) {
		return c.getResources().getDisplayMetrics().density;
	}

	/** returns BatteryTemperature in deg C, see http://stackoverflow.com/questions/3997289/get-temperature-of-battery-on-android
	 * 
	 * @param context
	 * @return
	 */
    public static double getBatteryTemperature(Context context)
    {
        Intent intent = context.registerReceiver(null, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));    
        return intent.getIntExtra(BatteryManager.EXTRA_TEMPERATURE,0) / 10.;
    }



	public static boolean isIntentAvailable(Context context, String action) {
		// from   http://alt236.blogspot.de/2011/06/checking-if-intent-handler-or-activity.html
	    final PackageManager packageManager = context.getPackageManager();
	    final Intent intent = new Intent(action);
	    List resolveInfo =
	            packageManager.queryIntentActivities(intent,
	                    PackageManager.MATCH_DEFAULT_ONLY);
		return resolveInfo.size() > 0;
	}

	public static ActivityInfo findAppByName (String searchString, Context context) {
		PackageManager pm = context.getPackageManager();
		// see:   http://osdir.com/ml/AndroidDevelopers/2009-03/msg01921.html
		Intent mainIntent = new Intent(Intent.ACTION_MAIN, null);
		mainIntent.addCategory(Intent.CATEGORY_LAUNCHER);
		List<ResolveInfo> list = pm.queryIntentActivities (mainIntent, 0);

		int n = 0;
		int i = -1;

		// 1st pass: search exact label match
		do { i++;
		if (list.get(i).loadLabel(pm).equals(searchString)) { n++; }
		} while (i < list.size()-1 && n <= 0);

		// 2nd pass: search case insensitive label match
		if (n==0) {
			i = -1;
			do { i++;
			if (list.get(i).loadLabel(pm).toString().toLowerCase().equals(searchString.toLowerCase())) {
				n++; }
			}  while (i < list.size()-1 && n <= 0);
		}

		// 3nd pass: search case insensitive lass or package match
		if (n==0) {
			i = -1;
			do { i++;
			if (list.get(i).activityInfo.name.toLowerCase().contains(searchString.toLowerCase())) {
				//outputText.append(list.get(i).loadLabel(pm)+"\n");
				n++; }
			}  while (i < list.size()-1 && n <= 0);
		}

		if (n > 0) { return list.get(i).activityInfo; }
		else return null;
	}

	/**
	 * @param context
	 * @param px
	 * @return scaledPixels (for fonts, dependend on resolution and user's font size preferences)
	 */
	public static float pxToSp(Context context, Float px) {
		float scaledDensity = context.getResources().getDisplayMetrics().scaledDensity;
		return px/scaledDensity;
	}

	/**
	 * @param px
	 * @param context
	 * @return  dp or dip independent Pixel (dependend on resolution, for 160 dpi dip = pixels)
	 * https://stackoverflow.com/questions/2025282/what-is-the-difference-between-px-dip-dp-and-sp
	 */
	public static float pxToDp( float px, Context context ) {
		Resources resources = context.getResources();
		DisplayMetrics metrics = resources.getDisplayMetrics();
		float dp = px / ( metrics.densityDpi / 160f );
		return dp;
	}

	/**
	 * @param dp
	 * @param context
	 * @return  dp or dip independent Pixel (dependend on resolution, for 160 dpi dip = pixels)
	 * https://stackoverflow.com/questions/2025282/what-is-the-difference-between-px-dip-dp-and-sp
	 */
	public static int dpToPx( float dp, Context context ) {
		Resources resources = context.getResources();
		DisplayMetrics metrics = resources.getDisplayMetrics();
		int px = (int) (dp * ( metrics.densityDpi / 160));
		return px;
	}

	public static void configureActionBar (Activity activity) {
		configureActionBar (activity, ACTION_BAR_COLOR);
	}

		// Set blue actionBar with icon, replaces android:theme="@style/ThemeDialogWhenLarge" in manifest activity
	@SuppressLint("NewApi")
	public static void configureActionBar (Activity activity, int color) {
		try { 
			if (Build.VERSION.SDK_INT >= 11) {
				ActionBar actionBar= activity.getActionBar();
				actionBar.setBackgroundDrawable(new ColorDrawable(color)); 	// background color
//				Log.d (TAG, String.format ("ActionBarColor: %x", color));
				actionBar.setDisplayOptions (ActionBar.DISPLAY_SHOW_HOME | ActionBar.DISPLAY_SHOW_TITLE);
				
				// set App Logo as specified in Manifest, try first activity Logo, then activity Icon, then app logo, then app Icon
				Drawable d = activity.getPackageManager().getActivityLogo(activity.getComponentName());
				if (d == null) d = activity.getPackageManager().getActivityIcon(activity.getComponentName());
				if (d == null) d = activity.getPackageManager().getApplicationLogo(activity.getApplicationInfo());
				if (d == null) d = activity.getPackageManager().getApplicationIcon(activity.getApplicationInfo());
				if (d != null) activity.getActionBar().setIcon(d);
			}
        } catch (Exception e) {e.printStackTrace();}
 	}
	
	public static void launchApp (ActivityInfo ai, Context context) {
		// launch selected app
		Intent intent = new Intent();
		intent.setAction(Intent.ACTION_MAIN);
		intent.addCategory(Intent.CATEGORY_LAUNCHER);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK |
				Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);
		intent.setComponent(new ComponentName(
				ai.applicationInfo.packageName, 
				ai.name));
		context.startActivity(intent);
	}

	/** launches application, 
	 *  searches first for label (Case sensitive), then case insensitive, then for class (substring, case insensitive)
	 * 
	 * @param searchString
	 * @param context (usually "this")
	 */
	public static boolean launchApp (String searchString, Context context) {
		ActivityInfo ai = findAppByName (searchString, context);
		if (ai != null) { 
			launchApp (ai, context); 
			return true;}
		else return false;
	}

	public static void lauchBrowser (Context context, String url) {
		Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
		try {context.startActivity(intent);  } catch (Exception e) {e.printStackTrace();}   	// launch Browser
	}
	
	public static boolean bakFile (String fileName) {
		boolean result = false;
		try {
			File f1 = new File (fileName);
			File f2 = new File (fileName.substring(0, fileName.lastIndexOf('.')) +".bak");
			Log.d (TAG, "Rename to : " +fileName.substring(0, fileName.lastIndexOf('.')) +".bak");
			result = f1.renameTo(f2);
		} catch (Exception e) {e.printStackTrace();}
		return result;
	}

	/**  reads a textfile from URLString, inserts \n after every line
	 * best to run in a separate thread to prevent ANR
	 * caution: there will be some CrLfs stripped in slow data connections! (GSM)
	 */
	public static boolean downloadFile (String URLString, String fileName) {
		boolean result = false;
		try {
			// Create a URL for the desired page
			URL url = new URL(URLString);
			chkDir (fileName); // create Directory if necessary
			// Read all the text returned by the server
			BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
			OutputStream   os = new FileOutputStream(fileName);  // add sdcard path
			OutputStreamWriter out = new OutputStreamWriter(os, "UTF-8");
			String str;
			while ((str = in.readLine()) != null) {
				out.write(str+"\n");
			}
			out.close();
			in.close();
			result = true;
		} catch (MalformedURLException e) {e.printStackTrace();
		} catch (IOException e) {e.printStackTrace();
		}
		return result;    	
	}

	/** downloads a text file, inserts \r\n after every line
	 * 
	 * @param URLString
	 * @return
	 */
	public static String downloadFile (String URLString) {
		String result = "";
		try {
			// Create a URL for the desired page
			URL url = new URL(URLString);				
			BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
			String str;
			while ((str = in.readLine()) != null) {
				result += str +"\r\n";
			}
			in.close();
			//				OutputStream   os = new FileOutputStream("sdcard/test.htm");  //TODO add sdcard path
			//				OutputStreamWriter out = new OutputStreamWriter(os, "UTF-8");
			//				out.write(result);
			//				out.close();
		} catch (MalformedURLException e) {
			e.printStackTrace(); 
			Log.d(TAG, "MalformedURLException " +e.toString()); 
		} catch (IOException e) {
			e.printStackTrace(); 
			Log.d(TAG, "IOException " +e.toString());
		} catch (Exception e) {
			e.printStackTrace(); 
			Log.d(TAG, "Exception " +e.toString());
		}
		return result;    	
	}


	/** saves a file form downloadUrl to fileName, should be in separate Thread
	 * 
	 * @param downloadUrl
	 * @param fileName
	 */
	public static boolean downloadFromUrl(String downloadUrl, String fileName) {
		// see http://www.androidsnippets.com/download-an-http-file-to-sdcard-with-progress-notification
		boolean result = false; 
		try {
//			File root = android.os.Environment.getExternalStorageDirectory();               
//			File dir = new File (root.getAbsolutePath() + "/xmls");
			File dir = new File (stripFileDir(fileName));
			if(! dir.exists()) {
				dir.mkdirs();
			}

//			URL url = new URL(downloadUrl); //you can write here any link
//			File file = new File(fileName);

			long startTime = System.currentTimeMillis();
			Log.d(TAG, "### downloadFromUrl: download begining");
			Log.d(TAG, "download   from url:  " + downloadUrl);
			Log.d(TAG, "downloaded file name: " + fileName);

			/* Open a connection to that URL. */
//			URLConnection ucon = url.openConnection();
//			HttpURLConnection ucon = (HttpURLConnection) url.openConnection();
			String inString = "";
//			try {
				inString = downloadFile (downloadUrl);
//				InputStream in = new BufferedInputStream(ucon.getInputStream());
//				inString = readStream (in);
//			} finally {
//				ucon.disconnect();
//			}
			saveString (inString, fileName);

			/*
			 * Define InputStreams to read from the URLConnection.
			 */
//			InputStream is = ucon.getInputStream();
//			BufferedInputStream bis = new BufferedInputStream(is);

			/*
			 * Read bytes to the Buffer until there is nothing more to read(-1).
			 */
//			ByteArrayBuffer baf = new ByteArrayBuffer(5000);
//			int current = 0;
//			while ((current = bis.read()) != -1) {
//				baf.append((byte) current);
//			}


			/* Convert the Bytes read to a String. */
//			FileOutputStream fos = new FileOutputStream(file);
////			fos.write(baf.toByteArray());
//			fos.flush();
//			fos.close();


			result = true;
			Log.d(TAG, "download finished in" + ((System.currentTimeMillis() - startTime) / 1000) + " sec");

//		} catch (IOException e) {
//			Log.d(TAG, "downloadFromUrl: IOError: " + e);

		} catch (Exception e) {
			Log.d(TAG, "downloadFromUrl: Error: " + e);
		}
		return result;
	}

	private static String readStream(InputStream in) {
		BufferedReader reader = null;
		StringBuffer response = new StringBuffer();
		try {
			reader = new BufferedReader(new InputStreamReader(in));
			String line = "";
			while ((line = reader.readLine()) != null) {
				response.append(line);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return response.toString();
	}


	public static boolean fileExists (String fileName) {
		if (fileName == null) return false;
		File file = new File (fileName);
		return file.exists();
	}

	public static boolean deleteFile (String fileName) {
		if ((fileName == null) || (! fileExists(fileName))) return false;
		File file = new File (fileName);
		return file.delete();
	}



	/** gets file name without path
	 * 
	 * @param path
	 * @return
	 */
	public static String stripFileName (String path){
		return path.substring(path.lastIndexOf('/')+1);
	}

	/** gets path without file name
	 * 
	 * @param path
	 * @return
	 */
	public static String stripFileDir (String path){
		if (path != null && path.contains("/")) { return path.substring(0, path.lastIndexOf('/'));}
		else return "";
	}

	/** removes file extension
	 * 
	 * @param path
	 * @return
	 */
	public static String removeFileExt (String path){
		if (path.contains(".")) { return path.substring(0, path.lastIndexOf('.'));}
		else return path;
	}
	

	/** gets file extension without dot
	 * 
	 * @param FileName
	 * @return
	 */
	public static String getFileExt(String FileName)
	{       
		return FileName.substring((FileName.lastIndexOf(".") + 1), FileName.length());
	}



	/** creates directory of filename if necessary and checks existance
	 * 
	 * @param Filename
	 * @return
	 */
	public static boolean chkDir (String Filename){
		String dirS = stripFileDir(Filename);
		if (dirS != null && dirS.trim().length() > 0){
			File dir = new File (dirS);
			if (!dir.exists()) dir.mkdirs(); // Create Directory if necessary
			return dir.exists();
		} else return true;
	}

	/** saves String s to file fileName, if necessary creates directory 
	 * 
	 * @param s
	 * @param fileName
	 * @return
	 */
	public static boolean saveString (String s, String fileName){
		if (SystemUtils.chkDir(fileName)){
			try{
				File file = new File (fileName);
				FileOutputStream fOut = new FileOutputStream(file);
				OutputStreamWriter osw = new OutputStreamWriter(fOut);
				osw.write(s);
				osw.flush();
				osw.close();
				return true;
			} catch (Exception e){
				e.printStackTrace();
				return false;
			}
		} else return false;
	}
	
	/** read a text file and write to ArrayList <String> s
	 * convert to String: String str = org.apache.commons.lang3.StringUtils.join(s);
	 * 
	 * @param s
	 * @param fileName
	 * @return
	 */
	public static boolean readText (ArrayList <String> s, String fileName) {
		BufferedReader in;
		boolean result = false;
		try {
//			Log.d(TAG, "readFile: " +fileName); 
			// TODO   Error handling: File not found, CSV error & File Exist check
			in = new BufferedReader(new FileReader(fileName));
			s.clear();
			String reader = "";
			while ((reader = in.readLine()) != null){
				s.add (reader);
			}
			in.close();
			result = true;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			Log.d(TAG, "readFile: FileNotFound Exception: " +fileName);
		} catch (IOException e) {
			e.printStackTrace();
			Log.d(TAG, "readFile: IO Exception");
		}
		return result;
	}

	/** saves String s to file fileName, if necessary creates directory 
	 * 
	 * @param s
	 * @param fileName
	 * @return
	 */
	public static boolean appendString (String s, String fileName){
		if (chkDir(fileName)){
			try{
				File file = new File(fileName);
				FileWriter fw = new FileWriter(file, true);
				fw.write(s);
				fw.close();
				return true;
			}
			catch( IOException e )
			{
				e.printStackTrace();
				return false;
			}
		} else return false;
	}

	/** returns private directory, eg. "/data/data/my.package/files/"
	 * 
	 * @param c
	 * @return
	 */
	public static String getPrivateDir (Context c) {
		return c.getFilesDir().getPath().toString() +"/";
	}
	
	
	/* needs
	    <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE" />
	 * see  http://stackoverflow.com/questions/1560788/how-to-check-internet-access-on-android-inetaddress-never-timeouts   
	 */
	/*
	public boolean isOnline() {
	    ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
	    NetworkInfo netInfo = cm.getActiveNetworkInfo();
//	    if (netInfo != null && netInfo.isConnectedOrConnecting()) {
	    if (netInfo != null && netInfo.isConnected()) {
	        return true;
	    }
	    return false;
	}
	 */

	/** call getBuildMillis (getPackageManager().getApplicationInfo(getPackageName(), 0))
	 *  Date: 
	   		FormatterUtils.dateStrg(SystemUtils.getBuildMillis (getPackageManager().getApplicationInfo(getPackageName(), 0)), 
	      		FormatterUtils.YMDPattern, Locale.US); 
	 * 
	 * @param ai
	 * @return
	 */
	public static long getBuildMillis (ApplicationInfo ai) {
		long millis = 0;
		try{
			ZipFile zf = new ZipFile(ai.sourceDir);
			ZipEntry ze = zf.getEntry("classes.dex");
			millis = ze.getTime();
			zf.close();
//			if (millis < 1041400800000L) millis = BuildConfig.buildTime;
		}catch(Exception e){
			e.printStackTrace();
		}
		return millis;	
	}
	
	public static ProgressDialog progressDialog = null;
	
	/** call: SystemUtils.showHourglass (this, getString(R.string.wait));
	 *  if finished, call: SystemUtils.stopHourglass ();
	 * @param context
	 * @param message
	 */
	public static void showHourglass (Context context, String message){ 
		// Display hourglass
		if (progressDialog == null && context != null) {
			try {
				progressDialog = new ProgressDialog(context);  
				progressDialog.setMessage(message);
				progressDialog.show();
			} catch (BadTokenException e) { e.printStackTrace();}
			catch (Exception e) {e.printStackTrace();}
		}
	}
	
	public static void stopHourglass (){ 
		try { 
			if (progressDialog != null) progressDialog.dismiss();
			progressDialog = null;
		} catch (Exception e) { }
	}
	
	public static boolean isBeta (Context context) {
		try {
			PackageManager pm = context.getPackageManager();
			return pm.getPackageInfo(context.getPackageName(), 0).versionName.toLowerCase().contains("beta")
					|| pm.getPackageInfo(context.getPackageName(), 0).versionName.toLowerCase().contains("debug") ;
		} catch (Exception e) {} 
		return false;
	}

	/** make overflow Icons visible
	 * call: 
	@Override
    public boolean onMenuOpened(int featureId, Menu menu) {
        SystemUtils.makeMenuIconsVisible (menu);
        return super.onMenuOpened(featureId, menu);
    }
	 */
	public static void makeMenuIconsVisible (Menu menu) {
		// http://blog.webandapp-solution.de/how-to-show-icons-in-actionbar-overflow-menu/
        if(menu != null && menu.getClass().getSimpleName().equals("MenuBuilder")){ 
                try{
                    Method m = menu.getClass().getDeclaredMethod(
                        "setOptionalIconsVisible", Boolean.TYPE);
                    m.setAccessible(true);
                    m.invoke(menu, true);
                }
                catch(NoSuchMethodException e){
                        //write in the LogCat 
                    Log.d(TAG, "onMenuOpened", e);
                }
                catch(Exception e){
                	 Log.d(TAG, "onMenuOpened", e);
                    // throw new RuntimeException(e);
                }            
        }	
	}

	/** 	Check whether SD-card is writable, sets WRITEABLE_SD_SPACE either to STRICKLING_SD_SPACE or DOWNLOAD_DIR or to PrivateDir
	 *
 	 */

//	public boolean tryWrite (File fileDir, String filename) {
//		boolean result = false;
//		File file = new File(fileDir, filename);
//
//		// Try creating directory
//		File dir = new File (fileDir.getAbsolutePath());
//		dir.mkdirs(); // no exception?
//		try {
//			FileOutputStream f = new FileOutputStream(file);
//			PrintWriter pw = new PrintWriter(f);
//			pw.println("Testfile, delete this file.");
//			pw.flush();
//			pw.close();
//			f.close();
//			file.delete();
//			result = true;
//		} catch (FileNotFoundException e) {
//			e.printStackTrace();
//		} catch (IOException e) {
//			e.printStackTrace();
//		}
//		return result;
//	}

	public static boolean writeableIsVerified = false;

	public static boolean checkSD (Context context) {
		boolean result = false;

		// Public external
		String fn = STRICKLING_SD_SPACE + "test.txt";
		result = saveString("Testfile, delete this file.", fn);
//		Log.d(TAG, "StrSD:   " +result);
		if (result) {
			writeableDir = STRICKLING_SD_SPACE;
			Log.d (TAG, "checkSD Public external " +writeableDir +"  verified: " +result);
			SystemUtils.deleteFile(fn);
		} else {

			// Public external documents
			String DOWNLOAD_DIR = Environment.getExternalStorageDirectory().getAbsolutePath() + "/Documents" + STRICKLING_DIR;
			if (Build.VERSION.SDK_INT >= 19) {
				DOWNLOAD_DIR = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS).getAbsolutePath() + STRICKLING_DIR;
			}
			fn = DOWNLOAD_DIR + "test.txt";
			result = saveString("Testfile, delete this file.", fn);
//			Log.d(TAG, "fn =  " +DOWNLOAD_DIR +",  " +result);
			if (result) {
				writeableDir = DOWNLOAD_DIR;
				SystemUtils.deleteFile(fn);
				Log.d (TAG, "checkSD Public external documents " +writeableDir +"  verified: " +result);
			} else {

				// Private external
				String privateExternal = context.getExternalFilesDir("").getAbsolutePath() + "/";
				fn = privateExternal + "test.txt";
				result = saveString("Testfile, delete this file.", fn);
			    Log.d(TAG, "fn =  " +fn +"  write: " +result);
				if (result) {
					writeableDir = privateExternal;
					SystemUtils.deleteFile(fn);
					Log.d (TAG, "checkSD Private external " +writeableDir +"  verified: " +result);
				} else {

					// Private internal
					writeableDir = context.getFilesDir().getPath().toString() + "/";
					fn = writeableDir + "test.txt";
					result = saveString("Testfile, delete this file.", fn);
					if (result) SystemUtils.deleteFile(fn);
					Log.d (TAG, "checkSD Private internal " +writeableDir +"  verified: " +result);
				}
			}
		}
		writeChecked = true;
		writeableIsVerified = result;
//		Log.d(TAG, "SystemUtils.checkSD.writeableDir =  " + writeableDir +);
		return result;
	}


	public static String getWriteableDir (Context context) {
		if (! writeChecked) checkSD(context);
		return writeableDir;
	}

	public static boolean checkWriteableDir (String dir) {
		boolean result = false;
		if (!dir.endsWith("/")) dir += "/";
		dir += "test.txt";
		result = saveString("Testfile, delete this file.", dir);
		Log.d(TAG, "fn =  " + dir + "  write: " + result);
		if (result) SystemUtils.deleteFile(dir);
		return result;
	}

	public static boolean checkWriteableFile (String fileName) {
		return checkWriteableDir (stripFileDir(fileName));
	}

	/** method for getting double values lossless from settings
	 *
	 * @param settings
	 * @param key
	 * @param defaultValue
	 * @return
	 */
	public static double getPrefsDouble (SharedPreferences settings, String key, double defaultValue) {
		return Double.longBitsToDouble(settings.getLong (key,Double.doubleToRawLongBits(defaultValue)));
	}

	/** compatibility, reads double as well as float parameter to settings
	 *
	 * @param settings
	 * @param key    (double value in key +"Double", in "key" ist float value stored)
	 * @param defaultValue
	 * @return
	 */
	public static double getPrefsDoubleCompat (SharedPreferences settings, String key, double defaultValue) {
		if (settings.contains(key+"Double") || ! settings.contains(key)) {
			return Double.longBitsToDouble(settings.getLong (key +"Double",Double.doubleToRawLongBits(defaultValue)));
		}  else return (double) settings.getFloat(key, (float) defaultValue);
	}

	/** method for storing double values lossless in settings
	 *
	 * @param editor
	 * @param key
	 * @param dValue
	 */
	public static void putPrefsDouble (SharedPreferences.Editor editor, String key, double dValue) {
		editor.putLong (key, Double.doubleToRawLongBits(dValue));
	}

	/** compatibility, writes double as well as float parameter to settings
	 *
	 * @param editor
	 * @param key		Writes fload value into key, double value into key +"Double"
	 * @param dValue
	 */
	public static void putPrefsDoubleCompat (SharedPreferences.Editor editor, String key, double dValue) {
		editor.putLong (key +"Double", Double.doubleToRawLongBits(dValue));
		editor.putFloat(key, 		  (float) dValue);
	}


}
