package strickling.utils;


/*****************************************************************************\
 *
 *  Copyright 2019 Dr. Wolfgang Strickling
 *  published under  GNU GPL 3
 *
 *   This file is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 \*****************************************************************************/
 

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageInfo;
import android.content.SharedPreferences;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.widget.TextView;

public class WhatsNew {
	

/**   shows a WhatsNew message, 
 * 			if android:versionCode in Manifest is higher than last call (buffered in sharedPreferences
 * 			and last call lower than newSince
 * 			and trim(whatsNewText) is not empty.
 *    
 *    usage in onCreate: 	
 *    		WhatsNew.showWhatsNew ((Activity) this, R.string.whatsNewTitle, R.string.whatsNewText, 
    				PREFS_NAME, getResources().getInteger(R.integer.whatsNewSince)); 

 * @param context
 * @param whatsNewTitleID   // resource identifier
 * @param whatsNewStringID  // of empty then no message
 * @param prefsName  // sharedPreferences
 * @param newSince  // version code of last major update to show, must be lower than current version
 */
	public static void showWhatsNew (Activity context, int whatsNewTitleID, int whatsNewStringID, String prefsName, int newSince) {
	    showWhatsNew (context, whatsNewTitleID, context.getString (whatsNewStringID), prefsName, newSince);
	}


		public static void showWhatsNew (Activity context, int whatsNewTitleID,
		String whatsNewString, String prefsName, int newSince) {
		try {
			PackageInfo pInfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
			//		    String version = pInfo.versionName;
			int verCode = pInfo.versionCode;
			// TODO check here if (Build.VERSION.SDK_INT >= 11)
			SharedPreferences settings = context.getSharedPreferences (prefsName, Context.MODE_MULTI_PROCESS);

			if (verCode > settings.getInt("WhatsNewSince", 0) 			// update since last call?
					&& settings.getInt("WhatsNewSince", 0) < newSince   // do not show each bugfix, only major updates
					&& ! whatsNewString.trim().equals("")) {

				SpannableString msgText = new  SpannableString (whatsNewString);
				Linkify.addLinks(msgText, Linkify.WEB_URLS);
				TextView msgView = new TextView(context);
				msgView.setText(msgText);


				AlertDialog.Builder builder;
				builder = new AlertDialog.Builder(context);
				builder.setTitle(whatsNewTitleID)
				.setView(msgView)  //builder.setMessage(infoText)
				.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int which) {
					}
				})
				//		    .setIcon(android.R.drawable.ic_dialog_alert)
				.setCancelable(true)
				.show();

				msgView.setMovementMethod(LinkMovementMethod.getInstance());
				// save actual showed version in settings
				SharedPreferences.Editor editor = settings.edit();
				editor.putInt ("WhatsNewSince", verCode);
				editor.commit();
			}

		}
		catch (Exception e) {e.printStackTrace();
		}
    }


}
